@@ -1,7 +1,7 @@
 //=========================================================
 //  MusE
 //  Linux Music Editor
-//  $Id: appearance.cpp,v 1.11.2.1 2005/08/14 21:47:31 spamatica Exp $
+//  $Id: appearance.cpp,v 1.11.2.5 2009/11/14 03:37:48 terminator356 Exp $
 //=========================================================
 
 #include <stdio.h>
@@ -25,6 +25,7 @@
 #include <qlistview.h>
 #include <qslider.h>
 #include <qstyle.h>
+#include <qtooltip.h>
 
 #include "icons.h"
 #include "appearance.h"
@@ -71,8 +72,18 @@ Appearance::Appearance(Arranger* a, QWidget* parent, const char* name)
       color  = 0;
       config = new GlobalConfigValues;
 
+      QToolTip::add(fontName0, tr("Main application font, and default font for any\n controls not defined here."));
+      QToolTip::add(fontName1, tr("For small controls like mixer strips.\nAlso timescale small numbers, arranger part name overlay,\n and effects rack."));
+      QToolTip::add(fontName2, tr("Midi track info panel. Transport controls."));
+      QToolTip::add(fontName3, tr("Controller graph and S/X buttons. Large numbers for time\n and tempo scale, and time signature."));
+      QToolTip::add(fontName4, tr("Time scale markers."));
+      QToolTip::add(fontName5, tr("List editor: meta event edit dialog multi-line edit box."));
+      QToolTip::add(fontName6, tr("Mixer label font. Auto-font-sizing up to chosen font size.\nWord-breaking but only with spaces."));
+      QToolTip::add(fontSize6, tr("Maximum mixer label auto-font-sizing font size."));
+      
       // ARRANGER
 
+      /*
       currentBg = ::config.canvasBgPixmap;
       if (currentBg.isEmpty())
             currentBg = "<none>";
@@ -91,6 +102,7 @@ Appearance::Appearance(Arranger* a, QWidget* parent, const char* name)
       eventSpecial->setChecked(config->canvasShowPartEvent & (1 << 6));
       eventButtonGroup->setEnabled(config->canvasShowPartType == 2);
       arrGrid->setChecked(config->canvasShowGrid);
+      */
 
 	// COLORS
       IdListViewItem* id;
@@ -133,6 +145,8 @@ Appearance::Appearance(Arranger* a, QWidget* parent, const char* name)
            new IdListViewItem(0x101, id, "foreground");
       id = new IdListViewItem(0, itemList, "Transport");
            new IdListViewItem(0x200, id, "handle");
+      id = new IdListViewItem(0, itemList, "Midi Editor");
+           new IdListViewItem(0x41d, id, "controller graph");
       id = new IdListViewItem(0, itemList, "Wave Editor");
            new IdListViewItem(0x300, id, "background");
 
@@ -158,6 +172,7 @@ Appearance::Appearance(Arranger* a, QWidget* parent, const char* name)
 	//    STYLE
       //---------------------------------------------------
 
+      /*
       themeComboBox->clear();
       QString cs = muse->style().name();
       cs = cs.lower();
@@ -168,6 +183,7 @@ Appearance::Appearance(Arranger* a, QWidget* parent, const char* name)
                   themeComboBox->setCurrentItem(i);
                   }
             }
+      */
 
       //---------------------------------------------------
 	//    Fonts
@@ -179,21 +195,22 @@ Appearance::Appearance(Arranger* a, QWidget* parent, const char* name)
       fontBrowse3->setPixmap(*openIcon);
       fontBrowse4->setPixmap(*openIcon);
       fontBrowse5->setPixmap(*openIcon);
+      fontBrowse6->setPixmap(*openIcon);
       connect(fontBrowse0, SIGNAL(clicked()), SLOT(browseFont0()));
       connect(fontBrowse1, SIGNAL(clicked()), SLOT(browseFont1()));
       connect(fontBrowse2, SIGNAL(clicked()), SLOT(browseFont2()));
       connect(fontBrowse3, SIGNAL(clicked()), SLOT(browseFont3()));
       connect(fontBrowse4, SIGNAL(clicked()), SLOT(browseFont4()));
       connect(fontBrowse5, SIGNAL(clicked()), SLOT(browseFont5()));
+      connect(fontBrowse6, SIGNAL(clicked()), SLOT(browseFont6()));
 
       connect(applyButton, SIGNAL(clicked()), SLOT(apply()));
       connect(okButton, SIGNAL(clicked()), SLOT(ok()));
       connect(cancelButton, SIGNAL(clicked()), SLOT(cancel()));
       connect(selectBgButton, SIGNAL(clicked()), SLOT(configBackground()));
+      connect(clearBgButton, SIGNAL(clicked()), SLOT(clearBackground()));
       connect(partShowevents, SIGNAL(toggled(bool)), eventButtonGroup, SLOT(setEnabled(bool)));
-      updateColor();
-
-      
+      //updateColor();
       }
 
 //---------------------------------------------------------
@@ -220,6 +237,40 @@ void Appearance::resetValues()
       palette13->setPaletteBackgroundColor(config->palette[13]);
       palette14->setPaletteBackgroundColor(config->palette[14]);
       palette15->setPaletteBackgroundColor(config->palette[15]);
+      
+      currentBg = ::config.canvasBgPixmap;
+      if (currentBg.isEmpty())
+            currentBg = tr("<none>");
+      currentBgLabel->setText(currentBg);
+
+      partShownames->setChecked(config->canvasShowPartType & 1);
+      partShowevents->setChecked(config->canvasShowPartType & 2);
+      partShowCakes->setChecked(!(config->canvasShowPartType & 2));
+
+      eventNoteon->setChecked(config->canvasShowPartEvent & (1 << 0));
+      eventPolypressure->setChecked(config->canvasShowPartEvent & (1 << 1));
+      eventController->setChecked(config->canvasShowPartEvent & (1 << 2));
+      eventProgramchange->setChecked(config->canvasShowPartEvent & (1 << 3));
+      eventAftertouch->setChecked(config->canvasShowPartEvent & (1 << 4));
+      eventPitchbend->setChecked(config->canvasShowPartEvent & (1 << 5));
+      eventSpecial->setChecked(config->canvasShowPartEvent & (1 << 6));
+      //eventButtonGroup->setEnabled(config->canvasShowPartType == 2);
+      eventButtonGroup->setEnabled(config->canvasShowPartType & 2);
+      arrGrid->setChecked(config->canvasShowGrid);
+
+      themeComboBox->clear();
+      QString cs = muse->style().name();
+      cs = cs.lower();
+
+      themeComboBox->insertStringList(QStyleFactory::keys());
+      for (int i = 0; i < themeComboBox->count(); ++i) {
+            if (themeComboBox->text(i).lower() == cs) {
+                  themeComboBox->setCurrentItem(i);
+                  }
+            }
+
+      updateColor();
+      
       }
 
 //---------------------------------------------------------
@@ -266,6 +317,11 @@ void Appearance::updateFonts()
       fontName5->setText(config->fonts[5].family());
       italic5->setChecked(config->fonts[5].italic());
       bold5->setChecked(config->fonts[5].bold());
+      
+      fontSize6->setValue(config->fonts[6].pointSize());
+      fontName6->setText(config->fonts[6].family());
+      italic6->setChecked(config->fonts[6].italic());
+      bold6->setChecked(config->fonts[6].bold());
       }
 
 //---------------------------------------------------------
@@ -281,8 +337,8 @@ void Appearance::apply()
                 showPartType  |= 1;
       if (partShowevents->isChecked())		
                 showPartType  |= 2;
-      if (partShowCakes->isChecked())		
-                showPartType  |= 4;
+      //if (partShowCakes->isChecked())		
+      //          showPartType  |= 4;
 
       config->canvasShowPartType = showPartType;
 
@@ -303,38 +359,56 @@ void Appearance::apply()
 
       config->canvasShowPartEvent = showPartEvent;
 
-      if (currentBg != "<none>")
+      if (currentBg == tr("<none>"))
+            config->canvasBgPixmap = QString();
+      else      
             config->canvasBgPixmap = currentBg;
+      
+      // Added by Tim. p3.3.9
+      config->fonts[0].setFamily(fontName0->text());
+      
       config->fonts[0].setPointSize(fontSize0->value());
       config->fonts[0].setItalic(italic0->isChecked());
       config->fonts[0].setBold(bold0->isChecked());
     	QApplication::setFont(config->fonts[0], true);
 
+      config->fonts[1].setFamily(fontName1->text());
       config->fonts[1].setPointSize(fontSize1->value());
       config->fonts[1].setItalic(italic1->isChecked());
       config->fonts[1].setBold(bold1->isChecked());
 
+      config->fonts[2].setFamily(fontName2->text());
       config->fonts[2].setPointSize(fontSize2->value());
       config->fonts[2].setItalic(italic2->isChecked());
       config->fonts[2].setBold(bold2->isChecked());
 
+      config->fonts[3].setFamily(fontName3->text());
       config->fonts[3].setPointSize(fontSize3->value());
       config->fonts[3].setItalic(italic3->isChecked());
       config->fonts[3].setBold(bold3->isChecked());
 
+      config->fonts[4].setFamily(fontName4->text());
       config->fonts[4].setPointSize(fontSize4->value());
       config->fonts[4].setItalic(italic4->isChecked());
       config->fonts[4].setBold(bold4->isChecked());
 
+      config->fonts[5].setFamily(fontName5->text());
       config->fonts[5].setPointSize(fontSize5->value());
       config->fonts[5].setItalic(italic5->isChecked());
       config->fonts[5].setBold(bold5->isChecked());
 
-	config->style = themeComboBox->currentText();
-	// setting up a new theme might change the fontsize, so re-read
+      config->fonts[6].setFamily(fontName6->text());
+      config->fonts[6].setPointSize(fontSize6->value());
+      config->fonts[6].setItalic(italic6->isChecked());
+      config->fonts[6].setBold(bold6->isChecked());
+
+      config->style = themeComboBox->currentText();
+    	// setting up a new theme might change the fontsize, so re-read
       fontSize0->setValue(QApplication::font().pointSize());
 
-	// set colors...
+      config->canvasShowGrid = arrGrid->isChecked();
+      
+      // set colors...
       ::config = *config;
       muse->changeConfig(true);
       }
@@ -365,12 +439,22 @@ void Appearance::cancel()
 void Appearance::configBackground()
       {
       QString cur(currentBg);
-      if (cur == "<none>")
+      if (cur == tr("<none>"))
             cur = museGlobalShare + "/wallpapers";
       currentBg = getImageFileName(cur, image_file_pattern, this,
          tr("MusE: load image"));
       if (currentBg.isEmpty())
-            currentBg = "<none>";
+            currentBg = tr("<none>");
+      currentBgLabel->setText(currentBg);
+      }
+
+//---------------------------------------------------------
+//   clearBackground
+//---------------------------------------------------------
+
+void Appearance::clearBackground()
+      {
+      currentBg = tr("<none>");
       currentBgLabel->setText(currentBg);
       }
 
@@ -423,6 +507,7 @@ void Appearance::colorItemSelectionChanged()
             case 0x41a: color = &config->selectTrackBg;  break;
             case 0x41b: color = &config->selectTrackFg;  break;
             case 0x41c: color = &config->partCanvasBg; break;
+            case 0x41d: color = &config->ctrlGraphFg; break;
 
             default:
                   color = 0;
@@ -619,6 +704,7 @@ void Appearance::browseFont2() { browseFont(2); }
 void Appearance::browseFont3() { browseFont(3); }
 void Appearance::browseFont4() { browseFont(4); }
 void Appearance::browseFont5() { browseFont(5); }
+void Appearance::browseFont6() { browseFont(6); }
 
 void Appearance::browseFont(int n)
       {
