@@ -28,6 +28,7 @@ class Appearance : public AppearanceDialogBase {
       void ok();
       void cancel();
       void configBackground();
+      void clearBackground();
       void colorItemSelectionChanged();
       void browseFont(int);
       void browseFont0();
@@ -36,6 +37,7 @@ class Appearance : public AppearanceDialogBase {
       void browseFont3();
       void browseFont4();
       void browseFont5();
+      void browseFont6();
       void rsliderChanged(int);
       void gsliderChanged(int);
       void bsliderChanged(int);
