@@ -67,14 +67,14 @@ QString TWhatsThis::text(const QPoint& pos)
       if (section == -1)
             return QString::null;
       switch(section) {
-            case COL_RECORD:   return QHeader::tr("Enable Recording"); break;
-            case COL_MUTE:     return QHeader::tr("Mute Indicator"); break;
-            case COL_SOLO:     return QHeader::tr("Solo Indicator"); break;
-            case COL_CLASS:    return QHeader::tr("Track Type"); break;
-            case COL_NAME:     return QHeader::tr("Track Name"); break;
-            case COL_OCHANNEL: return QHeader::tr("Output Channel Number"); break;
-            case COL_OPORT:    return QHeader::tr("Output Port"); break;
-            case COL_TIMELOCK: return QHeader::tr("Time Lock"); break;
+            case COL_RECORD:   return QHeader::tr("Enable recording. Click to toggle."); break;
+            case COL_MUTE:     return QHeader::tr("Mute indicator. Click to toggle.\nRight-click to toggle track on/off.\nMute is designed for rapid, repeated action.\nOn/Off is not!"); break;
+            case COL_SOLO:     return QHeader::tr("Solo indicator. Click to toggle.\nConnected tracks are also 'phantom' soloed,\n indicated by a dark square."); break;
+            case COL_CLASS:    return QHeader::tr("Track type. Right-click to change\n midi and drum track types."); break;
+            case COL_NAME:     return QHeader::tr("Track name. Double-click to edit.\nRight-click for more options."); break;
+            case COL_OCHANNEL: return QHeader::tr("Midi/drum track: Output channel number.\nAudio track: Channels.\nMid/right-click to change."); break;
+            case COL_OPORT:    return QHeader::tr("Midi/drum track: Output port.\nSynth track: Assigned midi port.\nLeft-click to change.\nRight-click to show GUI."); break;
+            case COL_TIMELOCK: return QHeader::tr("Time lock"); break;
             default: break;
             }
       return QString::null;
@@ -123,6 +123,7 @@ Arranger::Arranger(QMainWindow* parent, const char* name)
             raster->insertItem(tr(rastval[i]), i);
       raster->setCurrentItem(1);
       connect(raster, SIGNAL(activated(int)), SLOT(_setRaster(int)));
+      raster->setFocusPolicy(QWidget::NoFocus);
 
       // Song len
       label = new QLabel(tr("Len"), toolbar, "Len");
@@ -147,6 +148,7 @@ Arranger::Arranger(QMainWindow* parent, const char* name)
       connect(typeBox, SIGNAL(activated(int)), SLOT(modeChange(int)));
       QToolTip::add(typeBox, tr("midi song type"));
       QWhatsThis::add(typeBox, tr("midi song type"));
+      typeBox->setFocusPolicy(QWidget::NoFocus);
 
       label = new QLabel(tr("Pitch"), toolbar, "Pitch");
       label->setAlignment(AlignRight|AlignVCenter);
@@ -235,12 +237,16 @@ Arranger::Arranger(QMainWindow* parent, const char* name)
       header->addLabel(tr("Port"), 60);
       header->addLabel(tr("Ch"), 30);
       header->addLabel(tr("T"), fm1.width('T')+fw);
+      //header->addLabel(tr("Automation"),30);
       header->setResizeEnabled(false, COL_RECORD);
       header->setResizeEnabled(false, COL_MUTE);
       header->setResizeEnabled(false, COL_SOLO);
       header->setResizeEnabled(false, COL_CLASS);
       header->setResizeEnabled(false, COL_OCHANNEL);
       header->setResizeEnabled(false, COL_TIMELOCK);
+      //header->setResizeEnabled(true, COL_AUTOMATION);
+      header->setResizeEnabled(true, COL_NAME);
+      header->setResizeEnabled(true, COL_OPORT);
 
       header->setTracking(true);
 
@@ -284,7 +290,10 @@ Arranger::Arranger(QMainWindow* parent, const char* name)
       hscroll = new ScrollScale(-1000, -10, xscale, song->len(), Horizontal, editor, -offset);
       ib->setFixedHeight(hscroll->sizeHint().height());
 
-      vscroll = new QScrollBar(1, 20*20, 1, 5, 0, Vertical, editor);
+      // Changed p3.3.43 Too small steps for me...
+      //vscroll = new QScrollBar(1, 20*20, 1, 5, 0, Vertical, editor);
+      vscroll = new QScrollBar(1, 20*20, 5, 25, 0, Vertical, editor);
+      
       list->setScroll(vscroll);
 
       QValueList<int> vallist;
@@ -305,6 +314,8 @@ Arranger::Arranger(QMainWindow* parent, const char* name)
       connect(canvas, SIGNAL(setUsedTool(int)), this, SIGNAL(setUsedTool(int)));
       connect(canvas, SIGNAL(trackChanged(Track*)), list, SLOT(selectTrack(Track*)));
       connect(list, SIGNAL(keyPressExt(QKeyEvent*)), canvas, SLOT(redirKeypress(QKeyEvent*)));
+      connect(canvas, SIGNAL(selectTrackAbove()), list, SLOT(selectTrackAbove()));
+      connect(canvas, SIGNAL(selectTrackBelow()), list, SLOT(selectTrackBelow()));
 
       connect(this, SIGNAL(redirectWheelEvent(QWheelEvent*)), canvas, SLOT(redirectedWheelEvent(QWheelEvent*)));
       connect(list, SIGNAL(redirectWheelEvent(QWheelEvent*)), canvas, SLOT(redirectedWheelEvent(QWheelEvent*)));
@@ -340,7 +351,9 @@ Arranger::Arranger(QMainWindow* parent, const char* name)
       connect(canvas, SIGNAL(toolChanged(int)), SIGNAL(toolChanged(int)));
 //      connect(song, SIGNAL(posChanged(int, unsigned, bool)), SLOT(seek()));
 
-      connect(time, SIGNAL(addMarker(int)), SIGNAL(addMarker(int)));
+      // Removed p3.3.43 
+      // Song::addMarker() already emits a 'markerChanged'.
+      //connect(time, SIGNAL(addMarker(int)), SIGNAL(addMarker(int)));
       
       configChanged();  // set configuration values
       showTrackInfo(showTrackinfoFlag);
@@ -649,6 +662,12 @@ void Arranger::cmd(int cmd)
             case CMD_PASTE_CLONE_PART_TO_TRACK:
                   ncmd = PartCanvas::CMD_PASTE_CLONE_PART_TO_TRACK;
                   break;
+            case CMD_INSERT_PART:
+                  ncmd = PartCanvas::CMD_INSERT_PART;
+                  break;
+            case CMD_INSERT_EMPTYMEAS:
+                  ncmd = PartCanvas::CMD_INSERT_EMPTYMEAS;
+                  break;
             default:
                   return;
             }
@@ -938,3 +957,8 @@ void Arranger::wheelEvent(QWheelEvent* ev)
       {
       emit redirectWheelEvent(ev);
       }
+
+void Arranger::controllerChanged(Track *t)
+{
+      canvas->controllerChanged(t);
+}
