@@ -17,6 +17,7 @@
 #include "midieditor.h"
 #include "pcanvas.h"
 #include "mtrackinfobase.h"
+#include "trackautomationview.h"
 
 class QMainWindow;
 class QToolButton;
@@ -132,9 +133,9 @@ class Arranger : public QWidget {
       void trackInfoScroll(int);
       
       //void iNameChanged();
-      void iInputChannelChanged(const QString&);
+      ///void iInputChannelChanged(const QString&);
       void iOutputChannelChanged(int);
-      void iInputPortChanged(const QString&);
+      ///void iInputPortChanged(const QString&);
       void iOutputPortChanged(int);
       void iProgHBankChanged();
       void iProgLBankChanged();
@@ -166,6 +167,9 @@ class Arranger : public QWidget {
       void panRecClicked();
       void recEchoToggled(bool);
       void verticalScrollSetYpos(unsigned);
+      void inRoutesPressed();
+      void outRoutesPressed();
+      void routingPopupMenuActivated(int /*id*/);
       
    signals:
       void redirectWheelEvent(QWheelEvent*);
@@ -175,7 +179,7 @@ class Arranger : public QWidget {
       void dropMidiFile(const QString&);
       void startEditor(PartList*, int);
       void toolChanged(int);
-      void addMarker(int);
+      //void addMarker(int);
       void setUsedTool(int);
 
 
@@ -191,9 +195,11 @@ class Arranger : public QWidget {
       void setTool(int);
       void updateTrackInfo(int flags);
       void configChanged();
+      void controllerChanged(Track *t);
 
    public:
-      enum { CMD_CUT_PART, CMD_COPY_PART, CMD_PASTE_PART, CMD_PASTE_CLONE_PART, CMD_PASTE_PART_TO_TRACK, CMD_PASTE_CLONE_PART_TO_TRACK };
+      enum { CMD_CUT_PART, CMD_COPY_PART, CMD_PASTE_PART, CMD_PASTE_CLONE_PART, CMD_PASTE_PART_TO_TRACK, CMD_PASTE_CLONE_PART_TO_TRACK,
+             CMD_INSERT_PART, CMD_INSERT_EMPTYMEAS };
 
       Arranger(QMainWindow* parent, const char* name = 0);
       void setMode(int);
