@@ -10,6 +10,7 @@
 
 #include "song.h"
 #include "canvas.h"
+#include "trackautomationview.h"
 #define beats     4
 
 //---------------------------------------------------------
@@ -46,6 +47,7 @@ class PartCanvas : public Canvas {
       int curColorIndex;
       bool editMode;
 
+      std::vector<TrackAutomationView*> automationViews;
       Q_OBJECT
       virtual void keyPress(QKeyEvent*);
       virtual void mousePress(QMouseEvent*);
@@ -84,12 +86,15 @@ class PartCanvas : public Canvas {
       void splitItem(CItem* item, const QPoint&);
 
       void copy(PartList*);
-      void paste(bool clone = false, bool toTrack = true);
+      void paste(bool clone = false, bool toTrack = true, bool doInsert=false);
       int pasteAt(const QString&, Track*, int, bool clone = false, bool toTrack = true);
+      void movePartsTotheRight(int startTick, int length);
       //Part* readClone(Xml&, Track*, bool toTrack = true);
       void drawWavePart(QPainter&, const QRect&, WavePart*, const QRect&);
       Track* y2Track(int) const;
       void drawAudioTrack(QPainter& p, const QRect& r, AudioTrack* track);
+      void drawAutomation(QPainter& p, const QRect& r, AudioTrack* track);
+
 
    protected:
       virtual void drawCanvas(QPainter&, const QRect&);
@@ -103,6 +108,8 @@ class PartCanvas : public Canvas {
       void dropMidiFile(const QString&);
       void setUsedTool(int);
       void trackChanged(Track*);
+      void selectTrackAbove();
+      void selectTrackBelow();
 
       void startEditor(PartList*, int);
 
@@ -110,11 +117,13 @@ class PartCanvas : public Canvas {
       void returnPressed();
 
    public:
-      enum { CMD_CUT_PART, CMD_COPY_PART, CMD_PASTE_PART, CMD_PASTE_CLONE_PART, CMD_PASTE_PART_TO_TRACK, CMD_PASTE_CLONE_PART_TO_TRACK };
+      enum { CMD_CUT_PART, CMD_COPY_PART, CMD_PASTE_PART, CMD_PASTE_CLONE_PART, CMD_PASTE_PART_TO_TRACK, CMD_PASTE_CLONE_PART_TO_TRACK,
+             CMD_INSERT_PART, CMD_INSERT_EMPTYMEAS };
 
       PartCanvas(int* raster, QWidget* parent, int, int);
       void partsChanged();
       void cmd(int);
+      void controllerChanged(Track *t);
    public slots:
    void redirKeypress(QKeyEvent* e) { keyPress(e); }
       };
