@@ -37,6 +37,11 @@
 #include "event.h"
 #include "midiedit/drummap.h"
 #include "synth.h"
+#include "config.h"
+
+#ifdef DSSI_SUPPORT
+#include "dssihost.h"
+#endif
 
 extern QPopupMenu* populateAddSynth(QWidget* parent, QObject* obj = 0, const char* slot = 0);
 
@@ -58,14 +63,15 @@ void THeaderTip::maybeTip(const QPoint &pos)
       QString p;
       switch (section) {
             case COL_RECORD:   p = QHeader::tr("Enable Recording"); break;
-            case COL_MUTE:     p = QHeader::tr("Mute Indicator"); break;
-            case COL_SOLO:     p = QHeader::tr("Solo/Pre Fader Listening"); break;
+            case COL_MUTE:     p = QHeader::tr("Mute/Off Indicator"); break;
+            case COL_SOLO:     p = QHeader::tr("Solo Indicator"); break;
             case COL_CLASS:    p = QHeader::tr("Track Type"); break;
             case COL_NAME:     p = QHeader::tr("Track Name"); break;
-            case COL_OCHANNEL: p = QHeader::tr("Output Channel Number"); break;
+            case COL_OCHANNEL: p = QHeader::tr("Midi output channel number or audio channels"); break;
             //case COL_OPORT:    p = QHeader::tr("Output Port"); break;
-            case COL_OPORT:    p = QHeader::tr("Output midi port, or synth midi port"); break;
+            case COL_OPORT:    p = QHeader::tr("Midi output port or synth midi port"); break;
             case COL_TIMELOCK: p = QHeader::tr("Time Lock"); break;
+//            case COL_AUTOMATION: p = QHeader::tr("Automation parameter selection"); break;
             default: return;
             }
       tip(r, p);
@@ -171,6 +177,8 @@ void TList::paint(const QRect& r)
             p.drawTiledPixmap(rect, bgPixmap, QPoint(rect.x(), ypos + rect.y()));
       p.setClipRegion(rect);
 
+      //printf("TList::paint hasClipping:%d\n", p.hasClipping());   // Tested true.
+      
       int y  = rect.y();
       int w  = rect.width();
       int h  = rect.height();
@@ -179,7 +187,7 @@ void TList::paint(const QRect& r)
 
       //---------------------------------------------------
       //    Tracks
-      //---------------------------------------------------
+      //---------------------------------------------------                         
 
       TrackList* l = song->tracks();
       int idx = 0;
@@ -343,6 +351,19 @@ void TList::paint(const QRect& r)
                               p.drawText(r, Qt::AlignVCenter|Qt::AlignLeft, s);
                               }
                               break;
+//                        case COL_AUTOMATION:
+//                              {
+//                              QString s="-";
+//
+//                              if (!track->isMidiTrack()) {
+//                                    int count = ((AudioTrack*)track)->controller()->size();
+//                                    s.sprintf("%d", count);
+//                                    }
+//
+//
+//                              p.drawText(r, Qt::AlignVCenter|Qt::AlignLeft, s);
+//                              }
+//                              break;
                         default:
                               break;
                         }
@@ -587,10 +608,21 @@ void TList::oportPropertyPopupMenu(Track* t, int x, int y)
         QPopupMenu* p = new QPopupMenu(this);
         p->setCheckable(true);
         p->insertItem(tr("Show Gui"), 0);
-  
+        printf("synth hasgui %d, gui visible %d\n",synth->hasGui(), synth->guiVisible());
         p->setItemEnabled(0, synth->hasGui());
         p->setItemChecked(0, synth->guiVisible());
   
+        #ifndef OSC_SUPPORT
+        #ifdef DSSI_SUPPORT
+        if(dynamic_cast<DssiSynthIF*>(synth->sif()))
+        {
+            printf("entering this wierd if statement\n");
+          p->setItemChecked(0, false);
+          p->setItemEnabled(0, false);
+        }  
+        #endif
+        #endif
+        
         int n = p->exec(mapToGlobal(QPoint(x, y)), 0);
         if (n == 0) {
               bool show = !synth->guiVisible();
@@ -613,6 +645,17 @@ void TList::oportPropertyPopupMenu(Track* t, int x, int y)
       p->setItemEnabled(0, port->hasGui());
       p->setItemChecked(0, port->guiVisible());
 
+      #ifndef OSC_SUPPORT
+      #ifdef DSSI_SUPPORT
+      MidiDevice* dev = port->device();
+      if(dev && dev->isSynti() && (dynamic_cast<DssiSynthIF*>(((SynthI*)dev)->sif())))
+      {
+        p->setItemChecked(0, false);
+        p->setItemEnabled(0, false);
+      }  
+      #endif
+      #endif
+      
       int n = p->exec(mapToGlobal(QPoint(x, y)), 0);
       if (n == 0) {
             bool show = !port->guiVisible();
@@ -696,6 +739,14 @@ void TList::moveSelection(int n)
                         }
                   (*s)->setSelected(false);
                   (*t)->setSelected(true);
+
+                  // rec enable track if expected
+                  TrackList recd = getRecEnabledTracks();
+                  if (recd.size() == 1 && config.moveArmedCheckBox) { // one rec enabled track, move rec enabled with selection
+                    song->setRecordFlag((Track*)recd.front(),false);
+                    song->setRecordFlag((*t),true);
+                  }
+
                   if (editTrack && editTrack != *t)
                         returnPressed();
                   redraw();
@@ -705,6 +756,20 @@ void TList::moveSelection(int n)
       emit selectionChanged();
       }
 
+TrackList TList::getRecEnabledTracks()
+{
+  //printf("getRecEnabledTracks\n");
+      TrackList recEnabled;
+      TrackList* tracks = song->tracks();
+      for (iTrack t = tracks->begin(); t != tracks->end(); ++t) {
+        if ((*t)->recordFlag()) {
+          //printf("rec enabled track\n");
+          recEnabled.push_back(*t);
+        }
+      }
+      return recEnabled;
+}
+
 //---------------------------------------------------------
 //   mousePressEvent
 //---------------------------------------------------------
@@ -788,6 +853,7 @@ void TList::mousePressEvent(QMouseEvent* ev)
                     {
                       song->deselectTracks();
                       t->setSelected(true);
+
                       emit selectionChanged();
                       adjustScrollbar();
                     }  
@@ -878,10 +944,16 @@ void TList::mousePressEvent(QMouseEvent* ev)
                     
                   break;
             case COL_MUTE:
-                  if (t->off())
-                        t->setOff(false);
+                  // p3.3.29
+                  if ((button == QMouseEvent::RightButton) || (ev->state() & ControlButton))
+                    t->setOff(!t->off());
                   else
-                        t->setMute(!t->mute());
+                  {
+                    if (t->off())
+                          t->setOff(false);
+                    else
+                          t->setMute(!t->mute());
+                  }        
                   song->update(SC_MUTE);
                   break;
             case COL_SOLO:
@@ -894,6 +966,13 @@ void TList::mousePressEvent(QMouseEvent* ev)
                         if (!shift) {
                               song->deselectTracks();
                               t->setSelected(true);
+
+                              // rec enable track if expected
+                              TrackList recd = getRecEnabledTracks();
+                              if (recd.size() == 1 && config.moveArmedCheckBox) { // one rec enabled track, move rec enabled with selection
+                                song->setRecordFlag((Track*)recd.front(),false);
+                                song->setRecordFlag(t,true);
+                              }
                               }
                         else
                               t->setSelected(!t->selected());
@@ -1011,11 +1090,35 @@ void TList::selectTrack(Track* tr)
       song->deselectTracks();
       tr->setSelected(true);
       
+
+      // rec enable track if expected
+      TrackList recd = getRecEnabledTracks();
+      if (recd.size() == 1 && config.moveArmedCheckBox) { // one rec enabled track, move rec enabled with selection
+        song->setRecordFlag((Track*)recd.front(),false);
+        song->setRecordFlag(tr,true);
+      }
+
       // By T356. Force a redraw for wave tracks, since it does not seem to happen.
-      if(!tr->isMidiTrack())
+      //if(!tr->isMidiTrack())
         redraw();
       emit selectionChanged();
       }
+
+//---------------------------------------------------------
+//   selectTrackAbove
+//---------------------------------------------------------
+void TList::selectTrackAbove()
+{
+     moveSelection(-1);
+}
+//---------------------------------------------------------
+//   selectTrackBelow
+//---------------------------------------------------------
+void TList::selectTrackBelow()
+{
+     moveSelection(1);
+}
+
 //---------------------------------------------------------
 //   mouseMoveEvent
 //---------------------------------------------------------
@@ -1143,12 +1246,19 @@ void TList::wheelEvent(QWheelEvent* ev)
             case COL_NONE:
             case COL_CLASS:
             case COL_NAME:
+            //case COL_AUTOMATION:
                   break;
             case COL_MUTE:
-                  if (t->off())
-                        t->setOff(false);
+                  // p3.3.29
+                  if (ev->state() & ControlButton)
+                    t->setOff(!t->off());
                   else
-                        t->setMute(!t->mute());
+                  {
+                    if (t->off())
+                          t->setOff(false);
+                    else
+                          t->setMute(!t->mute());
+                  }        
                   song->update(SC_MUTE);
                   break;
 
@@ -1272,21 +1382,41 @@ void TList::setYPos(int y)
       if (pm.isNull())
             return;
       if (!pmValid) {
+            //printf("TList::setYPos y:%d delta:%d pmValid is false - redrawing...\n", y, delta);
             redraw();
             return;
             }
       int w = width();
       int h = height();
       QRect r;
+      //printf("TList::setYPos y:%d delta:%d w:%d h:%d\n", y, delta, w, h);
+      
       if (delta >= h || delta <= -h)
+      {
+            //printf("TList::setYPos delta >= h || delta <= -h\n");
             r = QRect(0, 0, w, h);
+      }      
       else if (delta < 0) {   // shift up
+            //printf("TList::setYPos delta < 0 : shift up\n");
             bitBlt(&pm,  0, 0, &pm, 0, -delta, w, h + delta, CopyROP, true);
             r = QRect(0, h + delta, w, -delta);
             }
       else {                  // shift down
+            //printf("TList::setYPos delta !< 0 : shift down\n");
             bitBlt(&pm,  0, delta, &pm, 0, 0, w, h-delta, CopyROP, true);
-            r = QRect(0, 0, w, delta);
+            
+            // NOTE: June 2 2010: On my machine with an old NV V8200 + prop drivers (curr 96.43.11),
+            //  this is a problem. There is severe graphical corruption.
+            // Not just here but several other windows (ex. ladspa browser), 
+            //  and I believe (?) other QT3 apps. QT4 apps don't do it. 
+            // Neither does it happen when xorg drivers used. 
+            //
+            // FIXME: This change cures it for me, but we shouldn't leave this in - shouldn't need to do this...
+            //
+            //r = QRect(0, 0, w, delta);
+            // Changed p3.3.43
+            r = QRect(0, 0, w, h);
+            
             }
       paint(r);
       update();
