@@ -11,6 +11,7 @@
 #include <qtooltip.h>
 #include <qwhatsthis.h>
 #include <qheader.h>
+#include "track.h"
 
 class QPainter;
 class ScrollScale;
@@ -31,6 +32,7 @@ enum TrackColumn {
       COL_TIMELOCK,
       COL_NONE = -1
       };
+//      COL_AUTOMATION, -- not enabled
 
 //----------------------------------------------------------
 //   THeaderTip
@@ -91,6 +93,7 @@ class TList : public QWidget {
       void redraw(const QRect& r);
       Track* y2Track(int) const;
       void classesPopupMenu(Track*, int x, int y);
+      TrackList getRecEnabledTracks();
 
    private slots:
       void returnPressed();
@@ -106,6 +109,8 @@ class TList : public QWidget {
       void setYPos(int);
       void redraw();
       void selectTrack(Track*);
+      void selectTrackAbove();
+      void selectTrackBelow();
 
    public:
       TList(Header*, QWidget* parent, const char* name);
