@@ -35,8 +35,12 @@ extern double curTime();
 Audio* audio;
 AudioDevice* audioDevice;   // current audio device in use
 
-static const unsigned char mmcDeferredPlayMsg[] = { 0x7f, 0x7f, 0x06, 0x03 };
-static const unsigned char mmcStopMsg[] =         { 0x7f, 0x7f, 0x06, 0x01 };
+// p3.3.25
+extern unsigned int volatile midiExtSyncTicks;
+
+
+//static const unsigned char mmcDeferredPlayMsg[] = { 0x7f, 0x7f, 0x06, 0x03 };
+//static const unsigned char mmcStopMsg[] =         { 0x7f, 0x7f, 0x06, 0x01 };
 
 const char* seqMsgList[] = {
       "SEQM_ADD_TRACK", "SEQM_REMOVE_TRACK", "SEQM_CHANGE_TRACK", "SEQM_MOVE_TRACK",
@@ -227,10 +231,9 @@ void Audio::stop(bool)
 
 bool Audio::sync(int jackState, unsigned frame)
       {
-      // Added by Tim. p3.3.20
-      if(debugMsg)
-        printf("Audio::sync state %s jackState %s frame %d\n", audioStates[state], audioStates[jackState], frame);
       
+// Changed by Tim. p3.3.24
+/*      
       bool done = true;
       if (state == LOOP1) 
             state = LOOP2;
@@ -245,11 +248,40 @@ bool Audio::sync(int jackState, unsigned frame)
                   done = audioPrefetch->seekDone();
             }
       
-      // Added by Tim. p3.3.20
-      //if(debugMsg)
-      //  printf("Audio::sync done:%d state %s\n", done, audioStates[state]);
-      
       return done;
+*/      
+      bool done = true;
+      if (state == LOOP1)
+            state = LOOP2;
+      else {
+            State s = State(jackState);
+            //
+            //  STOP -> START_PLAY      start rolling
+            //  STOP -> STOP            seek in stop state
+            //  PLAY -> START_PLAY  seek in play state
+
+            if (state != START_PLAY) {
+                //Pos p(frame, AL::FRAMES);
+                //    seek(p);
+                Pos p(frame, false);
+                seek(p);
+              if (!_freewheel)
+                      done = audioPrefetch->seekDone();
+                if (s == START_PLAY)
+                        state = START_PLAY;
+                }
+            else {
+                //if (frame != _seqTime.pos.frame()) {
+                if (frame != _pos.frame()) {
+                        // seek during seek
+                            //seek(Pos(frame, AL::FRAMES));
+                            seek(Pos(frame, false));
+                        }
+                done = audioPrefetch->seekDone();
+                  }
+            }
+      return done;
+      
       }
 
 //---------------------------------------------------------
@@ -307,7 +339,6 @@ void Audio::process(unsigned frames)
 
       int jackState = audioDevice->getState();
 
-      // Added by Tim. p3.3.20
       //if(debugMsg)
       //  printf("Audio::process Current state:%s jackState:%s\n", audioStates[state], audioStates[jackState]);
       
@@ -324,6 +355,16 @@ void Audio::process(unsigned frames)
             startRolling();
             }
       else if (isPlaying() && jackState == STOP) {
+            // p3.3.43 Make sure to stop bounce and freewheel mode, for example if user presses stop 
+            //  in QJackCtl before right-hand marker is reached (which is handled below).
+            //printf("Audio::process isPlaying() && jackState == STOP\n");
+            //if (_bounce) 
+            //{
+              //printf("  stopping bounce...\n");
+            //  _bounce = false;
+            //  write(sigFd, "F", 1);
+            //}
+            
             stopRolling();
             }
       else if (state == START_PLAY && jackState == STOP) {
@@ -382,7 +423,6 @@ void Audio::process(unsigned frames)
                && !(song->record()
                 || _bounce
                 || song->loop())) {
-                  // Added by Tim. p3.3.20
                   //if(debugMsg)
                   //  printf("Audio::process curTickPos >= song->len\n");
                   
@@ -428,10 +468,23 @@ void Audio::process(unsigned frames)
 // printf("  process: seek to %d, end %d\n", _loopFrame, loop.frame());
                         }
                   }
-            Pos ppp(_pos);
-            ppp += frames;
-            nextTickPos = ppp.tick();
+            
+            
+            // p3.3.25
+            if(extSyncFlag.value())
+            {
+              nextTickPos = curTickPos + midiExtSyncTicks;
+              // Probably not good - interfere with midi thread.
+              midiExtSyncTicks = 0;
             }
+            else
+            {
+              
+              Pos ppp(_pos);
+              ppp += frames;
+              nextTickPos = ppp.tick();
+            }
+          }
       //
       // resync with audio interface
       //
@@ -439,7 +492,6 @@ void Audio::process(unsigned frames)
       syncTime    = curTime();
       frameOffset = syncFrame - samplePos;
 
-      // Added by Tim. p3.3.13
       //printf("Audio::process calling process1:\n");
       
       process1(samplePos, offset, frames);
@@ -548,10 +600,11 @@ void Audio::process1(unsigned samplePos, unsigned offset, unsigned frames)
           float data[frames * channels];
           for (int i = 0; i < channels; ++i)
                 buffer[i] = data + i * frames;
-          // Added by Tim. p3.3.13
           //printf("Audio::process1 calling track->copyData for track:%s\n", track->name().latin1());
       
-          track->copyData(samplePos, channels, frames, buffer);
+          // p3.3.38
+          //track->copyData(samplePos, channels, frames, buffer);
+          track->copyData(samplePos, channels, -1, -1, frames, buffer);
         }
       }      
     }
@@ -591,7 +644,9 @@ void Audio::processMsg(AudioMsg* msg)
                   msg->snode->addPlugin(msg->plugin, msg->ival);
                   break;
             case AUDIO_SET_PLUGIN_CTRL_VAL:
-                  msg->plugin->track()->setPluginCtrlVal(msg->ival, msg->dval);
+                  //msg->plugin->track()->setPluginCtrlVal(msg->ival, msg->dval);
+                  // p3.3.43
+                  msg->snode->setPluginCtrlVal(msg->ival, msg->dval);
                   break;
             case AUDIO_SWAP_CONTROLLER_IDX:
                   msg->snode->swapControllerIDX(msg->a, msg->b);
@@ -719,10 +774,13 @@ void Audio::processMsg(AudioMsg* msg)
 void Audio::seek(const Pos& p)
       {
       if (_pos == p) {
-            printf("seek: already there\n");
-            return;
+            if(debugMsg)
+              printf("Audio::seek already there\n");
+            return;        
             }
       
+            // p3.3.23
+            //printf("Audio::seek frame:%d\n", p.frame());
       _pos        = p;
       if (!checkAudioDevice()) return;
       syncFrame   = audioDevice->framePos();
@@ -731,13 +789,18 @@ void Audio::seek(const Pos& p)
 
       midiSeq->msgSeek();     // handle stuck notes and set
                               // controller for new position
-      //if(genMCSync) 
-      //{
+      
+      // p3.3.31
+      // Don't send if external sync is on. The master, and our sync routing system will take care of that.
+      if(!extSyncFlag.value())
+      {
+        
         for(int port = 0; port < MIDI_PORTS; ++port) 
         {
           MidiPort* mp = &midiPorts[port];
           MidiDevice* dev = mp->device();
-          if(!dev || !mp->syncInfo().MCOut())
+          //if(!dev || !mp->syncInfo().MCOut())
+          if(!dev || !mp->syncInfo().MRTOut())
             continue;
             
           // Added by T356: Shall we check for device write open flag to see if it's ok to send?...
@@ -768,7 +831,7 @@ void Audio::seek(const Pos& p)
           if(isPlaying)
             mp->sendContinue();
         }
-      //}
+      }
         
       /*
       if(genMCSync) 
@@ -893,42 +956,50 @@ void Audio::startRolling()
       state = PLAY;
       write(sigFd, "1", 1);   // Play
 
-      // Changed by Tim. p3.3.6
-      //if (genMMC)
-      //    midiPorts[txSyncPort].sendSysex(mmcDeferredPlayMsg, sizeof(mmcDeferredPlayMsg));
-      //if (genMCSync) {
-      //      if (curTickPos)
-      //            midiPorts[txSyncPort].sendContinue();
-      //      else
-      //            midiPorts[txSyncPort].sendStart();
-      //      }
-      for(int port = 0; port < MIDI_PORTS; ++port) 
+      // p3.3.31
+      // Don't send if external sync is on. The master, and our sync routing system will take care of that.
+      if(!extSyncFlag.value())
       {
-        MidiPort* mp = &midiPorts[port];
-        MidiDevice* dev = mp->device();
-        if(!dev)
-          continue;
-            
-        // Shall we check open flags?
-        //if(!(dev->rwFlags() & 0x1) || !(dev->openFlags() & 1))
-        //if(!(dev->openFlags() & 1))
-        //  continue;
         
-        MidiSyncInfo& si = mp->syncInfo();
-          
-        //if(genMMC && si.MMCOut())
-        if(si.MMCOut())
-          mp->sendSysex(mmcDeferredPlayMsg, sizeof(mmcDeferredPlayMsg));
-        
-        //if(genMCSync && si.MCOut())
-        if(si.MCOut())
+        // Changed by Tim. p3.3.6
+        //if (genMMC)
+        //    midiPorts[txSyncPort].sendSysex(mmcDeferredPlayMsg, sizeof(mmcDeferredPlayMsg));
+        //if (genMCSync) {
+        //      if (curTickPos)
+        //            midiPorts[txSyncPort].sendContinue();
+        //      else
+        //            midiPorts[txSyncPort].sendStart();
+        //      }
+        for(int port = 0; port < MIDI_PORTS; ++port) 
         {
-          if(curTickPos)
-            mp->sendContinue();
-          else
-            mp->sendStart();
-        }  
-      }
+          MidiPort* mp = &midiPorts[port];
+          MidiDevice* dev = mp->device();
+          if(!dev)
+            continue;
+              
+          // Shall we check open flags?
+          //if(!(dev->rwFlags() & 0x1) || !(dev->openFlags() & 1))
+          //if(!(dev->openFlags() & 1))
+          //  continue;
+          
+          MidiSyncInfo& si = mp->syncInfo();
+            
+          //if(genMMC && si.MMCOut())
+          if(si.MMCOut())
+            //mp->sendSysex(mmcDeferredPlayMsg, sizeof(mmcDeferredPlayMsg));
+            mp->sendMMCDeferredPlay();
+          
+          //if(genMCSync && si.MCOut())
+          //if(si.MCOut())
+          if(si.MRTOut())
+          {
+            if(curTickPos)
+              mp->sendContinue();
+            else
+              mp->sendStart();
+          }  
+        }
+      }  
       
       /*
       for(iMidiDevice imd = midiDevices.begin(); imd != midiDevices.end(); ++imd) 
@@ -1037,6 +1108,8 @@ void Audio::startRolling()
                   }
               }
           }
+     
+     //tempomap.clearExtTempoList();     
      }
 
 //---------------------------------------------------------
@@ -1074,73 +1147,104 @@ void Audio::stopRolling()
         }
 
 #endif
-      // Changed by Tim. p3.3.6
-      //MidiPort* syncPort = &midiPorts[txSyncPort];
-      //if (genMMC) {
-      //      unsigned char mmcPos[] = {
-      //            0x7f, 0x7f, 0x06, 0x44, 0x06, 0x01,
-      //            0, 0, 0, 0, 0
-      //            };
-      //      int frame = tempomap.tick2frame(curTickPos);
-      //      MTC mtc(double(frame) / double(sampleRate));
-      //      mmcPos[6] = mtc.h() | (mtcType << 5);
-      //      mmcPos[7] = mtc.m();
-      //      mmcPos[8] = mtc.s();
-      //      mmcPos[9] = mtc.f();
-      //      mmcPos[10] = mtc.sf();
-      //      syncPort->sendSysex(mmcStopMsg, sizeof(mmcStopMsg));
-      //      syncPort->sendSysex(mmcPos, sizeof(mmcPos));
-      //      }
-      //if (genMCSync) {         // Midi Clock
+      
+      // p3.3.31
+      // Don't send if external sync is on. The master, and our sync routing system will take care of that.
+      if(!extSyncFlag.value())
+      {
+        
+        // Changed by Tim. p3.3.6
+        //MidiPort* syncPort = &midiPorts[txSyncPort];
+        //if (genMMC) {
+        //      unsigned char mmcPos[] = {
+        //            0x7f, 0x7f, 0x06, 0x44, 0x06, 0x01,
+        //            0, 0, 0, 0, 0
+        //            };
+        //      int frame = tempomap.tick2frame(curTickPos);
+        //      MTC mtc(double(frame) / double(sampleRate));
+        //      mmcPos[6] = mtc.h() | (mtcType << 5);
+        //      mmcPos[7] = mtc.m();
+        //      mmcPos[8] = mtc.s();
+        //      mmcPos[9] = mtc.f();
+        //      mmcPos[10] = mtc.sf();
+        //      syncPort->sendSysex(mmcStopMsg, sizeof(mmcStopMsg));
+        //      syncPort->sendSysex(mmcPos, sizeof(mmcPos));
+        //      }
+        //if (genMCSync) {         // Midi Clock
+              // send STOP and
+              // "set song position pointer"
+        //      syncPort->sendStop();
+        //      syncPort->sendSongpos(curTickPos * 4 / config.division);
+        //      }
+        for(int port = 0; port < MIDI_PORTS; ++port) 
+        {
+          MidiPort* mp = &midiPorts[port];
+          MidiDevice* dev = mp->device();
+          if(!dev)
+            continue;
+              
+          // Shall we check open flags?
+          //if(!(dev->rwFlags() & 0x1) || !(dev->openFlags() & 1))
+          //if(!(dev->openFlags() & 1))
+          //  continue;
+          
+          MidiSyncInfo& si = mp->syncInfo();
+            
+          //if(genMMC && si.MMCOut())
+          if(si.MMCOut())
+          {
+            //unsigned char mmcPos[] = {
+            //      0x7f, 0x7f, 0x06, 0x44, 0x06, 0x01,
+            //      0, 0, 0, 0, 0
+            //      };
+  
+            // p3.3.31
+            /*
+            int frame = tempomap.tick2frame(curTickPos);
+            MTC mtc(double(frame) / double(sampleRate));
+            */          
+            
+            //mmcPos[6] = mtc.h() | (mtcType << 5);
+            //mmcPos[7] = mtc.m();
+            //mmcPos[8] = mtc.s();
+            //mmcPos[9] = mtc.f();
+            //mmcPos[10] = mtc.sf();
+            
+            //mp->sendSysex(mmcStopMsg, sizeof(mmcStopMsg));
+            mp->sendMMCStop();
+            //mp->sendSysex(mmcPos, sizeof(mmcPos));
+            
+            // p3.3.31
+            // Added check of option send continue not start.
+            // Hmm, is this required? Seems to make other devices unhappy.
+            /*
+            if(!si.sendContNotStart())
+              mp->sendMMCLocate(mtc.h() | (mtcType << 5), 
+                              mtc.m(), mtc.s(), mtc.f(), mtc.sf());
+            */                  
+            
+          }
+        
+          //if(genMCSync && si.MCOut()) // Midi Clock
+          //if(si.MCOut()) // Midi Clock
+          if(si.MRTOut()) // 
+          {
             // send STOP and
             // "set song position pointer"
-      //      syncPort->sendStop();
-      //      syncPort->sendSongpos(curTickPos * 4 / config.division);
-      //      }
-      for(int port = 0; port < MIDI_PORTS; ++port) 
-      {
-        MidiPort* mp = &midiPorts[port];
-        MidiDevice* dev = mp->device();
-        if(!dev)
-          continue;
+            mp->sendStop();
             
-        // Shall we check open flags?
-        //if(!(dev->rwFlags() & 0x1) || !(dev->openFlags() & 1))
-        //if(!(dev->openFlags() & 1))
-        //  continue;
-        
-        MidiSyncInfo& si = mp->syncInfo();
-          
-        //if(genMMC && si.MMCOut())
-        if(si.MMCOut())
-        {
-          unsigned char mmcPos[] = {
-                0x7f, 0x7f, 0x06, 0x44, 0x06, 0x01,
-                0, 0, 0, 0, 0
-                };
-          int frame = tempomap.tick2frame(curTickPos);
-          MTC mtc(double(frame) / double(sampleRate));
-          mmcPos[6] = mtc.h() | (mtcType << 5);
-          mmcPos[7] = mtc.m();
-          mmcPos[8] = mtc.s();
-          mmcPos[9] = mtc.f();
-          mmcPos[10] = mtc.sf();
-          
-          mp->sendSysex(mmcStopMsg, sizeof(mmcStopMsg));
-          mp->sendSysex(mmcPos, sizeof(mmcPos));
-        }
-      
-        //if(genMCSync && si.MCOut()) // Midi Clock
-        if(si.MCOut()) // Midi Clock
-        {
-          // send STOP and
-          // "set song position pointer"
-          mp->sendStop();
-          mp->sendSongpos(curTickPos * 4 / config.division);
+            // p3.3.31
+            // Added check of option send continue not start.
+            // Hmm, is this required? Seems to make other devices unhappy.
+            /*
+            if(!si.sendContNotStart())
+              mp->sendSongpos(curTickPos * 4 / config.division);
+            */  
+            
+          }
         }
       }
       
-      
       /*
       for(iMidiDevice imd = midiDevices.begin(); imd != midiDevices.end(); ++imd) 
       {
