@@ -22,6 +22,8 @@
 // By T356. For caching jack in/out routing names BEFORE file save. 
 // Jack often shuts down during file save, causing the routes to be lost in the file.
 // cacheJackRouteNames() is ONLY called from MusE::save() in app.cpp
+// Update: Not required any more because the real problem was Jack RT priority, which has been fixed.
+/*
 typedef std::multimap <const int, QString> jackRouteNameMap;
 std::map <const AudioTrack*, jackRouteNameMap > jackRouteNameCache;
 typedef std::multimap <const int, QString>::const_iterator ciJackRouteNameMap;
@@ -54,13 +56,17 @@ void cacheJackRouteNames()
       }                            
     }
 }
+*/
+
 //---------------------------------------------------------
 //   AudioTrack
 //---------------------------------------------------------
 
 AudioTrack::AudioTrack(TrackType t)
+//AudioTrack::AudioTrack(TrackType t, int num_out_bufs)
    : Track(t)
       {
+      //_totalOutChannels = num_out_bufs; // Is either parameter-default MAX_CHANNELS, or custom value passed (used by syntis).
       _processed = false;
       _haveData = false;
       _sendMetronome = false;
@@ -68,19 +74,20 @@ AudioTrack::AudioTrack(TrackType t)
       _efxPipe  = new Pipeline();
       _recFile  = 0;
       _channels = 0;
-      _automationType = AUTO_READ;
+      _automationType = AUTO_OFF;
       //setChannels(1);
       setChannels(2);
-      addController(new CtrlList(AC_VOLUME));
-      addController(new CtrlList(AC_PAN));
-      addController(new CtrlList(AC_MUTE));
+      addController(new CtrlList(AC_VOLUME,"Volume",0.0,1.0));
+      addController(new CtrlList(AC_PAN, "Pan", -1.0, 1.0));
+      addController(new CtrlList(AC_MUTE,"Mute",0.0,1.0));
       
       // Changed by Tim. p3.3.15
       //outBuffers = new float*[MAX_CHANNELS];
       //for (int i = 0; i < MAX_CHANNELS; ++i)
       //      outBuffers[i] = new float[segmentSize];
-      for (int i = 0; i < MAX_CHANNELS; ++i)
-            posix_memalign((void**)(outBuffers + i), 16, sizeof(float) * segmentSize);
+      //for (int i = 0; i < MAX_CHANNELS; ++i)
+      //      posix_memalign((void**)(outBuffers + i), 16, sizeof(float) * segmentSize);
+      
       // Let's allocate it all in one block, and just point the remaining buffer pointers into the block
       //  which allows faster one-shot buffer copying.
       // Nope. Nice but interferes with possibility we don't know if other buffers are contiguous (jack buffers, local stack buffers etc.).
@@ -88,6 +95,18 @@ AudioTrack::AudioTrack(TrackType t)
       //for (int i = 0; i < MAX_CHANNELS; ++i)
       //  *(outBuffers + i) = sizeof(float) * segmentSize * i;
             
+      // p3.3.38
+      // Easy way, less desirable... Start out with enough for MAX_CHANNELS. Then multi-channel syntis can re-allocate, 
+      //  via a call to (a modified!) setChannels().
+      // Hard way, more desirable... Creating a synti instance passes the total channels to this constructor, overriding MAX_CHANNELS.
+      _totalOutChannels = MAX_CHANNELS;
+      outBuffers = new float*[_totalOutChannels];
+      for (int i = 0; i < _totalOutChannels; ++i)
+            posix_memalign((void**)&outBuffers[i], 16, sizeof(float) * segmentSize);
+      
+      // This is only set by multi-channel syntis...
+      _totalInChannels = 0;
+      
       bufferPos = MAXINT;
       
       setVolume(1.0);
@@ -98,6 +117,7 @@ AudioTrack::AudioTrack(TrackType t)
 AudioTrack::AudioTrack(const AudioTrack& t, bool cloneParts)
   : Track(t, cloneParts)
       {
+      _totalOutChannels = t._totalOutChannels; // Is either MAX_CHANNELS, or custom value (used by syntis).
       _processed      = false;
       _haveData       = false;
       _sendMetronome  = t._sendMetronome;
@@ -112,8 +132,18 @@ AudioTrack::AudioTrack(const AudioTrack& t, bool cloneParts)
       //outBuffers = new float*[MAX_CHANNELS];
       //for (int i = 0; i < MAX_CHANNELS; ++i)
       //      outBuffers[i] = new float[segmentSize];
-      for (int i = 0; i < MAX_CHANNELS; ++i)
-            posix_memalign((void**)(outBuffers + i), 16, sizeof(float) * segmentSize);
+      //for (int i = 0; i < MAX_CHANNELS; ++i)
+      //      posix_memalign((void**)(outBuffers + i), 16, sizeof(float) * segmentSize);
+      
+      // p3.3.38
+      int chans = _totalOutChannels;
+      // Number of allocated buffers is always MAX_CHANNELS or more, even if _totalOutChannels is less. 
+      if(chans < MAX_CHANNELS)
+        chans = MAX_CHANNELS;
+      outBuffers = new float*[chans];
+      for (int i = 0; i < chans; ++i)
+            posix_memalign((void**)&outBuffers[i], 16, sizeof(float) * segmentSize);
+      
       bufferPos = MAXINT;
       _recFile  = t._recFile;
       }
@@ -124,11 +154,26 @@ AudioTrack::~AudioTrack()
       //for (int i = 0; i < MAX_CHANNELS; ++i)
       //      delete[] outBuffers[i];
       //delete[] outBuffers;
-      for(int i = 0; i < MAX_CHANNELS; ++i) 
+      
+      // p3.3.15
+      //for(int i = 0; i < MAX_CHANNELS; ++i) 
+      //{
+      //  if(outBuffers[i])
+      //    free(outBuffers[i]);
+      //}
+      
+      // p3.3.38
+      int chans = _totalOutChannels;
+      // Number of allocated buffers is always MAX_CHANNELS or more, even if _totalOutChannels is less. 
+      if(chans < MAX_CHANNELS)
+        chans = MAX_CHANNELS;
+      for(int i = 0; i < chans; ++i) 
       {
         if(outBuffers[i])
           free(outBuffers[i]);
       }
+      delete[] outBuffers;
+      
 }
 
 //---------------------------------------------------------
@@ -166,47 +211,51 @@ Part* AudioTrack::newPart(Part*, bool /*clone*/)
 //---------------------------------------------------------
 
 void AudioTrack::addPlugin(PluginI* plugin, int idx)
+{
+  if (plugin == 0) 
+  {
+    PluginI* oldPlugin = (*_efxPipe)[idx];
+    if (oldPlugin) 
+    {
+      oldPlugin->setID(-1);
+      oldPlugin->setTrack(0);
+      
+      int controller = oldPlugin->parameters();
+      for (int i = 0; i < controller; ++i) 
       {
-      if (plugin == 0) {
-            PluginI* oldPlugin = (*_efxPipe)[idx];
-            if (oldPlugin) {
-                  
-                  oldPlugin->setID(-1);
-                  oldPlugin->setTrack(0);
-                  
-                  int controller = oldPlugin->parameters();
-                  for (int i = 0; i < controller; ++i) {
-                        int id = genACnum(idx, i);
-                        removeController(id);
-                        }
-                  }
-            }
-      efxPipe()->insert(plugin, idx);
-      if (plugin) {
-            plugin->setID(idx);
-            plugin->setTrack(this);
-                  
-            int controller = plugin->parameters();
-            for (int i = 0; i < controller; ++i) {
-                  int id = genACnum(idx, i);
-                  const char* name = plugin->paramName(i);
-                  float min, max;
-                  plugin->range(i, &min, &max);
-                  CtrlValueType t = plugin->valueType();
-                  CtrlList* cl = new CtrlList(id);
-                  cl->setRange(min, max);
-                  cl->setName(QString(name));
-                  cl->setValueType(t);
-                  LADSPA_PortRangeHint range = plugin->range(i);
-                  if(LADSPA_IS_HINT_TOGGLED(range.HintDescriptor))
-                    cl->setMode(CtrlList::DISCRETE);
-                  else  
-                    cl->setMode(CtrlList::INTERPOLATE);
-                  cl->setCurVal(plugin->param(i));
-                  addController(cl);
-                  }
-            }
+        int id = genACnum(idx, i);
+        removeController(id);
       }
+    }
+  }
+  efxPipe()->insert(plugin, idx);
+  if (plugin) 
+  {
+    plugin->setID(idx);
+    plugin->setTrack(this);
+          
+    int controller = plugin->parameters();
+    for (int i = 0; i < controller; ++i) 
+    {
+      int id = genACnum(idx, i);
+      const char* name = plugin->paramName(i);
+      float min, max;
+      plugin->range(i, &min, &max);
+      CtrlValueType t = plugin->valueType();
+      CtrlList* cl = new CtrlList(id);
+      cl->setRange(min, max);
+      cl->setName(QString(name));
+      cl->setValueType(t);
+      LADSPA_PortRangeHint range = plugin->range(i);
+      if(LADSPA_IS_HINT_TOGGLED(range.HintDescriptor))
+        cl->setMode(CtrlList::DISCRETE);
+      else  
+        cl->setMode(CtrlList::INTERPOLATE);
+      cl->setCurVal(plugin->param(i));
+      addController(cl);
+    }
+  }
+}
 
 //---------------------------------------------------------
 //   addAuxSend
@@ -879,13 +928,14 @@ void AudioTrack::readAuxSend(Xml& xml)
 
 bool AudioTrack::readProperties(Xml& xml, const QString& tag)
       {
-      if (tag == "plugin") {
+      if (tag == "plugin") 
+      {
             int rackpos;
             for(rackpos = 0; rackpos < PipelineDepth; ++rackpos) 
             {
               if(!(*_efxPipe)[rackpos]) 
-                              break;
-                              }
+                break;
+            }
             if(rackpos < PipelineDepth)
             {
               PluginI* pi = new PluginI();
@@ -895,10 +945,10 @@ bool AudioTrack::readProperties(Xml& xml, const QString& tag)
                 delete pi;
               else 
                 (*_efxPipe)[rackpos] = pi;
-                        }
+            }
             else
               printf("can't load plugin - plugin rack is already full\n");
-            }
+      }
       else if (tag == "auxSend")
             readAuxSend(xml);
       else if (tag == "prefader")
@@ -964,6 +1014,25 @@ bool AudioTrack::readProperties(Xml& xml, const QString& tag)
       return false;
       }
 
+//---------------------------------------------------------
+//   showPendingPluginNativeGuis
+//   This is needed because OSC needs all tracks with plugins to be already
+//    added to their track lists so it can find them and show their native guis.
+//---------------------------------------------------------
+
+void AudioTrack::showPendingPluginNativeGuis()
+{
+  for(int idx = 0; idx < PipelineDepth; ++idx)
+  {
+    PluginI* p = (*_efxPipe)[idx];
+    if(!p)
+      continue;
+    
+    if(p->isShowNativeGuiPending())
+      p->showNativeGui(true);
+  }      
+}
+
 //---------------------------------------------------------
 //   mapRackPluginsToControllers
 //---------------------------------------------------------
@@ -1109,12 +1178,13 @@ void AudioTrack::mapRackPluginsToControllers()
     */
 }
 
+/*
 //---------------------------------------------------------
 //   writeRouting
 //---------------------------------------------------------
 
 void AudioTrack::writeRouting(int level, Xml& xml) const
-      {
+{
       QString n;
       if (type() == Track::AUDIO_INPUT) {
                 ciJackRouteNameCache circ = jackRouteNameCache.find(this);
@@ -1168,7 +1238,9 @@ void AudioTrack::writeRouting(int level, Xml& xml) const
             }
         }  
       }  
-    }
+}
+*/       
+       
 //---------------------------------------------------------
 //   AudioInput
 //---------------------------------------------------------
