@@ -1,7 +1,7 @@
 //=========================================================
 //  MusE
 //  Linux Music Editor
-//  $Id: cliplist.cpp,v 1.6.2.1 2005/12/11 21:29:23 spamatica Exp $
+//  $Id: cliplist.cpp,v 1.6.2.3 2008/08/18 00:15:24 terminator356 Exp $
 //
 //  (C) Copyright 2000 Werner Schweer (ws@seh.de)
 //=========================================================
@@ -155,8 +155,12 @@ void ClipListEdit::closeEvent(QCloseEvent* e)
 //   songChanged
 //---------------------------------------------------------
 
-void ClipListEdit::songChanged(int)
+void ClipListEdit::songChanged(int type)
       {
+      // Is it simply a midi controller value adjustment? Forget it.
+      if(type == SC_MIDI_CONTROLLER)
+        return;
+    
       updateList();
       }
 
@@ -202,7 +206,7 @@ void ClipListEdit::writeStatus(int level, Xml& xml) const
 //   startChanged
 //---------------------------------------------------------
 
-void ClipListEdit::startChanged(const Pos& pos)
+void ClipListEdit::startChanged(const Pos& /*pos*/)//prevent compiler warning: unsused parameter
       {
 //      editor->view->triggerUpdate();
       }
@@ -211,7 +215,7 @@ void ClipListEdit::startChanged(const Pos& pos)
 //   lenChanged
 //---------------------------------------------------------
 
-void ClipListEdit::lenChanged(const Pos& pos)
+void ClipListEdit::lenChanged(const Pos& /*pos*/) //prevent compiler warning: unsused parameter
       {
 //      curClip.setLenFrame(pos.frame());
 //      editor->view->triggerUpdate();
@@ -223,7 +227,8 @@ void ClipListEdit::lenChanged(const Pos& pos)
 
 void ClipListEdit::clipSelectionChanged()
       {
-      ClipItem* item = (ClipItem*)(editor->view->selectedItem());
+//      ClipItem* item = (ClipItem*)(editor->view->selectedItem());
+
 //      if (item == 0) {
             editor->start->setEnabled(false);
             editor->len->setEnabled(false);
