@@ -40,6 +40,7 @@
 #include "midiport.h"
 #include "mididev.h"
 #include "driver/audiodev.h"
+#include "driver/jackmidi.h"
 #include "xml.h"
 #include "waveedit.h"
 #include "midi.h"
@@ -216,6 +217,7 @@ static void readConfigMidiPort(Xml& xml)
       int openFlags = 1;
       bool thruFlag = false;
       MidiSyncInfo tmpSi;
+      int type = MidiDevice::ALSA_MIDI;
 
       for (;;) {
             Xml::Token token = xml.parse();
@@ -226,6 +228,8 @@ static void readConfigMidiPort(Xml& xml)
                   case Xml::TagStart:
                         if (tag == "name")
                               device = xml.parse1();
+                        else if (tag == "type")
+                              type = xml.parseInt();
                         else if (tag == "record") {         // old
                               bool f = xml.parseInt();
                               if (f)
@@ -262,9 +266,27 @@ static void readConfigMidiPort(Xml& xml)
                                        idx, MIDI_PORTS);
                                     idx = 0;
                                     }
+                              
                               MidiDevice* dev = midiDevices.find(device);
+                              
+                              //if(debugMsg && !dev)
+                              //  fprintf(stderr, "readConfigMidiPort: device not found %s\n", device.latin1());
+                                
+                              if(!dev && type == MidiDevice::JACK_MIDI)
+                              {
+                                if(debugMsg)
+                                  fprintf(stderr, "readConfigMidiPort: creating jack midi device %s\n", device.latin1());
+                                dev = MidiJackDevice::createJackMidiDevice(device, openFlags);
+                              }
+                              
+                              if(debugMsg && !dev)
+                                fprintf(stderr, "readConfigMidiPort: device not found %s\n", device.latin1());
+                              
                               MidiPort* mp = &midiPorts[idx];
                               mp->syncInfo().copyParams(tmpSi);
+                              // p3.3.50 Indicate the port was found in the song file, even if no device is assigned to it.
+                              mp->setFoundInSongFile(true);
+                              
                               if (dev) {
                                     dev->setOpenFlags(openFlags);
                                     midiSeq->msgSetMidiDevice(mp, dev);
@@ -397,6 +419,8 @@ static void loadConfigMetronom(Xml& xml)
                               midiClickFlag = xml.parseInt();
                         else if (tag == "audioClickEnable")
                               audioClickFlag = xml.parseInt();
+                        else if (tag == "audioClickVolume")
+                              audioClickVolume = xml.parseFloat();
                         else
                               xml.unknown("Metronome");
                         break;
@@ -455,6 +479,7 @@ static void readSeqConfiguration(Xml& xml)
 
 void readConfiguration(Xml& xml, bool readOnlySequencer)
       {
+      int mixers = 0;
       for (;;) {
             Xml::Token token = xml.parse();
             if (token == Xml::Error || token == Xml::End)
@@ -484,6 +509,8 @@ void readConfiguration(Xml& xml, bool readOnlySequencer)
                               config.style = xml.parse1();
                         else if (tag == "useOldStyleStopShortCut")
                               config.useOldStyleStopShortCut = xml.parseInt();
+                        else if (tag == "moveArmedCheckBox")
+                              config.moveArmedCheckBox = xml.parseInt();
                         else if (tag == "externalWavEditor")
                               config.externalWavEditor = xml.parse1();
                         else if (tag == "font0")
@@ -570,6 +597,8 @@ void readConfiguration(Xml& xml, bool readOnlySequencer)
                               config.exp2ByteTimeSigs = xml.parseInt();
                         else if (tag == "expOptimNoteOffs")
                               config.expOptimNoteOffs = xml.parseInt();
+                        else if (tag == "importMidiSplitParts")
+                              config.importMidiSplitParts = xml.parseInt();
                         else if (tag == "midiInputDevice")
                               midiInputPorts = xml.parseInt();
                         else if (tag == "midiInputChannel")
@@ -592,8 +621,15 @@ void readConfiguration(Xml& xml, bool readOnlySequencer)
                               config.transportVisible = xml.parseInt();
                         else if (tag == "markerVisible")
                               config.markerVisible = xml.parseInt();
+                        
                         else if (tag == "mixerVisible")
-                              config.mixerVisible = xml.parseInt();
+                              // config.mixerVisible = xml.parseInt();  // Obsolete
+                              xml.skip(tag);
+                        else if (tag == "mixer1Visible")
+                              config.mixer1Visible = xml.parseInt();
+                        else if (tag == "mixer2Visible")
+                              config.mixer2Visible = xml.parseInt();
+                        
                         else if (tag == "showSplashScreen")
                               config.showSplashScreen = xml.parseInt();
                         else if (tag == "canvasShowPartType")
@@ -614,8 +650,23 @@ void readConfiguration(Xml& xml, bool readOnlySequencer)
                               config.geometryPianoroll = readGeometry(xml, tag);
                         else if (tag == "geometryDrumedit")
                               config.geometryDrumedit = readGeometry(xml, tag);
+                        
                         else if (tag == "geometryMixer")
-                              config.geometryMixer = readGeometry(xml, tag);
+                              // config.geometryMixer = readGeometry(xml, tag); // Obsolete
+                              xml.skip(tag);
+                        //else if (tag == "mixer1")
+                        //      config.mixer1.read(xml);
+                        //else if (tag == "mixer2")
+                        //      config.mixer2.read(xml);
+                        else if (tag == "Mixer")
+                        {
+                              if(mixers == 0)
+                                config.mixer1.read(xml);
+                              else  
+                                config.mixer2.read(xml);
+                              ++mixers;
+                        }
+                        
                         else if (tag == "bigtimeForegroundcolor")
                               config.bigTimeForegroundColor = readColor(xml);
                         else if (tag == "bigtimeBackgroundcolor")
@@ -636,10 +687,20 @@ void readConfiguration(Xml& xml, bool readOnlySequencer)
                                 xml.parseInt();
                         else if (tag == "mtctype")
                               mtcType= xml.parseInt();
+                        else if (tag == "sendClockDelay")
+                              syncSendFirstClockDelay = xml.parseUInt();
                         else if (tag == "extSync")
                               extSyncFlag.setValue(xml.parseInt());
                         else if (tag == "useJackTransport")
-                              useJackTransport = xml.parseInt();
+                              {
+                              useJackTransport.setValue(xml.parseInt());
+                              }
+                        else if (tag == "jackTransportMaster")
+                              {
+                                jackTransportMaster = xml.parseInt();
+                                if(audioDevice)
+                                  audioDevice->setMaster(jackTransportMaster);      
+                              }  
                         else if (tag == "syncgentype") {
                               // for compatibility
                               //int syncGenType= xml.parseInt();
@@ -716,6 +777,10 @@ void readConfiguration(Xml& xml, bool readOnlySequencer)
                               config.useOutputLimiter = xml.parseInt();
                         else if (tag == "vstInPlace")
                               config.vstInPlace = xml.parseInt();
+                        else if (tag == "dummyAudioSampleRate")
+                              config.dummyAudioSampleRate = xml.parseInt();
+                        else if (tag == "dummyAudioBufSize")
+                              config.dummyAudioBufSize = xml.parseInt();
                         else if (tag == "guiRefresh")
                               config.guiRefresh = xml.parseInt();
                         else if (tag == "helpBrowser")
@@ -874,6 +939,7 @@ static void writeSeqConfiguration(int level, Xml& xml, bool writePortInfo)
       xml.intTag(level, "preroll", precountPreroll);
       xml.intTag(level, "midiClickEnable", midiClickFlag);
       xml.intTag(level, "audioClickEnable", audioClickFlag);
+      xml.floatTag(level, "audioClickVolume", audioClickVolume);
       xml.tag(level--, "/metronom");
 
       xml.intTag(level, "rcEnable",   rcEnable);
@@ -905,6 +971,13 @@ static void writeSeqConfiguration(int level, Xml& xml, bool writePortInfo)
                   xml.strTag(level, "instrument", mport->instrument()->iname());
                   if (dev) {
                         xml.strTag(level, "name",   dev->name());
+                        
+                        // p3.3.38
+                        //if(dynamic_cast<MidiJackDevice*>(dev))
+                        if(dev->deviceType() != MidiDevice::ALSA_MIDI)
+                          //xml.intTag(level, "type", MidiDevice::JACK_MIDI);
+                          xml.intTag(level, "type", dev->deviceType());
+                        
                         // Changed by T356. "record" is old and by mistake written as rwFlags here. 
                         // openFlags was read before, but never written here.
                         //xml.intTag(level, "record", dev->rwFlags() & 0x2 ? 1 : 0);
@@ -968,6 +1041,8 @@ void MusE::writeGlobalConfiguration(int level, Xml& xml) const
       xml.intTag(level, "didYouKnow", config.showDidYouKnow);
       xml.intTag(level, "outputLimiter", config.useOutputLimiter);
       xml.intTag(level, "vstInPlace", config.vstInPlace);
+      xml.intTag(level, "dummyAudioBufSize", config.dummyAudioBufSize);
+      xml.intTag(level, "dummyAudioSampleRate", config.dummyAudioSampleRate);
 
       xml.intTag(level, "guiRefresh", config.guiRefresh);
       xml.strTag(level, "helpBrowser", config.helpBrowser);
@@ -978,6 +1053,7 @@ void MusE::writeGlobalConfiguration(int level, Xml& xml) const
       xml.intTag(level, "smfFormat", config.smfFormat);
       xml.intTag(level, "exp2ByteTimeSigs", config.exp2ByteTimeSigs);
       xml.intTag(level, "expOptimNoteOffs", config.expOptimNoteOffs);
+      xml.intTag(level, "importMidiSplitParts", config.importMidiSplitParts);
       xml.intTag(level, "startMode", config.startMode);
       xml.strTag(level, "startSong", config.startSong);
 
@@ -996,6 +1072,7 @@ void MusE::writeGlobalConfiguration(int level, Xml& xml) const
       xml.strTag(level, "theme", config.style);
       xml.strTag(level, "externalWavEditor", config.externalWavEditor);
       xml.intTag(level, "useOldStyleStopShortCut", config.useOldStyleStopShortCut);
+      xml.intTag(level, "moveArmedCheckBox", config.moveArmedCheckBox);
 
       //for (int i = 0; i < 6; ++i) {
       for (int i = 0; i < NUM_FONTS; ++i) {
@@ -1060,7 +1137,9 @@ void MusE::writeGlobalConfiguration(int level, Xml& xml) const
       xml.nput(level, "<mtcoffset>%02d:%02d:%02d:%02d:%02d</mtcoffset>\n",
         mtcOffset.h(), mtcOffset.m(), mtcOffset.s(),
         mtcOffset.f(), mtcOffset.sf());
+      //xml.uintTag(level, "sendClockDelay", syncSendFirstClockDelay);
       //xml.intTag(level, "useJackTransport", useJackTransport);
+      //xml.intTag(level, "jackTransportMaster", jackTransportMaster);
       extSyncFlag.save(level, xml);
       
 //      xml.intTag(level, "genMTCSync", genMTCSync);
@@ -1075,11 +1154,18 @@ void MusE::writeGlobalConfiguration(int level, Xml& xml) const
       xml.qrectTag(level, "geometryBigTime",   config.geometryBigTime);
       xml.qrectTag(level, "geometryPianoroll", config.geometryPianoroll);
       xml.qrectTag(level, "geometryDrumedit",  config.geometryDrumedit);
-      xml.qrectTag(level, "geometryMixer",     config.geometryMixer);
+      //xml.qrectTag(level, "geometryMixer",     config.geometryMixer);  // Obsolete
 
       xml.intTag(level, "bigtimeVisible", config.bigTimeVisible);
       xml.intTag(level, "transportVisible", config.transportVisible);
-      xml.intTag(level, "mixerVisible", config.mixerVisible);
+      
+      //xml.intTag(level, "mixerVisible", config.mixerVisible);  // Obsolete
+      xml.intTag(level, "mixer1Visible", config.mixer1Visible);
+      xml.intTag(level, "mixer2Visible", config.mixer2Visible);
+      //config.mixer1.write(level, xml, "mixer1");
+      //config.mixer2.write(level, xml, "mixer2");
+      config.mixer1.write(level, xml);
+      config.mixer2.write(level, xml);
 
       xml.intTag(level, "showSplashScreen", config.showSplashScreen);
       xml.intTag(level, "canvasShowPartType", config.canvasShowPartType);
@@ -1162,7 +1248,9 @@ void MusE::writeConfiguration(int level, Xml& xml) const
       xml.nput(level, "<mtcoffset>%02d:%02d:%02d:%02d:%02d</mtcoffset>\n",
         mtcOffset.h(), mtcOffset.m(), mtcOffset.s(),
         mtcOffset.f(), mtcOffset.sf());
-      xml.intTag(level, "useJackTransport", useJackTransport);
+      xml.uintTag(level, "sendClockDelay", syncSendFirstClockDelay);
+      xml.intTag(level, "useJackTransport", useJackTransport.value());
+      xml.intTag(level, "jackTransportMaster", jackTransportMaster);
       extSyncFlag.save(level, xml);
       
 //      xml.intTag(level, "genMTCSync", genMTCSync);
@@ -1175,15 +1263,24 @@ void MusE::writeConfiguration(int level, Xml& xml) const
       xml.intTag(level, "bigtimeVisible",   menuView->isItemChecked(bt_id));
       xml.intTag(level, "transportVisible", menuView->isItemChecked(tr_id));
       xml.intTag(level, "markerVisible",     menuView->isItemChecked(mr_id));
-      xml.intTag(level, "mixerVisible",     menuView->isItemChecked(aid1));
+      //xml.intTag(level, "mixerVisible",     menuView->isItemChecked(aid1));  // Obsolete
 
       xml.geometryTag(level, "geometryMain", this);
       if (transport)
             xml.geometryTag(level, "geometryTransport", transport);
       if (bigtime)
             xml.geometryTag(level, "geometryBigTime", bigtime);
-      if (audioMixer)
-            xml.geometryTag(level, "geometryMixer", audioMixer);
+      
+      //if (audioMixer)
+      //      xml.geometryTag(level, "geometryMixer", audioMixer);   // Obsolete
+      xml.intTag(level, "mixer1Visible",    menuView->isItemChecked(aid1a));
+      xml.intTag(level, "mixer2Visible",    menuView->isItemChecked(aid1b));
+      if (mixer1)
+            //mixer1->write(level, xml, "mixer1");
+            mixer1->write(level, xml);
+      if (mixer2)
+            //mixer2->write(level, xml, "mixer2");
+            mixer2->write(level, xml);
 
       arranger->writeStatus(level, xml);
       writeSeqConfiguration(level, xml, true);
@@ -1264,6 +1361,7 @@ void MidiFileConfig::updateValues()
       copyrightEdit->setText(config.copyright);
       optNoteOffs->setChecked(config.expOptimNoteOffs);
       twoByteTimeSigs->setChecked(config.exp2ByteTimeSigs);
+      splitPartsCheckBox->setChecked(config.importMidiSplitParts);
       }
 
 //---------------------------------------------------------
@@ -1282,6 +1380,7 @@ void MidiFileConfig::okClicked()
       config.copyright    = copyrightEdit->text();
       config.expOptimNoteOffs = optNoteOffs->isChecked();
       config.exp2ByteTimeSigs = twoByteTimeSigs->isChecked();
+      config.importMidiSplitParts = splitPartsCheckBox->isChecked();
 
       muse->changeConfig(true);  // write config file
       close();
@@ -1314,3 +1413,87 @@ void MusE::configGlobalSettings()
       }
 
 
+//---------------------------------------------------------
+//   write
+//---------------------------------------------------------
+
+//void MixerConfig::write(Xml& xml, const char* name)
+void MixerConfig::write(int level, Xml& xml)
+//void MixerConfig::write(int level, Xml& xml, const char* name)
+      {
+      //xml.stag(QString(name));
+      //xml.tag(level++, name.latin1());
+      xml.tag(level++, "Mixer");
+      //xml.tag(level++, name);
+      
+      xml.strTag(level, "name", name);
+      
+      //xml.tag("geometry",       geometry);
+      xml.qrectTag(level, "geometry", geometry);
+      
+      xml.intTag(level, "showMidiTracks",   showMidiTracks);
+      xml.intTag(level, "showDrumTracks",   showDrumTracks);
+      xml.intTag(level, "showInputTracks",  showInputTracks);
+      xml.intTag(level, "showOutputTracks", showOutputTracks);
+      xml.intTag(level, "showWaveTracks",   showWaveTracks);
+      xml.intTag(level, "showGroupTracks",  showGroupTracks);
+      xml.intTag(level, "showAuxTracks",    showAuxTracks);
+      xml.intTag(level, "showSyntiTracks",  showSyntiTracks);
+      
+      //xml.etag(name);
+      //xml.etag(level, name.latin1());
+      xml.etag(level, "Mixer");
+      //xml.etag(level, name);
+      }
+
+//---------------------------------------------------------
+//   read
+//---------------------------------------------------------
+
+//void MixerConfig::read(QDomNode node)
+void MixerConfig::read(Xml& xml)
+//void MixerConfig::read(Xml& xml, const QString& name)
+      {
+      for (;;) {
+            Xml::Token token(xml.parse());
+            const QString& tag(xml.s1());
+            switch (token) {
+                  case Xml::Error:
+                  case Xml::End:
+                        return;
+                  case Xml::TagStart:
+                        if (tag == "name")
+                              name = xml.parse1();
+                        else if (tag == "geometry")
+                              geometry = readGeometry(xml, tag);
+                        else if (tag == "showMidiTracks")
+                              showMidiTracks = xml.parseInt();
+                        else if (tag == "showDrumTracks")
+                              showDrumTracks = xml.parseInt();
+                        else if (tag == "showInputTracks")
+                              showInputTracks = xml.parseInt();
+                        else if (tag == "showOutputTracks")
+                              showOutputTracks = xml.parseInt();
+                        else if (tag == "showWaveTracks")
+                              showWaveTracks = xml.parseInt();
+                        else if (tag == "showGroupTracks")
+                              showGroupTracks = xml.parseInt();
+                        else if (tag == "showAuxTracks")
+                              showAuxTracks = xml.parseInt();
+                        else if (tag == "showSyntiTracks")
+                              showSyntiTracks = xml.parseInt();
+                        else
+                              //xml.unknown(name.latin1());
+                              xml.unknown("Mixer");
+                        break;
+                  case Xml::TagEnd:
+                        //if (tag == name)
+                        if (tag == "Mixer")
+                            return;
+                  default:
+                        break;
+                  }
+            }
+      
+      }
+
