@@ -11,6 +11,7 @@
 
 #include <qwidget.h>
 #include <qwhatsthis.h>
+#include <qtooltip.h>
 
 #include "synthconfigbase.h"
 
@@ -21,6 +22,19 @@ class QPopupMenu;
 class QHeader;
 class Xml;
 
+//----------------------------------------------------------
+//   MPHeaderTip
+//----------------------------------------------------------
+
+class MPHeaderTip : public QToolTip {
+
+   public:
+      MPHeaderTip(QWidget * parent) : QToolTip(parent) {}
+      virtual ~MPHeaderTip() {}
+   protected:
+      void maybeTip(const QPoint &);
+      };
+
 //---------------------------------------------------------
 //   MPWhatsThis
 //---------------------------------------------------------
@@ -43,13 +57,17 @@ class MPWhatsThis : public QWhatsThis {
 //---------------------------------------------------------
 
 class MPConfig : public SynthConfigBase {
+      MPHeaderTip* _mptooltip;
       QPopupMenu* popup;
       QPopupMenu* instrPopup;
-
+      
+      int _showAliases; // -1: None. 0: First aliases. 1: Second aliases etc.
+      
       Q_OBJECT
 
    private slots:
       void rbClicked(QListViewItem*, const QPoint&,int);
+      void mdevViewItemRenamed(QListViewItem*, int, const QString&);
       void songChanged(int);
       void selectionChanged();
       void addInstanceClicked();
@@ -57,6 +75,7 @@ class MPConfig : public SynthConfigBase {
 
    public:
       MPConfig(QWidget* parent, char* name);
+      ~MPConfig();
       };
 
 #endif
