@@ -17,6 +17,9 @@
 #include "../audio.h"
 #include "mpevent.h"
 //#include "sync.h"
+#include "utils.h"
+#include "audiodev.h"
+#include "xml.h"
 
 static int alsaSeqFdi = -1;
 static int alsaSeqFdo = -1;
@@ -167,6 +170,109 @@ void MidiAlsaDevice::close()
       }
 }
 
+//---------------------------------------------------------
+//   writeRouting
+//---------------------------------------------------------
+
+void MidiAlsaDevice::writeRouting(int level, Xml& xml) const
+{
+      // p3.3.45
+      // If this device is not actually in use by the song, do not write any routes.
+      // This prevents bogus routes from being saved and propagated in the med file.
+      if(midiPort() == -1)
+        return;
+     
+      QString s;
+      /*
+      //if(rwFlags() & 2)  // Readable
+      {
+        //RouteList* rl = _inRoutes;
+        //for (ciRoute r = rl->begin(); r != rl->end(); ++r) 
+        for (ciRoute r = _inRoutes.begin(); r != _inRoutes.end(); ++r) 
+        {
+          // Since an ALSA midi device supports read + write, this is the only way we can tell if this route is using the device as input.  
+          if(r->type == Route::TRACK_ROUTE)
+            continue;
+            
+          if(!r->name().isEmpty())
+          {
+            xml.tag(level++, "Route");
+            
+            //xml.strTag(level, "srcNode", r->name());
+            xml.tag(level, "source type=\"%d\" name=\"%s\"/", r->type, r->name().latin1());
+            
+            //xml.strTag(level, "dstNode", name());
+            xml.tag(level, "dest type=\"%d\" name=\"%s\"/", Route::ALSA_MIDI_ROUTE, name().latin1());
+            
+            xml.etag(level--, "Route");
+          }
+        }  
+      } 
+      */
+      
+      for (ciRoute r = _outRoutes.begin(); r != _outRoutes.end(); ++r) 
+      {
+        //if(r->type != Route::TRACK_ROUTE)
+        //{  
+        //  printf("MidiAlsaDevice::writeRouting Warning out route is not TRACK_ROUTE type\n");
+        //  continue;
+        //}
+          
+        if(!r->name().isEmpty())
+        {
+          //xml.tag(level++, "Route");
+          
+          s = QT_TR_NOOP("Route");
+          if(r->channel != -1)
+            s += QString(QT_TR_NOOP(" channel=\"%1\"")).arg(r->channel);
+          xml.tag(level++, s);
+          
+          /*
+          //xml.strTag(level, "srcNode", name());
+          if(r->channel != -1)  
+            //xml.tag(level, "source type=\"%d\" channel=\"%d\" name=\"%s\"/", Route::ALSA_MIDI_ROUTE, r->channel, name().latin1());
+            //xml.tag(level, "source type=\"%d\" channel=\"%d\" name=\"%s\"/", Route::MIDI_DEVICE_ROUTE, r->channel, name().latin1());
+            xml.tag(level, "source devtype=\"%d\" channel=\"%d\" name=\"%s\"/", MidiDevice::ALSA_MIDI, r->channel, name().latin1());
+          else  
+            //xml.tag(level, "source type=\"%d\" name=\"%s\"/", Route::ALSA_MIDI_ROUTE, name().latin1());
+            //xml.tag(level, "source type=\"%d\" name=\"%s\"/", Route::MIDI_DEVICE_ROUTE, name().latin1());
+          */  
+            //xml.tag(level, "source devtype=\"%d\" name=\"%s\"/", MidiDevice::ALSA_MIDI, name().latin1());
+            xml.tag(level, "source devtype=\"%d\" name=\"%s\"/", MidiDevice::ALSA_MIDI, Xml::xmlString(name()).latin1());
+          
+          /*
+          //xml.strTag(level, "dstNode", r->name());
+          if(r->channel != -1)  
+          {
+            if(r->type == Route::MIDI_DEVICE_ROUTE)  
+              xml.tag(level, "dest devtype=\"%d\" channel=\"%d\" name=\"%s\"/", r->device->deviceType(), r->channel, r->name().latin1());
+            else  
+              xml.tag(level, "dest type=\"%d\" channel=\"%d\" name=\"%s\"/", r->type, r->channel, r->name().latin1());
+          }
+          else  
+          {
+            if(r->type == Route::MIDI_DEVICE_ROUTE)  
+              xml.tag(level, "dest devtype=\"%d\" name=\"%s\"/", r->device->deviceType(), r->name().latin1());
+            else  
+              xml.tag(level, "dest type=\"%d\" name=\"%s\"/", r->type, r->name().latin1());
+          }
+          */
+          
+          s = QT_TR_NOOP("dest");
+          if(r->type == Route::MIDI_DEVICE_ROUTE)
+            s += QString(QT_TR_NOOP(" devtype=\"%1\"")).arg(r->device->deviceType());
+          else
+          if(r->type != Route::TRACK_ROUTE)
+            s += QString(QT_TR_NOOP(" type=\"%1\"")).arg(r->type);
+          //s += QString(QT_TR_NOOP(" name=\"%1\"/")).arg(r->name());
+          s += QString(QT_TR_NOOP(" name=\"%1\"/")).arg(Xml::xmlString(r->name()));
+          xml.tag(level, s);
+          
+          xml.etag(level--, "Route");
+        }
+      }
+}
+    
 //---------------------------------------------------------
 //   putEvent
 //---------------------------------------------------------
@@ -205,17 +311,21 @@ bool MidiAlsaDevice::putMidiEvent(const MidiPlayEvent& e)
                   int a   = e.dataA();
                   int b   = e.dataB();
                   int chn = e.channel();
-                  if (a < 0x1000) {          // 7 Bit Controller
+                  // p3.3.37
+                  //if (a < 0x1000) {          // 7 Bit Controller
+                  if (a < CTRL_14_OFFSET) {          // 7 Bit Controller
                         snd_seq_ev_set_controller(&event, chn, a, b);
                         }
-                  else if (a < 0x20000) {     // 14 bit high resolution controller
+                  //else if (a < 0x20000) {     // 14 bit high resolution controller
+                  else if (a < CTRL_RPN_OFFSET) {     // 14 bit high resolution controller
                         int ctrlH = (a >> 8) & 0x7f;
                         int ctrlL = a & 0x7f;
                         a = (ctrlH << 7) + ctrlL;
                         snd_seq_ev_set_controller(&event, chn, a, b);
                         event.type = SND_SEQ_EVENT_CONTROL14;
                         }
-                  else if (a < 0x30000) {     // RPN 7-Bit Controller
+                  //else if (a < 0x30000) {     // RPN 7-Bit Controller
+                  else if (a < CTRL_NRPN_OFFSET) {     // RPN 7-Bit Controller
                         int ctrlH = (a >> 8) & 0x7f;
                         int ctrlL = a & 0x7f;
                         a = (ctrlH << 7) + ctrlL;
@@ -223,7 +333,8 @@ bool MidiAlsaDevice::putMidiEvent(const MidiPlayEvent& e)
                         snd_seq_ev_set_controller(&event, chn, a, b);
                         event.type = SND_SEQ_EVENT_REGPARAM;
                         }
-                  else if (a < 0x40000) {     // NRPN 7-Bit Controller
+                  //else if (a < 0x40000) {     // NRPN 7-Bit Controller
+                  else if (a < CTRL_INTERNAL_OFFSET) {     // NRPN 7-Bit Controller
                         int ctrlH = (a >> 8) & 0x7f;
                         int ctrlL = a & 0x7f;
                         a = (ctrlH << 7) + ctrlL;
@@ -231,14 +342,16 @@ bool MidiAlsaDevice::putMidiEvent(const MidiPlayEvent& e)
                         snd_seq_ev_set_controller(&event, chn, a, b);
                         event.type = SND_SEQ_EVENT_NONREGPARAM;
                         }
-                  else if (a < 0x60000) {     // RPN14 Controller
+                  //else if (a < 0x60000) {     // RPN14 Controller
+                  else if (a < CTRL_NRPN14_OFFSET) {     // RPN14 Controller
                         int ctrlH = (a >> 8) & 0x7f;
                         int ctrlL = a & 0x7f;
                         a = (ctrlH << 7) + ctrlL;
                         snd_seq_ev_set_controller(&event, chn, a, b);
                         event.type = SND_SEQ_EVENT_REGPARAM;
                         }
-                  else if (a < 0x70000) {     // NRPN14 Controller
+                  //else if (a < 0x70000) {     // NRPN14 Controller
+                  else if (a < CTRL_NONE_OFFSET) {     // NRPN14 Controller
                         int ctrlH = (a >> 8) & 0x7f;
                         int ctrlL = a & 0x7f;
                         a = (ctrlH << 7) + ctrlL;
@@ -410,7 +523,11 @@ bool initMidiAlsa()
                   
                   }
             }
-      snd_seq_set_client_name(alsaSeq, "MusE Sequencer");
+      
+      // p3.3.38
+      //snd_seq_set_client_name(alsaSeq, "MusE Sequencer");
+      snd_seq_set_client_name(alsaSeq, audioDevice->clientName());
+      
       int ci = snd_seq_poll_descriptors_count(alsaSeq, POLLIN);
       int co = snd_seq_poll_descriptors_count(alsaSeq, POLLOUT);
 
@@ -713,28 +830,39 @@ void alsaProcessMidiInput()
                         break;
 
                   case SND_SEQ_EVENT_CLOCK:
-                        midiSeq->realtimeSystemInput(curPort, 0xf8);
+                        midiSeq->realtimeSystemInput(curPort, ME_CLOCK);
                         //mdev->syncInfo().trigMCSyncDetect();
                         break;
 
                   case SND_SEQ_EVENT_START:
-                        midiSeq->realtimeSystemInput(curPort, 0xfa);
+                        midiSeq->realtimeSystemInput(curPort, ME_START);
                         break;
 
                   case SND_SEQ_EVENT_CONTINUE:
-                        midiSeq->realtimeSystemInput(curPort, 0xfb);
+                        midiSeq->realtimeSystemInput(curPort, ME_CONTINUE);
                         break;
 
                   case SND_SEQ_EVENT_STOP:
-                        midiSeq->realtimeSystemInput(curPort, 0xfc);
+                        midiSeq->realtimeSystemInput(curPort, ME_STOP);
                         break;
 
                   case SND_SEQ_EVENT_TICK:
-                        midiSeq->realtimeSystemInput(curPort, 0xf9);
+                        midiSeq->realtimeSystemInput(curPort, ME_TICK);
                         //mdev->syncInfo().trigTickDetect();
                         break;
 
                   case SND_SEQ_EVENT_SYSEX:
+                        
+                        // TODO: Deal with large sysex, which are broken up into chunks!
+                        // For now, do not accept if the first byte is not SYSEX or the last byte is not EOX, 
+                        //  meaning it's a chunk, possibly with more chunks to follow.
+                        if((*((unsigned char*)ev->data.ext.ptr) != ME_SYSEX) ||
+                           (*(((unsigned char*)ev->data.ext.ptr) + ev->data.ext.len - 1) != ME_SYSEX_END))
+                        {
+                          printf("MusE: alsaProcessMidiInput sysex chunks not supported!\n");
+                          break;
+                        }
+                        
                         event.setTime(0);      // mark as used
                         event.setType(ME_SYSEX);
                         event.setData((unsigned char*)(ev->data.ext.ptr)+1,
@@ -775,9 +903,10 @@ void alsaProcessMidiInput()
             if(event.type())
             {
               mdev->recordEvent(event);
-              if(ev->type != SND_SEQ_EVENT_SYSEX)
+              // p3.3.26 1/23/10 Moved to MidiDevice now. Anticipating Jack midi support, so don't make it ALSA specific. Tim.
+              //if(ev->type != SND_SEQ_EVENT_SYSEX)
                 // Trigger general activity indicator detector. Sysex has no channel, don't trigger.
-                midiPorts[curPort].syncInfo().trigActDetect(event.channel());
+              //  midiPorts[curPort].syncInfo().trigActDetect(event.channel());
             }
                   
             snd_seq_free_event(ev);
