@@ -1,7 +1,7 @@
 //=========================================================
 //  MusE
 //  Linux Music Editor
-//  $Id: alsatimer.h,v 1.1.2.3 2005/08/21 18:11:28 spamatica Exp $
+//  $Id: alsatimer.h,v 1.1.2.4 2009/03/09 02:05:18 terminator356 Exp $
 //
 //  Plenty of code borrowed from timer.c example in 
 //  alsalib 1.0.7
@@ -30,7 +30,7 @@ class AlsaTimer : public Timer{
     char timername[64];
     signed int count;
     unsigned int ticks;
-
+    bool findBest;
 
     public:
        AlsaTimer();
@@ -46,6 +46,7 @@ class AlsaTimer : public Timer{
        virtual bool stopTimer();
        virtual unsigned int getTimerTicks(bool printTicks=false);
        
+       void setFindBestTimer(bool b) { findBest = b; }
 };
 
 #endif //__ALSATIMER_H__
