@@ -1,7 +1,7 @@
   //=========================================================
   //  MusE
   //  Linux Music Editor
-  //  $Id: rtctimer.cpp,v 1.1.2.8 2006/01/13 18:10:55 wschweer Exp $
+  //  $Id: rtctimer.cpp,v 1.1.2.11 2009/03/09 02:05:18 terminator356 Exp $
   //
   //  Most code moved from midiseq.cpp by Werner Schweer.
   //
@@ -51,6 +51,7 @@ signed int RtcTimer::initTimer()
     timerFd = ::open("/dev/rtc", O_RDONLY);
     if (timerFd == -1) {
           fprintf(stderr, "fatal error: open /dev/rtc failed: %s\n", strerror(errno));
+          fprintf(stderr, "hint: check if 'rtc' kernel module is loaded, or used by something else\n");
           undoSetuid();
           return timerFd;
           }
@@ -137,7 +138,7 @@ bool RtcTimer::stopTimer()
     return true;
     }
 
-unsigned int RtcTimer::getTimerTicks(bool printTicks)
+unsigned int RtcTimer::getTimerTicks(bool /*printTicks*/)// prevent compiler warning: unused parameter
     {
     if(TIMER_DEBUG)
       printf("getTimerTicks()\n");
