@@ -29,10 +29,13 @@
 
 static void addController(MPEventList* l, int tick, int port, int channel, int a, int b)
       {
-      if (a < 0x1000) {          // 7 Bit Controller
+      // p3.3.37
+      //if (a < 0x1000) {          // 7 Bit Controller
+      if (a < CTRL_14_OFFSET) {          // 7 Bit Controller
             l->add(MidiPlayEvent(tick, port, channel, ME_CONTROLLER, a, b));
             }
-      else if (a < 0x20000) {     // 14 Bit Controller
+      //else if (a < 0x20000) {     // 14 Bit Controller
+      else if (a < CTRL_RPN_OFFSET) {     // 14 Bit Controller
             int ctrlH = (a >> 8) & 0x7f;
             int ctrlL = a & 0x7f;
             int dataH = (b >> 7) & 0x7f;
@@ -40,14 +43,16 @@ static void addController(MPEventList* l, int tick, int port, int channel, int a
             l->add(MidiPlayEvent(tick, port, channel, ME_CONTROLLER, ctrlH, dataH));
             l->add(MidiPlayEvent(tick+1, port, channel, ME_CONTROLLER, ctrlL, dataL));
             }
-      else if (a < 0x30000) {     // RPN 7-Bit Controller
+      //else if (a < 0x30000) {     // RPN 7-Bit Controller
+      else if (a < CTRL_NRPN_OFFSET) {     // RPN 7-Bit Controller
             int ctrlH = (a >> 8) & 0x7f;
             int ctrlL = a & 0x7f;
             l->add(MidiPlayEvent(tick, port, channel, ME_CONTROLLER, CTRL_HRPN, ctrlH));
             l->add(MidiPlayEvent(tick+1, port, channel, ME_CONTROLLER, CTRL_LRPN, ctrlL));
             l->add(MidiPlayEvent(tick+2, port, channel, ME_CONTROLLER, CTRL_HDATA, b));
             }
-      else if (a < 0x40000) {     // NRPN 7-Bit Controller
+      //else if (a < 0x40000) {     // NRPN 7-Bit Controller
+      else if (a < CTRL_INTERNAL_OFFSET) {     // NRPN 7-Bit Controller
             int ctrlH = (a >> 8) & 0x7f;
             int ctrlL = a & 0x7f;
             l->add(MidiPlayEvent(tick, port, channel, ME_CONTROLLER, CTRL_HNRPN, ctrlH));
@@ -82,7 +87,8 @@ static void addController(MPEventList* l, int tick, int port, int channel, int a
                   }
             l->add(MidiPlayEvent(tick+tickoffset, port, channel, ME_PROGRAM, pr, 0));
             }
-      else if (a < 0x60000) {     // RPN14 Controller
+      //else if (a < 0x60000) {     // RPN14 Controller
+      else if (a < CTRL_NRPN14_OFFSET) {     // RPN14 Controller
             int ctrlH = (a >> 8) & 0x7f;
             int ctrlL = a & 0x7f;
             int dataH = (b >> 7) & 0x7f;
@@ -92,7 +98,8 @@ static void addController(MPEventList* l, int tick, int port, int channel, int a
             l->add(MidiPlayEvent(tick+2, port, channel, ME_CONTROLLER, CTRL_HDATA, dataH));
             l->add(MidiPlayEvent(tick+3, port, channel, ME_CONTROLLER, CTRL_LDATA, dataL));
             }
-      else if (a < 0x70000) {     // NRPN14 Controller
+      //else if (a < 0x70000) {     // NRPN14 Controller
+      else if (a < CTRL_NONE_OFFSET) {     // NRPN14 Controller
             int ctrlH = (a >> 8) & 0x7f;
             int ctrlL = a & 0x7f;
             int dataH = (b >> 7) & 0x7f;
@@ -112,7 +119,8 @@ void MusE::exportMidi()
       {
       MFile file(QString("midis"), QString(".mid"));
 
-      FILE* fp = file.open("w", midi_file_pattern, this, false, true,
+      //FILE* fp = file.open("w", midi_file_pattern, this, false, true,
+      FILE* fp = file.open("w", midi_file_save_pattern, this, false, true,
          tr("MusE: Export Midi"));
       if (fp == 0)
             return;
