@@ -1,7 +1,7 @@
 //=========================================================
 //  MusE
 //  Linux Music Editor
-//    $Id: editevent.cpp,v 1.12.2.3 2005/07/13 21:02:52 spamatica Exp $
+//    $Id: editevent.cpp,v 1.12.2.6 2009/02/02 21:38:00 terminator356 Exp $
 //  (C) Copyright 1999-2004 Werner Schweer (ws@seh.de)
 //=========================================================
 
@@ -23,7 +23,6 @@
 #include "event.h"
 #include "midictrl.h"
 #include "editevent.h"
-#include "pitchlabel.h"
 #include "pitchedit.h"
 #include "intlabel.h"
 #include "globals.h"
@@ -32,6 +31,7 @@
 #include "midiport.h"
 #include "midiedit/drummap.h"
 #include "instruments/minstrument.h"
+#include "midi.h"
 
 //---------------------------------------------------------
 //   string2qhex
@@ -286,8 +286,15 @@ EditMetaDialog::EditMetaDialog(int tick, const Event& ev,
 
       QLabel* l2 = new QLabel(tr("Meta Type"), this);
       il2 = new IntLabel(-1, 0, 127, this, -1);
+      il2->setFixedWidth(100);
       il2->setFrame(true);
       il2->setDark();
+      typeLabel = new QLabel(this);
+      typeLabel->setAlignment(AlignLeft | AlignVCenter);
+      QHBoxLayout* typeLayout = new QHBoxLayout(this);
+      typeLayout->addWidget(il2);
+      typeLayout->addWidget(typeLabel);
+      typeLayout->addStretch();
 
       hexButton = new QRadioButton(tr("Enter Hex"), this, "hextoggle");
       hexButton->setChecked(true);
@@ -307,14 +314,29 @@ EditMetaDialog::EditMetaDialog(int tick, const Event& ev,
             il2->setValue(0);
             }
 
+      typeChanged(il2->value());
+      connect(il2, SIGNAL(valueChanged(int)), SLOT(typeChanged(int)));
+      
       layout1->addWidget(l1,  0, 0);
       layout1->addWidget(pos, 0, 1, AlignLeft);
       layout1->addWidget(l2,  1, 0);
-      layout1->addWidget(il2, 1, 1, AlignLeft);
+      
+      //layout1->addWidget(il2, 1, 1, AlignLeft);
+      layout1->addLayout(typeLayout, 1, 1);
+      
       layout1->addMultiCellWidget(hexButton, 2, 2, 0, 1);
       layout1->addMultiCellWidget(edit, 3, 3, 0, 1);
       }
 
+//---------------------------------------------------------
+//   typeChanged
+//---------------------------------------------------------
+
+void EditMetaDialog::typeChanged(int val)
+{
+  typeLabel->setText(midiMetaName(val));
+}
+
 //---------------------------------------------------------
 //   toggled
 //---------------------------------------------------------
@@ -445,8 +467,7 @@ EditPAfterDialog::EditPAfterDialog(int tick, const Event& event,
       pos = new PosEdit(this);
 
       QLabel* l2 = new QLabel(tr("Pitch"), this);
-      pl = new PitchLabel(this);
-
+      pl = new PitchEdit(this);
       QLabel* l3 = new QLabel(tr("Pressure"), this);
       il2  = new IntLabel(-1, 0, 127, this, -1);
       il2->setFrame(true);
@@ -484,13 +505,12 @@ EditPAfterDialog::EditPAfterDialog(int tick, const Event& event,
 
 Event EditPAfterDialog::event()
       {
-      Event event(CAfter);
+      Event event(PAfter);
       event.setTick(pos->pos().tick());
       event.setA(pl->value());
       event.setB(il2->value());
       return event;
       }
-
 //---------------------------------------------------------
 //   getEvent
 //---------------------------------------------------------
@@ -519,7 +539,7 @@ Event EditCtrlDialog::event()
       if (num == CTRL_PROGRAM)
             event.setB(val);
       else
-            event.setB(valSlider->value());
+            event.setB(valSlider->value() + midiPorts[part->track()->outPort()].midiController(num)->bias());
       return event;
       }
 
@@ -593,22 +613,33 @@ EditCtrlDialog::EditCtrlDialog(int tick, const Event& event,
 
       valSlider->setRange(mc->minVal(), mc->maxVal());
       valSpinBox->setRange(mc->minVal(), mc->maxVal());
+      
       controllerName->setText(mc->name());
 
+      if(!event.empty())
+      {
+        if(num == CTRL_PROGRAM)
+        {
+          widgetStack->raiseWidget(1);
+          updatePatch();
+        }  
+        else  
+        {
+          widgetStack->raiseWidget(0);
+          valSlider->setValue(val - mc->bias());
+        }  
+      }
+      else
+        ctrlListClicked(ctrlList->selectedItem());
       connect(ctrlList, SIGNAL(clicked(QListBoxItem*)), SLOT(ctrlListClicked(QListBoxItem*)));
-      connect(buttonNewController, SIGNAL(pressed()), SLOT(newController()));
+      connect(buttonNewController, SIGNAL(clicked()), SLOT(newController()));
       connect(hbank,   SIGNAL(valueChanged(int)), SLOT(programChanged()));
       connect(lbank,   SIGNAL(valueChanged(int)), SLOT(programChanged()));
       connect(program, SIGNAL(valueChanged(int)), SLOT(programChanged()));
       connect(patchName, SIGNAL(released()), SLOT(instrPopup()));
-
-      ctrlListClicked(ctrlList->selectedItem());
-
       timePos->setValue(tick);
-      if (!event.empty() && num != CTRL_PROGRAM)
-            valSlider->setValue(event.dataB());
+      
       }
-
 //---------------------------------------------------------
 //   newController
 //---------------------------------------------------------
@@ -626,28 +657,48 @@ void EditCtrlDialog::newController()
       MidiPort* port          = &midiPorts[portn];
       MidiInstrument* instr   = port->instrument();
       MidiControllerList* mcl = instr->controller();
+      
+      MidiCtrlValListList* cll = port->controller();
+      int channel              = track->outChannel();
       for (iMidiController ci = mcl->begin(); ci != mcl->end(); ++ci)
-            pop->insertItem((*ci)->name());
+      {
+            if(cll->find(channel, ci->second->num()) == cll->end())
+              pop->insertItem(ci->second->name());
+      }
       int rv = pop->exec(buttonNewController->mapToGlobal(QPoint(0,0)));
       if (rv != -1) {
             QString s = pop->text(rv);
             for (iMidiController ci = mcl->begin(); ci != mcl->end(); ++ci) {
-                  if ((*ci)->name() == s) {
+                  MidiController* mc = ci->second;
+                  if (mc->name() == s) {
+                        if(cll->find(channel, mc->num()) == cll->end())
+                        {
+                          MidiCtrlValList* vl = new MidiCtrlValList(mc->num());
+                          cll->add(channel, vl);
+                          //song->update(SC_MIDI_CONTROLLER_ADD);
+                        }
                         for (int idx = 0; ;++idx) {
                               QString str = ctrlList->text(idx);
                               if (s == str)
+                              {
+                                    ctrlList->setSelected(idx, true);
+                                    ctrlListClicked(ctrlList->item(idx));
                                     break;
+                              }      
                               if (str.isNull()) {
                                     ctrlList->insertItem(s);
+                                    ctrlList->setSelected(idx, true);
+                                    ctrlListClicked(ctrlList->item(idx));
                                     break;
                                     }
                               }
+                              
+                        break;
                         }
                   }
             }
       delete pop;
       }
-
 //---------------------------------------------------------
 //   ctrlListClicked
 //---------------------------------------------------------
@@ -662,7 +713,7 @@ void EditCtrlDialog::ctrlListClicked(QListBoxItem* item)
       int portn                = track->outPort();
       MidiPort* port           = &midiPorts[portn];
       MidiCtrlValListList* cll = port->controller();
-
+      
       iMidiCtrlValList i;
       for (i = cll->begin(); i != cll->end(); ++i) {
             MidiCtrlValList* cl = i->second;
@@ -671,6 +722,10 @@ void EditCtrlDialog::ctrlListClicked(QListBoxItem* item)
             if (s == c->name()) {
                   if (num == CTRL_PROGRAM) {
                         widgetStack->raiseWidget(1);
+                        
+                        val = c->initVal();
+                        if(val == CTRL_VAL_UNKNOWN)
+                          val = 0;
                         updatePatch();
                         }
                   else {
@@ -679,6 +734,22 @@ void EditCtrlDialog::ctrlListClicked(QListBoxItem* item)
                         valSpinBox->setRange(c->minVal(), c->maxVal());
                         controllerName->setText(s);
                         val = c->initVal();
+                        
+                        if(val == CTRL_VAL_UNKNOWN || val == 0)
+                        {
+                          switch(num)
+                          {
+                            case CTRL_PANPOT:
+                              val = 64 - c->bias();
+                            break;
+                            case CTRL_VOLUME:
+                              val = 100;
+                            break;
+                            default:  
+                              val = 0;
+                            break;  
+                          } 
+                        }
                         valSlider->setValue(val);
                         }
                   break;
@@ -689,7 +760,7 @@ void EditCtrlDialog::ctrlListClicked(QListBoxItem* item)
       }
 
 //---------------------------------------------------------
-//   setPatch
+//   updatePatch
 //---------------------------------------------------------
 
 void EditCtrlDialog::updatePatch()
@@ -698,7 +769,7 @@ void EditCtrlDialog::updatePatch()
       int port              = track->outPort();
       int channel           = track->outChannel();
       MidiInstrument* instr = midiPorts[port].instrument();
-      const char* name = instr->getPatchName(channel, val, song->mtype());
+      const char* name = instr->getPatchName(channel, val, song->mtype(), track->type() == Track::DRUM);
       patchName->setText(QString(name));
 
       int hb = ((val >> 16) & 0xff) + 1;
@@ -734,8 +805,10 @@ void EditCtrlDialog::instrPopup()
       int channel = track->outChannel();
       int port    = track->outPort();
       MidiInstrument* instr = midiPorts[port].instrument();
-      instr->populatePatchPopup(pop, channel, song->mtype());
+      instr->populatePatchPopup(pop, channel, song->mtype(), track->type() == Track::DRUM);
 
+      if(pop->count() == 0)
+        return;
       int rv = pop->exec(patchName->mapToGlobal(QPoint(10,5)));
       if (rv != -1) {
             val = rv;
