@@ -29,10 +29,13 @@
 extern void initMidiSerial();
 #endif
 extern bool initMidiAlsa();
+extern bool initMidiJack();
 
 MidiDeviceList midiDevices;
 extern void processMidiInputTransformPlugins(MEvent&);
 
+extern unsigned int volatile lastExtMidiSyncTick;
+
 //---------------------------------------------------------
 //   initMidiDevices
 //---------------------------------------------------------
@@ -49,6 +52,14 @@ void initMidiDevices()
                                                           "your configuration.");
           exit(-1);
           }
+      
+      if(initMidiJack())
+          {
+          QMessageBox::critical(NULL, "MusE fatal error.", "MusE failed to initialize the\n" 
+                                                          "Jack midi subsystem, check\n"
+                                                          "your configuration.");
+          exit(-1);
+          }
       }
 
 //---------------------------------------------------------
@@ -71,14 +82,30 @@ void MidiDevice::init()
 
 MidiDevice::MidiDevice()
       {
-      _recBufFlipped = false;
+      ///_recBufFlipped = false;
+      //_tmpRecordCount = 0;
+      for(unsigned int i = 0; i < MIDI_CHANNELS + 1; ++i)
+        _tmpRecordCount[i] = 0;
+      
+      _sysexFIFOProcessed = false;
+      //_sysexWritingChunks = false;
+      _sysexReadingChunks = false;
+      
       init();
       }
 
 MidiDevice::MidiDevice(const QString& n)
    : _name(n)
       {
-      _recBufFlipped = false;
+      ///_recBufFlipped = false;
+      //_tmpRecordCount = 0;
+      for(unsigned int i = 0; i < MIDI_CHANNELS + 1; ++i)
+        _tmpRecordCount[i] = 0;
+      
+      _sysexFIFOProcessed = false;
+      //_sysexWritingChunks = false;
+      _sysexReadingChunks = false;
+      
       init();
       }
 
@@ -87,7 +114,8 @@ MidiDevice::MidiDevice(const QString& n)
 //    return true if event filtered
 //---------------------------------------------------------
 
-static bool filterEvent(const MEvent& event, int type, bool thru)
+//static bool filterEvent(const MEvent& event, int type, bool thru)
+bool filterEvent(const MEvent& event, int type, bool thru)
       {
       switch(event.type()) {
             case ME_NOTEON:
@@ -131,6 +159,64 @@ static bool filterEvent(const MEvent& event, int type, bool thru)
       return false;
       }
 
+//---------------------------------------------------------
+//   afterProcess
+//    clear all recorded events after a process cycle
+//---------------------------------------------------------
+
+void MidiDevice::afterProcess()
+{
+  //while (_tmpRecordCount--)
+  //  _recordFifo.remove();
+  
+  for(unsigned int i = 0; i < MIDI_CHANNELS + 1; ++i)
+  {
+    while (_tmpRecordCount[i]--)
+      _recordFifo[i].remove();
+  } 
+}
+
+//---------------------------------------------------------
+//   beforeProcess
+//    "freeze" fifo for this process cycle
+//---------------------------------------------------------
+
+void MidiDevice::beforeProcess()
+{
+  //if (!jackPort(0).isZero())
+  //      audioDriver->collectMidiEvents(this, jackPort(0));
+  
+  //_tmpRecordCount = _recordFifo.getSize();
+  for(unsigned int i = 0; i < MIDI_CHANNELS + 1; ++i)
+    _tmpRecordCount[i] = _recordFifo[i].getSize();
+  
+  // Reset this.
+  _sysexFIFOProcessed = false;
+}
+
+/*
+//---------------------------------------------------------
+//   getEvents
+//---------------------------------------------------------
+
+void MidiDevice::getEvents(unsigned , unsigned , int ch, MPEventList* dst)  //from //to
+{
+  for (int i = 0; i < _tmpRecordCount; ++i) {
+        const MidiPlayEvent& ev = _recordFifo.peek(i);
+        if (ch == -1 || (ev.channel() == ch))
+              dst->insert(ev);
+        }
+  
+  //while(!recordFifo.isEmpty())
+  //{
+  //  MidiPlayEvent e(recordFifo.get());
+  //  if (ch == -1 || (e.channel() == ch))
+  //        dst->insert(e);
+  //}  
+}
+*/
+
+/*
 //---------------------------------------------------------
 //   recordEvent
 //---------------------------------------------------------
@@ -143,6 +229,7 @@ MREventList* MidiDevice::recordEvents()
   else  
     return &_recordEvents2; 
 }
+*/
 
 //---------------------------------------------------------
 //   recordEvent
@@ -150,7 +237,13 @@ MREventList* MidiDevice::recordEvents()
 
 void MidiDevice::recordEvent(MidiRecordEvent& event)
       {
-      event.setTime(audio->timestamp());
+      // p3.3.35
+      // TODO: Tested, but record resolution not so good. Switch to wall clock based separate list in MidiDevice. And revert this line.
+      //event.setTime(audio->timestamp());
+      event.setTime(extSyncFlag.value() ? lastExtMidiSyncTick : audio->timestamp());
+      
+      //printf("MidiDevice::recordEvent event time:%d\n", event.time());
+      
       // Added by Tim. p3.3.8
       
       // By T356. Set the loop number which the event came in at.
@@ -162,36 +255,54 @@ void MidiDevice::recordEvent(MidiRecordEvent& event)
             printf("MidiInput: ");
             event.dump();
             }
-#if 0
+
       int typ = event.type();
+      
+      if(_port != -1)
+      {
+        int idin = midiPorts[_port].syncInfo().idIn();
+        
+// p3.3.26 1/23/10 Section was disabled, enabled by Tim.
+//#if 0
+  
+        //---------------------------------------------------
+        // filter some SYSEX events
+        //---------------------------------------------------
+  
+        if (typ == ME_SYSEX) {
+              const unsigned char* p = event.data();
+              int n = event.len();
+              if (n >= 4) {
+                    if ((p[0] == 0x7f)
+                      //&& ((p[1] == 0x7f) || (p[1] == rxDeviceId))) {
+                      && ((p[1] == 0x7f) || (idin == 0x7f) || (p[1] == idin))) {
+                          if (p[2] == 0x06) {
+                                //mmcInput(p, n);
+                                midiSeq->mmcInput(_port, p, n);
+                                return;
+                                }
+                          if (p[2] == 0x01) {
+                                //mtcInputFull(p, n);
+                                midiSeq->mtcInputFull(_port, p, n);
+                                return;
+                                }
+                          }
+                    else if (p[0] == 0x7e) {
+                          //nonRealtimeSystemSysex(p, n);
+                          midiSeq->nonRealtimeSystemSysex(_port, p, n);
+                          return;
+                          }
+                    }
+              }
+          else    
+            // p3.3.26 1/23/10 Moved here from alsaProcessMidiInput(). Anticipating Jack midi support, so don't make it ALSA specific. Tim. 
+            // Trigger general activity indicator detector. Sysex has no channel, don't trigger.
+            midiPorts[_port].syncInfo().trigActDetect(event.channel());
+              
+//#endif
 
-      //---------------------------------------------------
-      // filter some SYSEX events
-      //---------------------------------------------------
-
-      if (typ == ME_SYSEX) {
-            const unsigned char* p = event.data();
-            int n = event.len();
-            if (n >= 4) {
-                  if ((p[0] == 0x7f)
-                     && ((p[1] == 0x7f) || (p[1] == rxDeviceId))) {
-                        if (p[2] == 0x06) {
-                              mmcInput(p, n);
-                              return;
-                              }
-                        if (p[2] == 0x01) {
-                              mtcInputFull(p, n);
-                              return;
-                              }
-                        }
-                  else if (p[0] == 0x7e) {
-                        nonRealtimeSystemSysex(p, n);
-                        return;
-                        }
-                  }
-            }
-#endif
-
+      }
+      
       //
       //  process midi event input filtering and
       //    transformation
@@ -212,25 +323,39 @@ void MidiDevice::recordEvent(MidiRecordEvent& event)
       // transfer noteOn events to gui for step recording and keyboard
       // remote control
       //
-      if (event.type() == ME_NOTEON) {
+      if (typ == ME_NOTEON) {
             int pv = ((event.dataA() & 0xff)<<8) + (event.dataB() & 0xff);
             song->putEvent(pv);
             }
       
-      if(_recBufFlipped)
-        _recordEvents2.add(event);     // add event to secondary list of recorded events
-      else
-        _recordEvents.add(event);     // add event to primary list of recorded events
+      ///if(_recBufFlipped)
+      ///  _recordEvents2.add(event);     // add event to secondary list of recorded events
+      ///else
+      ///  _recordEvents.add(event);     // add event to primary list of recorded events
+      
+      //if(_recordFifo.put(MidiPlayEvent(event)))
+      //  printf("MidiDevice::recordEvent: fifo overflow\n");
+      
+      // p3.3.38
+      // Do not bother recording if it is NOT actually being used by a port.
+      // Because from this point on, process handles things, by selected port.
+      if(_port == -1)
+        return;
+      
+      // Split the events up into channel fifos. Special 'channel' number 17 for sysex events.
+      unsigned int ch = (typ == ME_SYSEX)? MIDI_CHANNELS : event.channel();
+      if(_recordFifo[ch].put(MidiPlayEvent(event)))
+        printf("MidiDevice::recordEvent: fifo channel %d overflow\n", ch);
       }
 
 //---------------------------------------------------------
 //   find
 //---------------------------------------------------------
 
-MidiDevice* MidiDeviceList::find(const QString& s)
+MidiDevice* MidiDeviceList::find(const QString& s, int typeHint)
       {
       for (iMidiDevice i = begin(); i != end(); ++i)
-            if ((*i)->name() == s)
+            if( (typeHint == -1 || typeHint == (*i)->deviceType()) && ((*i)->name() == s) )
                   return *i;
       return 0;
       }
@@ -349,11 +474,15 @@ bool MidiDevice::putEvent(const MidiPlayEvent& ev)
                         }
                   }
 #if 1 // if ALSA cannot handle RPN NRPN etc.
-            if (a < 0x1000) {          // 7 Bit Controller
+            
+            // p3.3.37
+            //if (a < 0x1000) {          // 7 Bit Controller
+            if (a < CTRL_14_OFFSET) {          // 7 Bit Controller
                   //putMidiEvent(MidiPlayEvent(0, 0, chn, ME_CONTROLLER, a, b));
                   putMidiEvent(ev);
                   }
-            else if (a < 0x20000) {     // 14 bit high resolution controller
+            //else if (a < 0x20000) {     // 14 bit high resolution controller
+            else if (a < CTRL_RPN_OFFSET) {     // 14 bit high resolution controller
                   int ctrlH = (a >> 8) & 0x7f;
                   int ctrlL = a & 0x7f;
                   int dataH = (b >> 7) & 0x7f;
@@ -361,7 +490,8 @@ bool MidiDevice::putEvent(const MidiPlayEvent& ev)
                   putMidiEvent(MidiPlayEvent(0, 0, chn, ME_CONTROLLER, ctrlH, dataH));
                   putMidiEvent(MidiPlayEvent(0, 0, chn, ME_CONTROLLER, ctrlL, dataL));
                   }
-            else if (a < 0x30000) {     // RPN 7-Bit Controller
+            //else if (a < 0x30000) {     // RPN 7-Bit Controller
+            else if (a < CTRL_NRPN_OFFSET) {     // RPN 7-Bit Controller
                   int ctrlH = (a >> 8) & 0x7f;
                   int ctrlL = a & 0x7f;
                   putMidiEvent(MidiPlayEvent(0, 0, chn, ME_CONTROLLER, CTRL_HRPN, ctrlH));
@@ -372,7 +502,8 @@ bool MidiDevice::putEvent(const MidiPlayEvent& ev)
                   //  events do not upset the last *RPN controller.
                   sendNullRPNParams(chn, false);
                 }
-            else if (a < 0x40000) {     // NRPN 7-Bit Controller
+            //else if (a < 0x40000) {     // NRPN 7-Bit Controller
+            else if (a < CTRL_INTERNAL_OFFSET) {     // NRPN 7-Bit Controller
                   int ctrlH = (a >> 8) & 0x7f;
                   int ctrlL = a & 0x7f;
                   putMidiEvent(MidiPlayEvent(0, 0, chn, ME_CONTROLLER, CTRL_HNRPN, ctrlH));
@@ -381,7 +512,8 @@ bool MidiDevice::putEvent(const MidiPlayEvent& ev)
                   
                   sendNullRPNParams(chn, true);
                   }
-            else if (a < 0x60000) {     // RPN14 Controller
+            //else if (a < 0x60000) {     // RPN14 Controller
+            else if (a < CTRL_NRPN14_OFFSET) {     // RPN14 Controller
                   int ctrlH = (a >> 8) & 0x7f;
                   int ctrlL = a & 0x7f;
                   int dataH = (b >> 7) & 0x7f;
@@ -393,7 +525,8 @@ bool MidiDevice::putEvent(const MidiPlayEvent& ev)
                   
                   sendNullRPNParams(chn, false);
                   }
-            else if (a < 0x70000) {     // NRPN14 Controller
+            //else if (a < 0x70000) {     // NRPN14 Controller
+            else if (a < CTRL_NONE_OFFSET) {     // NRPN14 Controller
                   int ctrlH = (a >> 8) & 0x7f;
                   int ctrlL = a & 0x7f;
                   int dataH = (b >> 7) & 0x7f;
