@@ -1,7 +1,7 @@
 //=========================================================
 //  MusE
 //  Linux Music Editor
-//    $Id: dlist.cpp,v 1.9.2.2 2006/01/05 18:46:47 spamatica Exp $
+//    $Id: dlist.cpp,v 1.9.2.7 2009/10/16 21:50:16 terminator356 Exp $
 //  (C) Copyright 1999 Werner Schweer (ws@seh.de)
 //=========================================================
 
@@ -13,6 +13,7 @@
 
 #include <stdio.h>
 
+#include "audio.h"
 #include "pitchedit.h"
 #include "midiport.h"
 #include "drummap.h"
@@ -64,6 +65,9 @@ void DList::draw(QPainter& p, const QRect& rect)
                   p.save();
                   p.setWorldXForm(false);
                   switch (k) {
+                        case COL_VOL:
+                              s.setNum(dm->vol);
+                              break;
                         case COL_QNT:
                               s.setNum(dm->quant);
                               break;
@@ -107,7 +111,7 @@ void DList::draw(QPainter& p, const QRect& rect)
                               s.setNum(dm->channel+1);
                               break;
                         case COL_PORT:
-                              s = midiPorts[dm->port].portname();
+                              s.sprintf("%d:%s", dm->port+1, midiPorts[dm->port].portname().latin1());
                               align = AlignVCenter | AlignLeft;
                               break;
                         }
@@ -145,7 +149,8 @@ void DList::draw(QPainter& p, const QRect& rect)
       int n = header->count();
       x = 0;
       for (int i = 0; i < n; i++) {
-            x += header->sectionSize(i);
+            //x += header->sectionSize(i);
+            x += header->sectionSize(header->mapToSection(i));
             p.drawLine(x, 0, x, height());
             }
       p.setWorldXForm(true);
@@ -159,16 +164,45 @@ void DList::devicesPopupMenu(DrumMap* t, int x, int y, bool changeAll)
       {
       QPopupMenu* p = midiPortsPopup(this);
       int n = p->exec(mapToGlobal(QPoint(x, y)), 0);
+      bool doemit = false;
       if (n != -1) {
             if (!changeAll)
+            {
+                if(n != t->port)
+                {
+                  audio->msgIdle(true);
+                  //audio->msgRemapPortDrumCtlEvents(getSelectedInstrument(), -1, -1, n);
+                  song->remapPortDrumCtrlEvents(getSelectedInstrument(), -1, -1, n);
+                  audio->msgIdle(false);
                   t->port = n;
+                  doemit = true;
+                }  
+            }      
             else {
+                  audio->msgIdle(true);
+                  // Delete all port controller events.
+                  //audio->msgChangeAllPortDrumCtrlEvents(false);
+                  song->changeAllPortDrumCtrlEvents(false);
+                  
                   for (int i = 0; i < DRUM_MAPSIZE; i++)
                         drumMap[i].port = n;
+                  // Add all port controller events.
+                  //audio->msgChangeAllPortDrumCtrlEvents(true);
+                  song->changeAllPortDrumCtrlEvents(true);
+                  
+                  audio->msgIdle(false);
+                  doemit = true;
                   }
             }
       delete p;
-      }
+      if(doemit)
+      {
+        int instr = getSelectedInstrument();
+        if(instr != -1)
+          //emit curDrumInstrumentChanged(instr);
+          song->update(SC_DRUMMAP);
+      }            
+    }
 
 //---------------------------------------------------------
 //   viewMousePressEvent
@@ -203,7 +237,7 @@ void DList::viewMousePressEvent(QMouseEvent* ev)
       // In that case, treat it as if a return was pressed
 
       if (button == QMouseEvent::LeftButton) {
-            if ((editEntry && editEntry != dm  || col != selectedColumn) && editEntry != 0) {
+            if (((editEntry && editEntry != dm)  || col != selectedColumn) && editEntry != 0) {
                   returnPressed();
                   }
             }
@@ -221,6 +255,14 @@ void DList::viewMousePressEvent(QMouseEvent* ev)
                         devicesPopupMenu(dm, mapx(x), mapy(pitch * TH), changeAll);
                         }
                   break;
+            case COL_VOL:
+                  val = dm->vol + incVal;
+                  if (val < 0)
+                        val = 0;
+                  else if (val > 200)
+                        val = 200;
+                  dm->vol = (unsigned char)val;      
+                  break;
             case COL_QNT:
                   dm->quant += incVal;
                   // ?? range
@@ -256,7 +298,15 @@ void DList::viewMousePressEvent(QMouseEvent* ev)
                         val = 0;
                   else if (val > 127)
                         val = 127;
-                  dm->anote = val;
+                  if(val != dm->anote)
+                  {  
+                    audio->msgIdle(true);
+                    //audio->msgRemapPortDrumCtlEvents(pitch, val, -1, -1);
+                    song->remapPortDrumCtrlEvents(pitch, val, -1, -1);
+                    audio->msgIdle(false);
+                    dm->anote = val;
+                    song->update(SC_DRUMMAP);
+                  }
                   emit keyPressed(pitch, shift);//(dm->anote, shift);
                   break;
             case COL_CHANNEL:
@@ -265,12 +315,33 @@ void DList::viewMousePressEvent(QMouseEvent* ev)
                         val = 0;
                   else if (val > 127)
                         val = 127;
+                  
                   if (ev->state() & ControlButton) {
+                        audio->msgIdle(true);
+                        // Delete all port controller events.
+                        //audio->msgChangeAllPortDrumCtrlEvents(false);
+                        song->changeAllPortDrumCtrlEvents(false, true);
+                        
                         for (int i = 0; i < DRUM_MAPSIZE; i++)
                               drumMap[i].channel = val;
+                        // Add all port controller events.
+                        //audio->msgChangeAllPortDrumCtrlEvents(true);
+                        song->changeAllPortDrumCtrlEvents(true, true);
+                        audio->msgIdle(false);
+                        song->update(SC_DRUMMAP);
                         }
                   else
+                  {
+                      if(val != dm->channel)
+                      {
+                        audio->msgIdle(true);
+                        //audio->msgRemapPortDrumCtlEvents(pitch, -1, val, -1);
+                        song->remapPortDrumCtrlEvents(pitch, -1, val, -1);
+                        audio->msgIdle(false);
                         dm->channel = val;
+                        song->update(SC_DRUMMAP);
+                      }  
+                  }      
                   break;
             case COL_LV1:
                   val = dm->lv1 + incVal;
@@ -344,9 +415,10 @@ void DList::viewMouseDoubleClickEvent(QMouseEvent* ev)
 
       int section = header->sectionAt(x);
 
-      if (section == COL_NAME || section == COL_LEN || section == COL_LV1 ||
-         section == COL_LV2 || section == COL_LV3 || section == COL_LV4) {
-         lineEdit(pitch, section);
+      if ((section == COL_NAME || section == COL_VOL || section == COL_LEN || section == COL_LV1 ||
+         section == COL_LV2 || section == COL_LV3 || section == COL_LV4) && (ev->button() == QMouseEvent::LeftButton))
+         {
+           lineEdit(pitch, section);
          }
       else
             viewMousePressEvent(ev);
@@ -377,6 +449,11 @@ void DList::lineEdit(int line, int section)
                   editor->setText(dm->name);
                   break;
 
+                  case COL_VOL: {
+                  editor->setText(QString::number(dm->vol));
+                  break;
+                  }
+                  
                   case COL_LEN: {
                   editor->setText(QString::number(dm->len));
                   break;
@@ -454,6 +531,7 @@ void DList::setCurDrumInstrument(int instr)
       if (currentlySelected != dm) {
             currentlySelected = &drumMap[instr];
             emit curDrumInstrumentChanged(instr);
+            song->update(SC_DRUMMAP);
             }
       }
 
@@ -473,15 +551,27 @@ void DList::sizeChange(int, int, int)
 void DList::returnPressed()
       {
       int val = -1;
-      if (selectedColumn != COL_NAME) {
+      if (selectedColumn != COL_NAME) 
+      {
             val = atoi(editor->text().ascii());
-            if (selectedColumn != COL_LEN) {
+            if (selectedColumn != COL_LEN) 
+            {
+              if(selectedColumn == COL_VOL)
+              {
+                  if (val > 200) //Check bounds for volume
+                  val = 200;
+                  if (val < 0)
+                  val = 0;
+              }
+              else
+              {
                   if (val > 127) //Check bounds for lv1-lv4 values
                   val = 127;
                   if (val < 0)
                   val = 0;
-                  }
-            }
+              }    
+            }  
+      }     
 
       switch(selectedColumn) {
             case COL_NAME:
@@ -492,6 +582,10 @@ void DList::returnPressed()
                   editEntry->len = atoi(editor->text().ascii());
                   break;
 
+            case COL_VOL:
+                  editEntry->vol = val;
+                  break;
+
             case COL_LV1:
                   editEntry->lv1 = val;
                   break;
@@ -616,15 +710,9 @@ void DList::viewMouseReleaseEvent(QMouseEvent* ev)
       if (drag == DRAG) {
             int y = ev->y();
             unsigned dPitch = y / TH;
-            DrumMap dm = drumMap[sPitch];
-            drumMap[sPitch] = drumMap[dPitch];
-            drumMap[dPitch] = dm;
             setCursor(QCursor(arrowCursor));
-            drumInmap[int(drumMap[sPitch].enote)]  = sPitch;
-            drumOutmap[int(drumMap[int(sPitch)].anote)] = sPitch;
-            drumInmap[int(drumMap[int(dPitch)].enote)]  = dPitch;
-            drumOutmap[int(drumMap[int(dPitch)].anote)] = dPitch;
             currentlySelected = &drumMap[int(dPitch)];
+            emit curDrumInstrumentChanged(dPitch);
             emit mapChanged(sPitch, dPitch); //Track pitch change done in canvas
             }
       drag = NORMAL;
