@@ -22,6 +22,7 @@
 #include <qapplication.h>
 #include <qclipboard.h>
 #include <qmessagebox.h>
+#include <qaction.h>
 
 #include "drumedit.h"
 #include "mtscale.h"
@@ -43,11 +44,21 @@
 #include "audio.h"
 #include "gconfig.h"
 
+/*
 static const char* map_file_pattern[] = {
-      "presets (*.map *.map.gz *.map.bz2)",
+      "Presets (*.map *.map.gz *.map.bz2)",
       "All Files (*)",
       0
       };
+static const char* map_file_save_pattern[] = {
+      "Presets (*.map)",
+      "gzip compressed presets (*.map.gz)",
+      "bzip2 compressed presets (*.map.bz2)",
+      "All Files (*)",
+      0
+      };
+*/      
+
 int DrumEdit::_quantInit = 96;
 int DrumEdit::_rasterInit = 96;
 int DrumEdit::_widthInit = 600;
@@ -181,6 +192,12 @@ DrumEdit::DrumEdit(PartList* pl, QWidget* parent, const char* name, unsigned ini
       menuSelect->insertSeparator();
       menuSelect->insertItem(tr("Inside Loop"),  DrumCanvas::CMD_SELECT_ILOOP);
       menuSelect->insertItem(tr("Outside Loop"), DrumCanvas::CMD_SELECT_OLOOP);
+      
+      menuSelect->insertSeparator();
+
+      menuSelect->insertItem(tr("Previous Part"), DrumCanvas::CMD_SELECT_PREV_PART);
+      menuSelect->insertItem(tr("Next Part"), DrumCanvas::CMD_SELECT_NEXT_PART);
+      
       menuEdit->insertItem(tr("&Select"), menuSelect);
 
       QPopupMenu* menuScriptPlugins = new QPopupMenu(this);
@@ -240,7 +257,7 @@ DrumEdit::DrumEdit(PartList* pl, QWidget* parent, const char* name, unsigned ini
       QPushButton* ctrl = new QPushButton(tr("ctrl"), mainw, "Ctrl");
       ctrl->setFont(config.fonts[3]);
       hscroll           = new ScrollScale(-25, -2, xscale, 20000, Horizontal, mainw);
-      ctrl->setFixedSize(50, hscroll->sizeHint().height());
+      ctrl->setFixedSize(40, hscroll->sizeHint().height());
       QToolTip::add(ctrl, tr("Add Controller View"));
 
       QSizeGrip* corner = new QSizeGrip(mainw);
@@ -308,6 +325,9 @@ DrumEdit::DrumEdit(PartList* pl, QWidget* parent, const char* name, unsigned ini
       new DWhatsThis(header, header);
 
       dlist = new DList(header, split1w1, yscale);
+      // p3.3.44
+      setCurDrumInstrument(dlist->getSelectedInstrument());
+      
       connect(dlist, SIGNAL(keyPressed(int, bool)), canvas, SLOT(keyPressed(int, bool)));
       connect(dlist, SIGNAL(keyReleased(int, bool)), canvas, SLOT(keyReleased(int, bool)));
       connect(dlist, SIGNAL(mapChanged(int, int)), canvas, SLOT(mapChanged(int, int)));
@@ -690,7 +710,8 @@ void DrumEdit::writeConfiguration(int level, Xml& xml)
 
 void DrumEdit::load()
       {
-      QString fn = getOpenFileName("drummaps", map_file_pattern,
+      //QString fn = getOpenFileName("drummaps", map_file_pattern,
+      QString fn = getOpenFileName("drummaps", drum_map_file_pattern,
          this, tr("Muse: Load Drum Map"), 0);
       if (fn.isEmpty())
             return;
@@ -742,7 +763,8 @@ ende:
 
 void DrumEdit::save()
       {
-      QString fn = getSaveFileName(QString("drummaps"), map_file_pattern,
+      //QString fn = getSaveFileName(QString("drummaps"), map_file_pattern,
+      QString fn = getSaveFileName(QString("drummaps"), drum_map_file_save_pattern,
         this, tr("MusE: Store Drum Map"));
       if (fn.isEmpty())
             return;
@@ -841,8 +863,13 @@ CtrlEdit* DrumEdit::addCtrl()
       connect(tools2,   SIGNAL(toolChanged(int)),   ctrlEdit, SLOT(setTool(int)));
       connect(dlist,    SIGNAL(curDrumInstrumentChanged(int)), SLOT(setCurDrumInstrument(int)));
 
+      //printf("DrumEdit::addCtrl curDrumInstrument:%d\n", dlist->getSelectedInstrument());
+      
       setCurDrumInstrument(dlist->getSelectedInstrument());
 
+      // p3.3.44
+      ctrlEdit->setTool(tools2->curTool());
+      
       ctrlEdit->setXPos(hscroll->pos());
       ctrlEdit->setXMag(hscroll->getScaleValue());
 
@@ -1106,6 +1133,9 @@ void DrumEdit::initShortcuts()
       menuSelect->setAccel(shortcuts[SHRT_SELECT_INVERT].key, DrumCanvas::CMD_SELECT_INVERT);
       menuSelect->setAccel(shortcuts[SHRT_SELECT_ILOOP].key,  DrumCanvas::CMD_SELECT_ILOOP);
       menuSelect->setAccel(shortcuts[SHRT_SELECT_OLOOP].key,  DrumCanvas::CMD_SELECT_OLOOP);
+      
+      menuSelect->setAccel(shortcuts[SHRT_SELECT_PREV_PART].key, DrumCanvas::CMD_SELECT_PREV_PART);
+      menuSelect->setAccel(shortcuts[SHRT_SELECT_NEXT_PART].key, DrumCanvas::CMD_SELECT_NEXT_PART);
       }
 
 //---------------------------------------------------------
