@@ -100,14 +100,137 @@ static const char *oct_xpm[] = {
       "########################...............#",
       "#######################................#", //------------------------
       ".......................................#",
+      "..........................###..........#",
+      ".........................#...#.........#",
+      ".........................#.............#",
+      ".........................#.............#",
+      ".........................#...#.........#",     // 10
+      "..........................###..........#",
       ".......................................#",
       ".......................................#",
       ".......................................#",
+    };
+
+static const char *mk1_xpmC1[] = {
+      "40 10 2 1",
+      ". c #c0c0c0",
+      "# c #000000",
       ".......................................#",
-      ".......................................#",     // 10
+      "..........................###.....#....#",
+      ".........................#...#...##....#",
+      ".........................#........#....#",
+      ".........................#........#....#",
+      ".........................#...#....#....#",     // 10
+      "..........................###....###...#",
       ".......................................#",
       ".......................................#",
       ".......................................#",
+      };
+static const char *mk1_xpmC2[] = {
+      "40 10 2 1",
+      ". c #c0c0c0",
+      "# c #000000",
+      ".......................................#",
+      "..........................###....##....#",
+      ".........................#...#..#..#...#",
+      ".........................#........#....#",
+      ".........................#.......#.....#",
+      ".........................#...#..#......#",     // 10
+      "..........................###...####...#",
+      ".......................................#",
+      ".......................................#",
+      ".......................................#",
+      };
+static const char *mk1_xpmC3[] = {
+      "40 10 2 1",
+      ". c #c0c0c0",
+      "# c #000000",
+      ".......................................#",
+      "..........................###....##....#",
+      ".........................#...#..#..#...#",
+      ".........................#........#....#",
+      ".........................#.........#...#",
+      ".........................#...#..#..#...#",     // 10
+      "..........................###....##....#",
+      ".......................................#",
+      ".......................................#",
+      ".......................................#",
+      };
+static const char *mk1_xpmC4[] = {
+      "40 10 2 1",
+      ". c #c0c0c0",
+      "# c #000000",
+      ".......................................#",
+      "..........................###...#..#...#",
+      ".........................#...#..#..#...#",
+      ".........................#......####...#",
+      ".........................#.........#...#",
+      ".........................#...#.....#...#",     // 10
+      "..........................###......#...#",
+      ".......................................#",
+      ".......................................#",
+      ".......................................#",
+      };
+static const char *mk1_xpmC5[] = {
+      "40 10 2 1",
+      ". c #c0c0c0",
+      "# c #000000",
+      ".......................................#",
+      "..........................###...####...#",
+      ".........................#...#..#......#",
+      ".........................#......###....#",
+      ".........................#.........#...#",
+      ".........................#...#.....#...#",     // 10
+      "..........................###...###....#",
+      ".......................................#",
+      ".......................................#",
+      ".......................................#",
+      };
+
+static const char *mk1_xpmC6[] = {
+      "40 10 2 1",
+      ". c #c0c0c0",
+      "# c #000000",
+      ".......................................#",
+      "..........................###....###...#",
+      ".........................#...#..#......#",
+      ".........................#......###....#",
+      ".........................#......#..#...#",
+      ".........................#...#..#..#...#",     // 10
+      "..........................###...###....#",
+      ".......................................#",
+      ".......................................#",
+      ".......................................#",
+      };
+
+static const char *mk1_xpmC7[] = {
+      "40 10 2 1",
+      ". c #c0c0c0",
+      "# c #000000",
+      ".......................................#",
+      "..........................###...####...#",
+      ".........................#...#.....#...#",
+      ".........................#........#....#",
+      ".........................#.......#.....#",
+      ".........................#...#..#......#",     // 10
+      "..........................###...#......#",
+      ".......................................#",
+      ".......................................#",
+      ".......................................#",
+      };
+static const char *mk1_xpmC8[] = {
+      "40 10 2 1",
+      ". c #c0c0c0",
+      "# c #000000",
+      ".......................................#",
+      "..........................###....##....#",
+      ".........................#...#..#..#....#",
+      ".........................#.......##....#",
+      ".........................#......#..#...#",
+      ".........................#...#..#..#...#",     // 10
+      "..........................###....##....#",
+      ".......................................#",
+      ".......................................#",
       ".......................................#",
       };
 
@@ -233,6 +356,15 @@ Piano::Piano(QWidget* parent, int ymag)
       setMouseTracking(true);
       curPitch = -1;
       octave = new QPixmap(oct_xpm);
+      c_keys[0] = new QPixmap(mk1_xpmC8);
+      c_keys[1] = new QPixmap(mk1_xpmC7);
+      c_keys[2] = new QPixmap(mk1_xpmC6);
+      c_keys[3] = new QPixmap(mk1_xpmC5);
+      c_keys[4] = new QPixmap(mk1_xpmC4);
+      c_keys[5] = new QPixmap(mk1_xpmC3);
+      c_keys[6] = new QPixmap(mk1_xpmC2);
+      c_keys[7] = new QPixmap(mk1_xpmC1);
+
       mk1 = new QPixmap(mk1_xpm);
       mk2 = new QPixmap(mk2_xpm);
       mk3 = new QPixmap(mk3_xpm);
@@ -249,6 +381,20 @@ void Piano::draw(QPainter& p, const QRect& r)
       {
       QPoint offset(0, KH*2);
       p.drawTiledPixmap(r, *octave, r.topLeft()+offset);
+
+      // draw C notes
+      for (int drawKey = 0; drawKey < 8;drawKey++) {
+        int octaveSize=91;
+
+        int drawY = octaveSize * drawKey + 81 - KH*2;
+        if (drawY > r.y() && drawY < r.y() + r.height()) {
+          //printf("drawing c %d at %d r.y %d r.x %d\n",drawKey, drawY, r.y(), r.x());
+          p.drawPixmap(0,drawY,*c_keys[drawKey]);
+        }
+      }
+      //p.drawTiledPixmap(r, *c1, r.topLeft()+offset + coffset);
+      //printf("drawText KH %d %d, x %d y %d\n",KH, curPitch, r.x(), r.y());
+      //p.drawText(r,Qt::AlignAuto,"A");
       if (curPitch == -1)
             return;
       int y = pitch2y(curPitch);
