@@ -87,7 +87,7 @@ class PianoCanvas : public EventCanvas {
          CMD_OVER_QUANTIZE, CMD_ON_QUANTIZE, CMD_ONOFF_QUANTIZE,
          CMD_ITERATIVE_QUANTIZE,
          CMD_SELECT_ALL, CMD_SELECT_NONE, CMD_SELECT_INVERT,
-         CMD_SELECT_ILOOP, CMD_SELECT_OLOOP,
+         CMD_SELECT_ILOOP, CMD_SELECT_OLOOP, CMD_SELECT_PREV_PART, CMD_SELECT_NEXT_PART, 
          CMD_MODIFY_GATE_TIME, CMD_MODIFY_VELOCITY,
          CMD_CRESCENDO, CMD_TRANSPOSE, CMD_THIN_OUT, CMD_ERASE_EVENT,
          CMD_NOTE_SHIFT, CMD_MOVE_CLOCK, CMD_COPY_MEASURE,
