@@ -131,6 +131,10 @@ void MidiEditor::writePartList(int level, Xml& xml) const
             Track* track = part->track();
             int trkIdx   = song->tracks()->index(track);
             int partIdx  = track->parts()->index(part);
+            
+            if((trkIdx == -1) || (partIdx == -1))
+              printf("MidiEditor::writePartList error: trkIdx:%d partIdx:%d\n", trkIdx, partIdx);
+              
             xml.put(level, "<part>%d:%d</part>", trkIdx, partIdx);
             }
       }
@@ -196,7 +200,7 @@ void MidiEditor::setCurDrumInstrument(int instr)
       }
 
 //---------------------------------------------------------
-//   curMidiPart
+//   curCanvasPart
 //---------------------------------------------------------
 
 Part* MidiEditor::curCanvasPart() 
@@ -219,3 +223,13 @@ WavePart* MidiEditor::curWavePart()
     return 0; 
 }
 
+//---------------------------------------------------------
+//   setCurCanvasPart
+//---------------------------------------------------------
+
+void MidiEditor::setCurCanvasPart(Part* part) 
+{ 
+  if(canvas) 
+    canvas->setCurrentPart(part); 
+}
+
