@@ -6,7 +6,7 @@
 //  (C) Copyright 2000-2004 Werner Schweer (ws@seh.de)
 //=========================================================
 
-#include "config.h"
+//#include "config.h"
 
 #include <qpopupmenu.h>
 #include "mididev.h"
@@ -21,9 +21,9 @@
 #include "app.h"
 #include "song.h"
 
-#ifdef DSSI_SUPPORT
-#include "dssihost.h"
-#endif
+//#ifdef DSSI_SUPPORT
+//#include "dssihost.h"
+//#endif
 
 MidiPort midiPorts[MIDI_PORTS];
 
@@ -50,7 +50,8 @@ MidiPort::MidiPort()
       _device     = 0;
       _instrument = 0;
       _controller = new MidiCtrlValListList();
-      
+      _foundInSongFile = false;
+
       //
       // create minimum set of managed controllers
       // to make midi mixer operational
@@ -131,7 +132,9 @@ void MidiPort::setMidiDevice(MidiDevice* dev)
             // The 'reverb level' controller would still be at '100', and could adversely affect the song,
             //  but if the instrument has an available initial value of say '0', it will be used instead.
             //
-            if(_instrument)
+            //if(_instrument)
+            // p3.3.39 NOT for syntis! Use midiState an/or initParams for that.
+            if(_instrument && !_device->isSynti())
             {
               MidiControllerList* cl = _instrument->controller();
               MidiController* mc;
@@ -159,7 +162,7 @@ void MidiPort::setMidiDevice(MidiDevice* dev)
                     {
                       int ctl = mc->num();
                       
-#ifdef DSSI_SUPPORT
+///#ifdef DSSI_SUPPORT
                       // Exclude dssi synths from this, as some of them have hundreds of controls.
                       // Another difference is dssi synth devices (usually) have readable default port values,
                       //  unlike a midi output port, which cannot be queried for a current or default value,
@@ -168,15 +171,16 @@ void MidiPort::setMidiDevice(MidiDevice* dev)
                       // Another difference is dssi controls are best manipulated as ladspa controls -
                       //  (they ARE ladspa controls). This is stuff I mainly put for midi ports and MESS...
                       // I DO allow midi control of those ladspa controls, so our midi controls shall be updated here...
-                      if(!_device->isSynti() || (dynamic_cast<DssiSynthIF*>(((SynthI*)_device)->sif()) == 0))
-                      {  
-#endif
+                      // p3.3.39 Only non-syntis! Use midiState an/or initParams for that.
+                      ///if(!_device->isSynti() || (dynamic_cast<DssiSynthIF*>(((SynthI*)_device)->sif()) == 0))
+                      ///{  
+///#endif
                         // Note the addition of bias!
                         _device->putEvent(MidiPlayEvent(0, portno(), chan,
                           ME_CONTROLLER, ctl, mc->initVal() + mc->bias()));
-#ifdef DSSI_SUPPORT
-                      }
-#endif
+///#ifdef DSSI_SUPPORT
+                      ///}
+///#endif
                         
                       // Set it once so the 'last HW value' is set, and control knobs are positioned at the value...
                       //setHwCtrlState(chan, ctl, mc->initVal() + mc->bias());
@@ -190,23 +194,26 @@ void MidiPort::setMidiDevice(MidiDevice* dev)
             }
 
             // init HW controller state
-            for (iMidiCtrlValList i = _controller->begin(); i != _controller->end(); ++i) {
+            // p3.3.39 NOT for syntis! Use midiState an/or initParams for that.
+            if(!_device->isSynti())
+            {
+              for (iMidiCtrlValList i = _controller->begin(); i != _controller->end(); ++i) {
                   int channel = i->first >> 24;
                   int cntrl   = i->first & 0xffffff;
                   int val     = i->second->hwVal();
                   if (val != CTRL_VAL_UNKNOWN) {
                         
                         
-#ifdef DSSI_SUPPORT
+///#ifdef DSSI_SUPPORT
                       // Not for dssi synths...
-                      if(!_device->isSynti() || (dynamic_cast<DssiSynthIF*>(((SynthI*)_device)->sif()) == 0))
-                      {  
-#endif
+                      ///if(!_device->isSynti() || (dynamic_cast<DssiSynthIF*>(((SynthI*)_device)->sif()) == 0))
+                      ///{  
+///#endif
                         _device->putEvent(MidiPlayEvent(0, portno(), channel,
                           ME_CONTROLLER, cntrl, val));
-#ifdef DSSI_SUPPORT
-                      }
-#endif
+///#ifdef DSSI_SUPPORT
+                      ///}
+///#endif
                         
                         // Set it once so the 'last HW value' is set, and control knobs are positioned at the value...
                         setHwCtrlState(channel, cntrl, val);
@@ -215,7 +222,8 @@ void MidiPort::setMidiDevice(MidiDevice* dev)
                         //setHwCtrlStates(channel, cntrl, CTRL_VAL_UNKNOWN, val);
                         }
                   }
-            }
+              }
+            }  
 
       else
             clearDevice();
@@ -433,11 +441,12 @@ void MidiPort::sendGmOn()
 
 void MidiPort::sendGsOn()
       {
-      static unsigned char data2[] = { 0x41, 0x10, 0x42, 0x12, 0x40, 0x01, 0x33, 0x50, 0x3c };
-      static unsigned char data3[] = { 0x41, 0x10, 0x42, 0x12, 0x40, 0x01, 0x34, 0x50, 0x3b };
-
-      sendSysex(data2, sizeof(data2));
-      sendSysex(data3, sizeof(data3));
+      //static unsigned char data2[] = { 0x41, 0x10, 0x42, 0x12, 0x40, 0x01, 0x33, 0x50, 0x3c };
+      //static unsigned char data3[] = { 0x41, 0x10, 0x42, 0x12, 0x40, 0x01, 0x34, 0x50, 0x3b };
+      //sendSysex(data2, sizeof(data2));
+      //sendSysex(data3, sizeof(data3));
+      sendSysex(gsOnMsg2, gsOnMsg2Len);
+      sendSysex(gsOnMsg3, gsOnMsg3Len);
       }
 
 //---------------------------------------------------------
@@ -464,6 +473,56 @@ void MidiPort::sendSysex(const unsigned char* p, int n)
             }
       }
 
+//---------------------------------------------------------
+//   sendMMCLocate
+//---------------------------------------------------------
+
+void MidiPort::sendMMCLocate(unsigned char ht, unsigned char m, unsigned char s, unsigned char f, unsigned char sf, int devid)
+{
+  unsigned char msg[mmcLocateMsgLen];
+  memcpy(msg, mmcLocateMsg, mmcLocateMsgLen);
+  if(devid != -1)
+    msg[1]  = devid;
+  else
+    msg[1]  = _syncInfo.idOut();
+  msg[6]    = ht;
+  msg[7]    = m;
+  msg[8]    = s;
+  msg[9]    = f;
+  msg[10]   = sf;
+  sendSysex(msg, mmcLocateMsgLen);
+}
+
+//---------------------------------------------------------
+//   sendMMCStop
+//---------------------------------------------------------
+
+void MidiPort::sendMMCStop(int devid)
+{
+  unsigned char msg[mmcStopMsgLen];
+  memcpy(msg, mmcStopMsg, mmcStopMsgLen);
+  if(devid != -1)
+    msg[1] = devid;
+  else
+    msg[1] = _syncInfo.idOut();
+  sendSysex(msg, mmcStopMsgLen);
+}
+
+//---------------------------------------------------------
+//   sendMMCDeferredPlay
+//---------------------------------------------------------
+
+void MidiPort::sendMMCDeferredPlay(int devid)
+{
+  unsigned char msg[mmcDeferredPlayMsgLen];
+  memcpy(msg, mmcDeferredPlayMsg, mmcDeferredPlayMsgLen);
+  if(devid != -1)
+    msg[1] = devid;
+  else
+    msg[1] = _syncInfo.idOut();
+  sendSysex(msg, mmcDeferredPlayMsgLen);
+}
+
 //---------------------------------------------------------
 //   sendStart
 //---------------------------------------------------------
@@ -930,3 +989,39 @@ void MidiPort::setNullSendValue(int v)
   if(_instrument) 
     _instrument->setNullSendValue(v); 
 }
+
+//---------------------------------------------------------
+//   writeRouting    // p3.3.50
+//---------------------------------------------------------
+
+void MidiPort::writeRouting(int level, Xml& xml) const
+{
+      // If this device is not actually in use by the song, do not write any routes.
+      // This prevents bogus routes from being saved and propagated in the med file.
+      if(!device())
+        return;
+     
+      QString s;
+      
+      for (ciRoute r = _outRoutes.begin(); r != _outRoutes.end(); ++r) 
+      {
+        if(r->type == Route::TRACK_ROUTE && !r->name().isEmpty())
+        {
+          //xml.tag(level++, "Route");
+          
+          s = QT_TR_NOOP("Route");
+          if(r->channel != -1 && r->channel != 0)  
+            s += QString(QT_TR_NOOP(" channelMask=\"%1\"")).arg(r->channel);  // Use new channel mask.
+          xml.tag(level++, s);
+          
+          xml.tag(level, "source mport=\"%d\"/", portno());
+          
+          s = QT_TR_NOOP("dest");
+          s += QString(QT_TR_NOOP(" name=\"%1\"/")).arg(Xml::xmlString(r->name()));
+          xml.tag(level, s);
+          
+          xml.etag(level--, "Route");
+        }
+      }
+}
+    
