@@ -11,6 +11,7 @@
 
 #include "globaldefs.h"
 #include "sync.h"
+#include "route.h"
 
 class MidiDevice;
 class MidiInstrument;
@@ -33,7 +34,11 @@ class MidiPort {
       AutomationType _automationType[MIDI_CHANNELS];
       // Holds sync settings and detection monitors.
       MidiSyncInfo _syncInfo;
-
+      // p3.3.50 Just a flag to say the port was found in the song file, even if it has no device right now.
+      bool _foundInSongFile;
+      
+      RouteList _inRoutes, _outRoutes;
+      
       void clearDevice();
 
    public:
@@ -60,6 +65,8 @@ class MidiPort {
       bool hasGui() const;
 
       int portno() const;
+      bool foundInSongFile() const              { return _foundInSongFile; }
+      void setFoundInSongFile(bool b)           { _foundInSongFile = b; }
 
       MidiDevice* device() const                { return _device; }
       const QString& state() const              { return _state; }
@@ -77,6 +84,12 @@ class MidiPort {
       int nullSendValue();
       void setNullSendValue(int v);
 
+      RouteList* inRoutes()    { return &_inRoutes; }
+      RouteList* outRoutes()   { return &_outRoutes; }
+      bool noInRoute() const   { return _inRoutes.empty();  }
+      bool noOutRoute() const  { return _outRoutes.empty(); }
+      void writeRouting(int, Xml&) const;
+      
       // send events to midi device and keep track of
       // device state:
       void sendGmOn();
@@ -91,6 +104,11 @@ class MidiPort {
       void sendSongpos(int);
       void sendClock();
       void sendSysex(const unsigned char* p, int n);
+      void sendMMCLocate(unsigned char ht, unsigned char m,
+                         unsigned char s, unsigned char f, unsigned char sf, int devid = -1);
+      void sendMMCStop(int devid = -1);
+      void sendMMCDeferredPlay(int devid = -1);
+      
       bool sendEvent(const MidiPlayEvent&);
       AutomationType automationType(int channel) { return _automationType[channel]; }
       void setAutomationType(int channel, AutomationType t) {
