@@ -8,6 +8,8 @@
 //  (C) Copyright 2003 Werner Schweer (ws@seh.de)
 //=========================================================
 
+#include <qmessagebox.h>
+#include <qobject.h>
 #include <stdio.h>
 #include <fcntl.h>
 #include <sys/ioctl.h>
@@ -22,6 +24,7 @@
 #include "midictrl.h"
 #include "audio.h"
 #include "driver/alsamidi.h"
+#include "driver/jackmidi.h"
 #include "sync.h"
 #include "synth.h"
 #include "song.h"
@@ -127,6 +130,9 @@ void MidiSeq::processMsg(const ThreadMsg* m)
 
 void MidiSeq::processStop()
       {
+      // p3.3.28
+      playStateExt = false; // not playing
+      
       //
       //    stop stuck notes
       //
@@ -269,6 +275,10 @@ signed int MidiSeq::selectTimer()
     }
     delete timer;
     timer=NULL;
+    QMessageBox::critical( 0, /*tr*/(QString("Failed to start timer!")),
+              /*tr*/(QString("No functional timer was available.\n"
+                         "RTC timer not available, check if /dev/rtc is available and readable by current user\n"
+                         "Alsa timer not available, check if module snd_timer is available and /dev/snd/timer is available")));
     printf("No functional timer available!!!\n");
     exit(1);
     }
@@ -374,22 +384,29 @@ void MidiSeq::updatePollFd()
       for (iMidiDevice imd = midiDevices.begin(); imd != midiDevices.end(); ++imd) {
             MidiDevice* dev = *imd;
             int port = dev->midiPort();
+            const QString name = dev->name();
             if (port == -1)
                   continue;
             if ((dev->rwFlags() & 0x2) || (extSyncFlag.value()
                //&& (rxSyncPort == port || rxSyncPort == -1))) {
                //&& (dev->syncInfo().MCIn()))) {
                && (midiPorts[port].syncInfo().MCIn()))) {
+                  if(dev->selectRfd() < 0){
+                    //fprintf(stderr, "WARNING: read-file-descriptor for {%s} is negative\n", name.latin1());
+                  }
                   addPollFd(dev->selectRfd(), POLLIN, ::midiRead, this, dev);
                   }
-            if (dev->bytesToWrite())
+            if (dev->bytesToWrite()){
+                  if(dev->selectWfd() < 0){
+                    //fprintf(stderr, "WARNING: write-file-descriptor for {%s} is negative\n", name.latin1());
+                  }
                   addPollFd(dev->selectWfd(), POLLOUT, ::midiWrite, this, dev);
+            }      
             }
       // special handling for alsa midi:
       // (one fd for all devices)
       //    this allows for processing of some alsa events
       //    even if no alsa driver is active (assigned to a port)
-
       addPollFd(alsaSelectRfd(), POLLIN, ::alsaMidiRead, this, 0);
       }
 
@@ -665,21 +682,33 @@ void MidiSeq::processTimerTick()
             // printf("Midi Time Code Sync generation not impl.\n");
 //            }
 
+      // p3.3.25
+      int tickpos = audio->tickPos();
+      bool extsync = extSyncFlag.value();
       //
       // play all events upto curFrame
       //
       for (iMidiDevice id = midiDevices.begin(); id != midiDevices.end(); ++id) {
             MidiDevice* md = *id;
+            // Is it a Jack midi device? p3.3.36 
+            //MidiJackDevice* mjd = dynamic_cast<MidiJackDevice*>(md);
+            //if(mjd)
+            if(md->deviceType() == MidiDevice::JACK_MIDI)
+              continue;
+            if(md->isSynti())      // syntis are handled by audio thread
+                  continue;
             int port = md->midiPort();
             MidiPort* mp = port != -1 ? &midiPorts[port] : 0;
-            if (md->isSynti())      // syntis are handled by audio thread
-                  continue;
             MPEventList* el = md->playEvents();
             if (el->empty())
                   continue;
             iMPEvent i = md->nextPlayEvent();
             for (; i != el->end(); ++i) {
-                  if (i->time() > curFrame) {
+                  // p3.3.25
+                  // If syncing to external midi sync, we cannot use the tempo map.
+                  // Therefore we cannot get sub-tick resolution. Just use ticks instead of frames.
+                  //if (i->time() > curFrame) {
+                  if (i->time() > (extsync ? tickpos : curFrame)) {
                         //printf("  curT %d  frame %d\n", i->time(), curFrame);
                         break; // skip this event
                         }
