@@ -1,7 +1,7 @@
 //=========================================================
 //  MusE
 //  Linux Music Editor
-//  $Id: meter.h,v 1.1.1.1 2003/10/27 18:52:17 wschweer Exp $
+//  $Id: meter.h,v 1.1.1.1.2.2 2009/05/03 04:14:00 terminator356 Exp $
 //
 //  (C) Copyright 2000 Werner Schweer (ws@seh.de)
 //=========================================================
@@ -13,6 +13,10 @@
 #include <qpixmap.h>
 
 class Meter : public QFrame {
+   public:
+      enum MeterType {DBMeter, LinMeter};
+   private:  
+      MeterType mtype;
       bool overflow;
       double val;
       double maxVal;
@@ -21,6 +25,8 @@ class Meter : public QFrame {
 
       QPixmap bgPm;
       QPixmap fgPm;  // for double buffering
+      
+      void drawVU(int w, int h);
 
       Q_OBJECT
       virtual void drawContents(QPainter* p);
@@ -29,15 +35,15 @@ class Meter : public QFrame {
 
    public slots:
       void resetPeaks();
-      void setVal(int, int, bool);
+      //void setVal(int, int, bool);
+      void setVal(double, double, bool);
 
    signals:
       void mousePress();
 
    public:
-      Meter(QWidget* parent);
+      Meter(QWidget* parent, MeterType type = DBMeter);
       void setRange(double min, double max);
       };
-
 #endif
 
