@@ -17,11 +17,15 @@
 #include <qcombobox.h>
 #include <qtooltip.h>
 #include <qtimer.h>
+//#include <qpopupmenu.h>
+#include <qcursor.h>
 
 #include <math.h>
+#include "app.h"
 #include "midi.h"
 #include "midictrl.h"
 #include "mstrip.h"
+#include "midiport.h"
 #include "globals.h"
 #include "audio.h"
 #include "song.h"
@@ -38,6 +42,7 @@
 #include "gconfig.h"
 #include "ttoolbutton.h"
 //#include "utils.h"
+#include "popupmenu.h"
 
 enum { KNOB_PAN, KNOB_VAR_SEND, KNOB_REV_SEND, KNOB_CHO_SEND };
 
@@ -311,7 +316,8 @@ MidiStrip::MidiStrip(QWidget* parent, MidiTrack* t)
         useSoloIconSet2 = false;
       }  
       
-      QToolTip::add(solo, tr("pre fader listening"));
+      //QToolTip::add(solo, tr("pre fader listening"));
+      QToolTip::add(solo, tr("solo mode"));
       solo->setToggleButton(true);
       solo->setOn(t->solo());
       solo->setFixedWidth(STRIP_WIDTH/2);
@@ -342,7 +348,7 @@ MidiStrip::MidiStrip(QWidget* parent, MidiTrack* t)
       dev_ch_label->setFont(config.fonts[1]);
       // Dealing with a horizontally constrained label. Ignore vertical. Use a minimum readable point size.
       //autoAdjustFontSize(dev_ch_label, dev_ch_label->text(), false, true, config.fonts[6].pointSize(), 5);
-      QToolTip::add(dev_ch_label, tr("output: device - channel"));
+      QToolTip::add(dev_ch_label, tr("output port and channel"));
       smBox1->addWidget(dev_ch_label);
       smBox1->addWidget(record);
       
@@ -350,15 +356,36 @@ MidiStrip::MidiStrip(QWidget* parent, MidiTrack* t)
       layout->addLayout(smBox2);
 
       //---------------------------------------------------
-      //    output routing
+      //    routing
       //---------------------------------------------------
 
-      route = new QToolButton(this);
-      route->setFont(config.fonts[1]);
-      route->setFixedWidth(STRIP_WIDTH);
-      route->setText(tr("Route"));
-      QToolTip::add(route, tr("set routing"));
-      layout->addWidget(route);
+      // p3.3.38
+      //route = new QToolButton(this);
+      //route->setFont(config.fonts[1]);
+      //route->setFixedWidth(STRIP_WIDTH);
+      //route->setText(tr("Route"));
+      //QToolTip::add(route, tr("set routing"));
+      //layout->addWidget(route);
+      QHBoxLayout* rBox = new QHBoxLayout(0);
+      iR = new QToolButton(this);
+      iR->setFont(config.fonts[1]);
+      iR->setFixedWidth((STRIP_WIDTH-4)/2);
+      iR->setText(tr("iR"));
+      iR->setToggleButton(false);
+      QToolTip::add(iR, tr("input routing"));
+      rBox->addWidget(iR);
+      connect(iR, SIGNAL(pressed()), SLOT(iRoutePressed()));
+      oR = new QToolButton(this);
+      oR->setFont(config.fonts[1]);
+      oR->setFixedWidth((STRIP_WIDTH-4)/2);
+      oR->setText(tr("oR"));
+      oR->setToggleButton(false);
+      // TODO: Works OK, but disabled for now, until we figure out what to do about multiple out routes and display values...
+      oR->setEnabled(false);
+      QToolTip::add(oR, tr("output routing"));
+      rBox->addWidget(oR);
+      connect(oR, SIGNAL(pressed()), SLOT(oRoutePressed()));
+      layout->addLayout(rBox);
 
       //---------------------------------------------------
       //    automation mode
@@ -432,6 +459,13 @@ void MidiStrip::songChanged(int val)
         //label->setFont(config.fonts[1]);
         setLabelFont();
       }  
+      
+      // p3.3.47 Update the routing popup menu if anything relevant changes.
+      //if(gRoutingPopupMenuMaster == this && track && (val & (SC_ROUTE | SC_CHANNELS | SC_CONFIG))) 
+      if(val & (SC_ROUTE | SC_CHANNELS | SC_CONFIG))      // p3.3.50
+        // Use this handy shared routine.
+        //muse->updateRouteMenus(track);
+        muse->updateRouteMenus(track, this);              // p3.3.50
     }
 
 //---------------------------------------------------------
@@ -517,6 +551,7 @@ void MidiStrip::labelDoubleClicked(int idx)
 }
 
 
+/*
 //---------------------------------------------------------
 //   routeClicked
 //---------------------------------------------------------
@@ -524,6 +559,7 @@ void MidiStrip::labelDoubleClicked(int idx)
 void MidiStrip::routeClicked()
       {
       }
+*/
 
 //---------------------------------------------------------
 //   heartBeat
@@ -584,7 +620,6 @@ void MidiStrip::updateControls()
               //slider->blockSignals(true);
               if(double(nvolume) != slider->value())
               {
-                // Added by Tim. p3.3.6
                 //printf("MidiStrip::updateControls setting volume slider\n");
                 
                 slider->setValue(double(nvolume));
@@ -596,7 +631,6 @@ void MidiStrip::updateControls()
             int ivol = nvolume;
             nvolume -= ctrl->bias();
             if(nvolume != volume) {
-                // Added by Tim. p3.3.6
                 //printf("MidiStrip::updateControls setting volume slider\n");
                 
                 //slider->blockSignals(true);
@@ -604,7 +638,6 @@ void MidiStrip::updateControls()
                 //sl->setValue(double(nvolume));
                 if(ivol == 0)
                 {
-                  // Added by Tim. p3.3.6
                   //printf("MidiStrip::updateControls setting volume slider label\n");  
                   
                   sl->setValue(sl->minValue() - 0.5 * (sl->minValue() - sl->off()));
@@ -614,14 +647,12 @@ void MidiStrip::updateControls()
                   double v = -fast_log10(float(127*127)/float(ivol*ivol))*20.0;
                   if(v > sl->maxValue())
                   {
-                    // Added by Tim. p3.3.6
                     //printf("MidiStrip::updateControls setting volume slider label\n");
                     
                     sl->setValue(sl->maxValue());
                   }  
                   else  
                   {
-                    // Added by Tim. p3.3.6
                     //printf("MidiStrip::updateControls setting volume slider label\n");
                     
                     sl->setValue(v);
@@ -651,7 +682,6 @@ void MidiStrip::updateControls()
               npan -= ctrl->bias();
               if(double(npan) != gcon->knob->value())
               {
-                // Added by Tim. p3.3.6
                 //printf("MidiStrip::updateControls setting pan knob\n");
                 
                 gcon->knob->setValue(double(npan));
@@ -663,7 +693,6 @@ void MidiStrip::updateControls()
             npan -= ctrl->bias();
             if(npan != pan) 
             {
-                // Added by Tim. p3.3.6
                 //printf("MidiStrip::updateControls setting pan label and knob\n");
                 
                 //controller[KNOB_PAN].knob->blockSignals(true);
@@ -928,3 +957,257 @@ void MidiStrip::updateOffState() // Ripped from AudioStrip, hehh(mg)
       if (mute)
             mute->setEnabled(val);
       }
+
+//---------------------------------------------------------
+//   routingPopupMenuActivated
+//---------------------------------------------------------
+
+void MidiStrip::routingPopupMenuActivated(int n)
+{
+  if(gRoutingPopupMenuMaster != this || !track || !track->isMidiTrack())
+    return;
+  muse->routingPopupMenuActivated(track, n);
+}
+
+//---------------------------------------------------------
+//   iRoutePressed
+//---------------------------------------------------------
+
+void MidiStrip::iRoutePressed()
+{
+  if(!track || !track->isMidiTrack())
+    return;
+  
+  //song->chooseMidiRoutes(iR, (MidiTrack*)track, false);
+  PopupMenu* pup = muse->prepareRoutingPopupMenu(track, false);
+  if(!pup)
+    return;
+  
+  //pup->disconnect();
+  gRoutingPopupMenuMaster = this;
+  connect(pup, SIGNAL(activated(int)), SLOT(routingPopupMenuActivated(int)));
+  connect(pup, SIGNAL(aboutToHide()), muse, SLOT(routingPopupMenuAboutToHide()));
+  pup->popup(QCursor::pos(), 0);
+  iR->setDown(false);     
+  return;
+  
+  /*
+  RouteList* irl = track->inRoutes();
+  //Route dst(track, -1);
+
+  QPopupMenu* pup = new QPopupMenu(iR);
+  pup->setCheckable(true);
+  
+  int gid = 0;
+  
+  //MidiInPortList* tl = song->midiInPorts();
+  //for(iMidiInPort i = tl->begin();i != tl->end(); ++i) 
+  for(int i = 0; i < MIDI_PORTS; ++i)
+  {
+    //MidiInPort* track = *i;
+    // NOTE: Could possibly list all devices, bypassing ports, but no, let's stick wth ports.
+    MidiPort* mp = &midiPorts[i];
+    MidiDevice* md = mp->device();
+    if(!md)
+      continue;
+    
+    if(!(md->rwFlags() & 2))
+      continue;
+      
+    //printf("MidiStrip::iRoutePressed adding submenu portnum:%d\n", i);
+    
+    //QMenu* m = menu->addMenu(track->name());
+    QPopupMenu* subp = new QPopupMenu(iR);
+    
+    for(int ch = 0; ch < MIDI_CHANNELS; ++ch) 
+    {
+      //QAction* a = m->addAction(QString("Channel %1").arg(ch+1));
+      //subp->insertItem(QT_TR_NOOP(QString("Channel %1").arg(ch+1)), i * MIDI_CHANNELS + ch);
+      gid = i * MIDI_CHANNELS + ch;
+      
+      //printf("MidiStrip::iRoutePressed inserting gid:%d\n", gid);
+      
+      subp->insertItem(QString("Channel %1").arg(ch+1), gid);
+      //a->setCheckable(true);
+      //Route src(track, ch, RouteNode::TRACK);
+      //Route src(md, ch);
+      //Route r = Route(src, dst);
+      //a->setData(QVariant::fromValue(r));
+      //a->setChecked(rl->indexOf(r) != -1);
+      Route srcRoute(md, ch);
+      for(iRoute ir = irl->begin(); ir != irl->end(); ++ir) 
+      {
+        //if(*ir == dst) 
+        if(*ir == srcRoute) 
+        {
+          subp->setItemChecked(gid, true);
+          break;
+        }
+      }
+    }
+    pup->insertItem(QT_TR_NOOP(md->name()), subp);
+  }
+      
+      int n = pup->exec(QCursor::pos());
+      delete pup;
+      if (n != -1) 
+      {
+            int mdidx = n / MIDI_CHANNELS;
+            int ch = n % MIDI_CHANNELS;
+            
+            //if(debugMsg)
+              printf("MidiStrip::iRoutePressed mdidx:%d ch:%d\n", mdidx, ch);
+              
+            MidiPort* mp = &midiPorts[mdidx];
+            MidiDevice* md = mp->device();
+            if(!md)
+              return;
+            
+            if(!(md->rwFlags() & 2))
+              return;
+              
+            
+            //QString s(pup->text(n));
+            //QT_TR_NOOP(md->name())
+            
+            //Route srcRoute(s, false, -1);
+            Route srcRoute(md, ch);
+            //Route srcRoute(md, -1);
+            //Route dstRoute(track, -1);
+            Route dstRoute(track, ch);
+
+            //if (track->type() == Track::AUDIO_INPUT)
+            //      srcRoute.channel = dstRoute.channel = n & 0xf;
+            iRoute iir = irl->begin();
+            for (; iir != irl->end(); ++iir) {
+                  if (*iir == srcRoute)
+                        break;
+                  }
+            if (iir != irl->end()) {
+                  // disconnect
+                  printf("MidiStrip::iRoutePressed removing route src device name: %s dst track name: %s\n", md->name().latin1(), track->name().latin1());
+                  audio->msgRemoveRoute(srcRoute, dstRoute);
+                  }
+            else {
+                  // connect
+                  printf("MidiStrip::iRoutePressed adding route src device name: %s dst track name: %s\n", md->name().latin1(), track->name().latin1());
+                  audio->msgAddRoute(srcRoute, dstRoute);
+                  }
+            printf("MidiStrip::iRoutePressed calling msgUpdateSoloStates\n");
+            audio->msgUpdateSoloStates();
+            printf("MidiStrip::iRoutePressed calling song->update\n");
+            song->update(SC_ROUTE);
+      }
+      //delete pup;
+      iR->setDown(false);     // pup->exec() catches mouse release event
+      printf("MidiStrip::iRoutePressed end\n");
+      */
+      
+}
+
+//---------------------------------------------------------
+//   oRoutePressed
+//---------------------------------------------------------
+
+void MidiStrip::oRoutePressed()
+      {
+  if(!track || !track->isMidiTrack())
+    return;
+  
+  //song->chooseMidiRoutes(oR, (MidiTrack*)track, true);
+  PopupMenu* pup = muse->prepareRoutingPopupMenu(track, true);
+  if(!pup)
+    return;
+  
+  //pup->disconnect();
+  gRoutingPopupMenuMaster = this;
+  connect(pup, SIGNAL(activated(int)), SLOT(routingPopupMenuActivated(int)));
+  connect(pup, SIGNAL(aboutToHide()), muse, SLOT(routingPopupMenuAboutToHide()));
+  pup->popup(QCursor::pos(), 0);
+  oR->setDown(false);     
+  return;
+      
+      /*
+      QPopupMenu* pup = new QPopupMenu(oR);
+      pup->setCheckable(true);
+      AudioTrack* t = (AudioTrack*)track;
+      RouteList* orl = t->outRoutes();
+
+      switch(track->type()) {
+            case Track::MIDI:
+            case Track::DRUM:
+                  delete pup;
+                  return;
+            case Track::AUDIO_OUTPUT:
+                  {
+                  int gid = 0;
+                  for (int i = 0; i < channel; ++i) {
+                        char buffer[128];
+                        snprintf(buffer, 128, "%s %d", tr("Channel").latin1(), i+1);
+                        MenuTitleItem* titel = new MenuTitleItem(QString(buffer));
+                        pup->insertItem(titel);
+
+                        if (!checkAudioDevice()) return;
+                        std::list<QString> ol = audioDevice->inputPorts();
+                        for (std::list<QString>::iterator ip = ol.begin(); ip != ol.end(); ++ip) {
+                              int id = pup->insertItem(*ip, (gid * 16) + i);
+                              Route dst(*ip, true, i);
+                              ++gid;
+                              for (iRoute ir = orl->begin(); ir != orl->end(); ++ir) {
+                                    if (*ir == dst) {
+                                          pup->setItemChecked(id, true);
+                                          break;
+                                          }
+                                    }
+                              }
+                        if (i+1 != channel)
+                              pup->insertSeparator();
+                        }
+                  }
+                  break;
+            case Track::AUDIO_INPUT:
+                  addWavePorts(t, pup, orl);
+            case Track::WAVE:
+            case Track::AUDIO_GROUP:
+            case Track::AUDIO_SOFTSYNTH:
+                  addOutPorts(t, pup, orl);
+                  addGroupPorts(t, pup, orl);
+                  break;
+            case Track::AUDIO_AUX:
+                  addOutPorts(t, pup, orl);
+                  break;
+            }
+      int n = pup->exec(QCursor::pos());
+      if (n != -1) {
+            QString s(pup->text(n));
+            Route srcRoute(t, -1);
+            Route dstRoute(s, true, -1);
+
+            if (track->type() == Track::AUDIO_OUTPUT)
+                  srcRoute.channel = dstRoute.channel = n & 0xf;
+
+            // check if route src->dst exists:
+            iRoute iorl = orl->begin();
+            for (; iorl != orl->end(); ++iorl) {
+                  if (*iorl == dstRoute)
+                        break;
+                  }
+            if (iorl != orl->end()) {
+                  // disconnect if route exists
+                  audio->msgRemoveRoute(srcRoute, dstRoute);
+                  }
+            else {
+                  // connect if route does not exist
+                  audio->msgAddRoute(srcRoute, dstRoute);
+                  }
+            audio->msgUpdateSoloStates();
+            song->update(SC_ROUTE);
+            }
+      delete pup;
+      oR->setDown(false);     // pup->exec() catches mouse release event
+      */
+      
+      
+      }
+
+
