@@ -1,7 +1,7 @@
 //=========================================================
 //  MusE
 //  Linux Music Editor
-//  $Id: rack.h,v 1.5.2.2 2006/02/11 22:21:21 spamatica Exp $
+//  $Id: rack.h,v 1.5.2.3 2006/09/24 19:32:31 terminator356 Exp $
 //
 //  (C) Copyright 2000 Werner Schweer (ws@seh.de)
 //=========================================================
@@ -45,6 +45,9 @@ class EffectRack : public QListBox {
    public:
       EffectRack(QWidget*, AudioTrack* t);
       ~EffectRack();
+      
+      AudioTrack* getTrack() { return track; } 
+      QPoint getDragPos() { return dragPos; }
       };
 
 #endif
