@@ -1,7 +1,7 @@
 //=========================================================
 //  MusE
 //  Linux Music Editor
-//  $Id: midiitransform.cpp,v 1.2 2003/12/29 11:59:03 wschweer Exp $
+//  $Id: midiitransform.cpp,v 1.2.2.2 2009/05/24 21:43:44 terminator356 Exp $
 //
 //  (C) Copyright 2001-2003 Werner Schweer (ws@seh.de)
 //=========================================================
@@ -19,6 +19,8 @@
 #include <qtoolbutton.h>
 
 #include "spinboxFP.h"
+#include "midi.h"
+#include "midictrl.h"
 #include "event.h"
 #include "mpevent.h"
 #include "midiitransform.h"
@@ -29,9 +31,24 @@
 #include "comboQuant.h"
 #include "pitchedit.h"
 
-static int eventTypeTable[] = {
-      1, 6, 4, 7, 8, 10, 11
+#define MIDITRANSFORM_NOTE        0
+#define MIDITRANSFORM_POLY        1
+#define MIDITRANSFORM_CTRL        2
+#define MIDITRANSFORM_ATOUCH      3
+#define MIDITRANSFORM_PITCHBEND   4
+#define MIDITRANSFORM_NRPN        5
+#define MIDITRANSFORM_RPN         6
+
+static int selTypeTable[] = {
+      MIDITRANSFORM_NOTE, MIDITRANSFORM_POLY, MIDITRANSFORM_CTRL, MIDITRANSFORM_ATOUCH,
+         MIDITRANSFORM_PITCHBEND, MIDITRANSFORM_NRPN, MIDITRANSFORM_RPN
       };
+      
+static int procTypeTable[] = {
+      MIDITRANSFORM_POLY, MIDITRANSFORM_CTRL, MIDITRANSFORM_ATOUCH,
+         MIDITRANSFORM_PITCHBEND, MIDITRANSFORM_NRPN, MIDITRANSFORM_RPN
+      };
+
 static int procVal2Map[] = { 0, 1, 2, 3, 4, 5, 6, 7, 10, 11 };
 
 struct TDict {
@@ -66,8 +83,9 @@ class MidiInputTransformation {
       ValOp selChannel;
       int selChannela, selChannelb;
 
-      TransformOperator procEvent;
+      InputTransformProcEventOp procEvent;
       int eventType;
+      
       TransformOperator procVal1;
       int procVal1a, procVal1b;
       TransformOperator procVal2;
@@ -83,15 +101,15 @@ class MidiInputTransformation {
       MidiInputTransformation(const QString& s) {
             name         = s;
             selEventOp   = All;
-            selType      = 0x90;
+            selType      = MIDITRANSFORM_NOTE;
             selVal1      = Ignore;
             selVal1a     = 0;
             selVal1b     = 0;
             selVal2      = Ignore;
             selVal2a     = 0;
             selVal2b     = 0;
-            procEvent    = Keep;
-            eventType    = 0x90;
+            procEvent    = KeepType;
+            eventType    = MIDITRANSFORM_POLY;
             procVal1     = Keep;
             procVal1a    = 0;
             procVal1b    = 0;
@@ -113,6 +131,7 @@ class MidiInputTransformation {
             }
       void write(int level, Xml& xml) const;
       int apply(MidiRecordEvent& ev) const;
+      bool typesMatch(MidiRecordEvent& e, int selType) const;
       };
 
 typedef std::list<MidiInputTransformation*> MidiInputTransformationList;
@@ -199,29 +218,31 @@ static bool filterValOp(ValOp op, int val, int val1, int val2)
 
 int MidiInputTransformation::apply(MidiRecordEvent& event) const
       {
+      int t = event.type();
+      
       switch (selEventOp) {
             case Equal:
-                  switch(event.type()) {
-                        case 0x90:
-                        case 0x80:
-                              if (selType != 0x90)
+                  switch(t) {
+                        case ME_NOTEON:
+                        case ME_NOTEOFF:
+                              if (selType != MIDITRANSFORM_NOTE)
                                     return 0;
                               break;
                         default:
-                              if (event.type() != selType)
+                              if(!typesMatch(event, selType))
                                     return 0;
                               break;
                         }
                   break;
             case Unequal:
                   switch(event.type()) {
-                        case 0x90:
-                        case 0x80:
-                              if (selType == 0x90)
+                        case ME_NOTEON:
+                        case ME_NOTEOFF:
+                              if (selType == MIDITRANSFORM_NOTE)
                                     return 0;
                               break;
                         default:
-                              if (event.type() == selType)
+                              if(typesMatch(event, selType))
                                     return 0;
                               break;
                         }
@@ -243,8 +264,40 @@ int MidiInputTransformation::apply(MidiRecordEvent& event) const
 
       // transform event
 //printf("transform\n");
-      if (procEvent != Keep)
-            event.setType(eventType);
+      if (procEvent != KeepType)
+      {
+        switch(eventType)
+        {
+          case MIDITRANSFORM_POLY:
+            event.setType(ME_POLYAFTER);
+          break;
+          case MIDITRANSFORM_CTRL:
+            event.setType(ME_CONTROLLER);
+          break;
+          case MIDITRANSFORM_ATOUCH:
+            event.setType(ME_AFTERTOUCH);
+          break;
+          case MIDITRANSFORM_PITCHBEND:
+          {
+            event.setType(ME_PITCHBEND);
+          }  
+          break;
+          case MIDITRANSFORM_NRPN: 
+          {
+            event.setA(MidiController::NRPN);
+            event.setType(ME_CONTROLLER);
+          }  
+          break;
+          case MIDITRANSFORM_RPN:
+          {
+            event.setA(MidiController::RPN);
+            event.setType(ME_CONTROLLER);
+          }  
+          break;
+          default:
+          break;
+        }
+      }
 
       //---------------------------------------------------
       //    transform value A
@@ -461,6 +514,60 @@ int MidiInputTransformation::apply(MidiRecordEvent& event) const
       return 2;
       }
 
+//---------------------------------------------------------
+//   typesMatch
+//---------------------------------------------------------
+
+bool MidiInputTransformation::typesMatch(MidiRecordEvent& e, int selType) const
+      {
+      bool matched = false;
+      int t = e.type();
+      switch (selType)
+         {
+         case MIDITRANSFORM_NOTE:
+            matched = ((t == ME_NOTEON) || (t == ME_NOTEOFF));
+            break;
+         case MIDITRANSFORM_POLY:
+            matched = (t == ME_POLYAFTER);
+            break;
+         case MIDITRANSFORM_CTRL:
+            matched = (t == ME_CONTROLLER);
+            break;
+         case MIDITRANSFORM_ATOUCH:
+            matched = (t == ME_AFTERTOUCH);
+            break;
+         case MIDITRANSFORM_PITCHBEND:
+            {
+            //if (t == ME_CONTROLLER) {
+            //      MidiController::ControllerType c = midiControllerType(e.dataA());
+            //      matched = (c == MidiController::Pitch);
+            matched = (t = ME_PITCHBEND);
+            }
+            break;
+         case MIDITRANSFORM_NRPN:
+            {
+            if (t == ME_CONTROLLER) {
+                  MidiController::ControllerType c = midiControllerType(e.dataA());
+                  matched = (c == MidiController::NRPN);
+                  }
+            }
+            break;
+         case MIDITRANSFORM_RPN:
+            {
+            if (t == ME_CONTROLLER) {
+                  MidiController::ControllerType c = midiControllerType(e.dataA());
+                  matched = (c == MidiController::RPN);
+                  }
+            }
+            break;
+         default:
+            fprintf(stderr, "Error matching type in MidiTransformerDialog: unknown eventtype!\n");
+            break;
+         }
+      //printf("Event type=%d, selType =%d matched=%d\n", e.type(), selType, matched);
+      return matched;
+      }
+
 //---------------------------------------------------------
 //   MidiInputTransformDialog
 //    Widgets:
@@ -551,6 +658,32 @@ MidiInputTransformDialog::MidiInputTransformDialog(QWidget* parent,
       //  populate preset list
       //---------------------------------------------------
 
+      updatePresetList();
+      connect(song, SIGNAL(songChanged(int)), SLOT(songChanged(int)));
+      }
+
+//---------------------------------------------------------
+//   songChanged
+//---------------------------------------------------------
+
+void MidiInputTransformDialog::songChanged(int flags)
+{
+  // Whenever a song is loaded, flags is -1. Since transforms are part of configuration, 
+  //  use SC_CONFIG here, to filter unwanted song change events.
+  if(flags & SC_CONFIG)
+    updatePresetList();
+}
+
+//---------------------------------------------------------
+//   updatePresetList
+//---------------------------------------------------------
+
+void MidiInputTransformDialog::updatePresetList()
+{
+      cmt = 0;
+      presetList->clear();
+      
+      modul1select->setOn(true);
       for (iMidiInputTransformation i = mtlist.begin(); i != mtlist.end(); ++i) {
             presetList->insertItem((*i)->name);
             if (cmt == 0)
@@ -561,16 +694,15 @@ MidiInputTransformDialog::MidiInputTransformDialog(QWidget* parent,
             cmt = new MidiInputTransformation(tr("New"));
             mtlist.push_back(cmt);
             presetList->insertItem(tr("New"));
+            presetList->setCurrentItem(0);
             }
-      presetList->setCurrentItem(0);
-      modul1select->setOn(true);
       changeModul(0);
 
       modul1enable->setChecked(modules[0].valid);
       modul2enable->setChecked(modules[1].valid);
       modul3enable->setChecked(modules[2].valid);
       modul4enable->setChecked(modules[3].valid);
-      }
+}
 
 //---------------------------------------------------------
 //   closeEvent
@@ -615,7 +747,7 @@ void MidiInputTransformation::write(int level, Xml& xml) const
             xml.intTag(level, "quantVal", quantVal);
             }
       if (funcOp == Transform || funcOp == Insert) {
-            if (procEvent != Keep) {
+            if (procEvent != KeepType) {
                   xml.intTag(level, "procEventOp", int(procEvent));
                   xml.intTag(level, "eventType", int(eventType));
                   }
@@ -693,7 +825,7 @@ void readMidiInputTransform(Xml& xml)
                         else if (tag == "quantVal")
                               trans.quantVal = xml.parseInt();
                         else if (tag == "procEventOp")
-                              trans.procEvent = TransformOperator(xml.parseInt());
+                              trans.procEvent = InputTransformProcEventOp(xml.parseInt());
                         else if (tag == "eventType")
                               trans.eventType = xml.parseInt();
                         else if (tag == "procVal1Op")
@@ -754,12 +886,23 @@ void readMidiInputTransform(Xml& xml)
                         else if (tag == "apply")
                               apply = xml.parseInt();
                         else
-                              xml.unknown("midiTransform");
+                              xml.unknown("midiInputTransform");
                         break;
                   case Xml::TagEnd:
                         if (xml.s1() == "midiInputTransform") {
 // printf("midi Input transform <%s> apply %d\n",
 //   trans.name.latin1(), apply);
+                              
+                              // By T356. A previous bug caused some .med files to grow very large
+                              //  with duplicate transforms. Here we can eliminate those duplicates.
+                              for(iMidiInputTransformation i = mtlist.begin(); i != mtlist.end(); ++i) 
+                              {
+                                if((*i)->name == trans.name)
+                                {
+                                  return;
+                                }  
+                              }
+                              
                               MidiInputTransformation* t = new MidiInputTransformation(trans);
                               // search free slot in modules
                               if (apply != -1) {
@@ -780,6 +923,26 @@ void readMidiInputTransform(Xml& xml)
             }
       }
 
+//---------------------------------------------------------
+//   clearMidiInputTransforms
+//---------------------------------------------------------
+
+void clearMidiInputTransforms()
+{
+  for (int i = 0; i < MIDI_INPUT_TRANSFORMATIONS; ++i) 
+  {
+    modules[i].transform = 0;
+    modules[i].valid = false;
+  }
+  for (iMidiInputTransformation i = mtlist.begin(); i != mtlist.end(); ++i) 
+  {
+    MidiInputTransformation* t = *i;
+    if(t)
+      delete t;
+  }
+  mtlist.clear();
+}
+
 //---------------------------------------------------------
 //   accept
 //---------------------------------------------------------
@@ -842,7 +1005,7 @@ void MidiInputTransformDialog::selEventOpSel(int val)
 
 void MidiInputTransformDialog::selTypeSel(int val)
       {
-      cmt->selType = eventTypeTable[val];
+      cmt->selType = selTypeTable[val];
       selVal1aChanged(cmt->selVal1a);
       selVal1bChanged(cmt->selVal1b);
       }
@@ -873,9 +1036,12 @@ void MidiInputTransformDialog::selVal2OpSel(int val)
 
 void MidiInputTransformDialog::procEventOpSel(int val)
       {
-      TransformOperator op = val == 0 ? Keep : Fix;
-      procType->setEnabled(op == Fix);
+      InputTransformProcEventOp op = val == 0 ? KeepType : FixType;
+      procType->setEnabled(op == FixType);
       cmt->procEvent = op;
+      
+      procVal1aChanged(cmt->procVal1a);
+      procVal1bChanged(cmt->procVal1b);
       }
 
 //---------------------------------------------------------
@@ -884,7 +1050,9 @@ void MidiInputTransformDialog::procEventOpSel(int val)
 
 void MidiInputTransformDialog::procEventTypeSel(int val)
       {
-      cmt->eventType = eventTypeTable[val];
+      cmt->eventType = procTypeTable[val];
+      procVal1aChanged(cmt->procVal1a);
+      procVal1bChanged(cmt->procVal1b);
       }
 
 //---------------------------------------------------------
@@ -923,6 +1091,8 @@ void MidiInputTransformDialog::procVal1OpSel(int val)
                   procVal1b->setEnabled(true);
                   break;
             }
+      procVal1aChanged(cmt->procVal1a);
+      procVal1bChanged(cmt->procVal1b);
       }
 
 //---------------------------------------------------------
@@ -1072,7 +1242,7 @@ void MidiInputTransformDialog::selVal1aChanged(int val)
       {
       cmt->selVal1a = val;
       if ((cmt->selEventOp != All)
-         && (cmt->selType == 0x90)) {
+         && (cmt->selType == MIDITRANSFORM_NOTE)) {
             selVal1a->setSuffix(" - " + pitch2string(val));
             }
       else
@@ -1087,7 +1257,7 @@ void MidiInputTransformDialog::selVal1bChanged(int val)
       {
       cmt->selVal1b = val;
       if ((cmt->selEventOp != All)
-         && (cmt->selType == 0x90)) {
+         && (cmt->selType == MIDITRANSFORM_NOTE)) {
             selVal1b->setSuffix(" - " + pitch2string(val));
             }
       else
@@ -1119,6 +1289,15 @@ void MidiInputTransformDialog::selVal2bChanged(int val)
 void MidiInputTransformDialog::procVal1aChanged(int val)
       {
       cmt->procVal1a = val;
+      
+      if((cmt->procEvent == KeepType && cmt->selType == MIDITRANSFORM_NOTE) && 
+           (cmt->procVal1 == Fix || cmt->procVal1 == ScaleMap || cmt->procVal1 == Dynamic || 
+            cmt->procVal1 == Random || cmt->procVal1 == Flip)) 
+        {
+            procVal1a->setSuffix(" - " + pitch2string(val));
+        }
+      else
+            procVal1a->setSuffix(QString(""));
       }
 
 //---------------------------------------------------------
@@ -1128,6 +1307,15 @@ void MidiInputTransformDialog::procVal1aChanged(int val)
 void MidiInputTransformDialog::procVal1bChanged(int val)
       {
       cmt->procVal1b = val;
+      
+      if((cmt->procEvent == KeepType && cmt->selType == MIDITRANSFORM_NOTE) && 
+           (cmt->procVal1 == Fix || cmt->procVal1 == ScaleMap || cmt->procVal1 == Dynamic || 
+            cmt->procVal1 == Random || cmt->procVal1 == Flip)) 
+        {
+            procVal1b->setSuffix(" - " + pitch2string(val));
+        }
+      else
+            procVal1b->setSuffix(QString(""));
       }
 
 //---------------------------------------------------------
@@ -1418,8 +1606,8 @@ void MidiInputTransformDialog::presetChanged(QListBoxItem* item)
       selEventOp->setCurrentItem(cmt->selEventOp);
       selEventOpSel(cmt->selEventOp);
 
-      for (unsigned i = 0; i < sizeof(eventTypeTable)/sizeof(*eventTypeTable); ++i) {
-            if (eventTypeTable[i] == cmt->selType) {
+      for (unsigned i = 0; i < sizeof(selTypeTable)/sizeof(*selTypeTable); ++i) {
+            if (selTypeTable[i] == cmt->selType) {
                   selType->setCurrentItem(i);
                   break;
                   }
@@ -1450,6 +1638,13 @@ void MidiInputTransformDialog::presetChanged(QListBoxItem* item)
       funcOpSel(i);
       }
 
+      for (unsigned i = 0; i < sizeof(procTypeTable)/sizeof(*procTypeTable); ++i) {
+            if (procTypeTable[i] == cmt->eventType) {
+                  procType->setCurrentItem(i);
+                  break;
+                  }
+            }
+
       procEventOp->setCurrentItem(cmt->procEvent);
       procEventOpSel(cmt->procEvent);
 
@@ -1488,5 +1683,12 @@ void MidiInputTransformDialog::presetChanged(QListBoxItem* item)
 
       procChannelVala->setValue(cmt->procChannela);
       procChannelValb->setValue(cmt->procChannelb);
+      
+      procPortOp->setCurrentItem(cmt->procPort);
+      procPortOpSel(cmt->procPort);
+      
+      procChannelOp->setCurrentItem(cmt->procChannel);
+      procChannelOpSel(cmt->procChannel);
+      
       }
 
