@@ -17,12 +17,14 @@
 #include <uuid/uuid.h>
 
 #include "event.h"
+#include "audioconvert.h"
 
 class Track;
 class MidiTrack;
 class WaveTrack;
 class Xml;
 class Part;
+//class AudioConvertMap;
 
 // typedef std::multimap<unsigned, Event*, std::less<unsigned> >::iterator iEvent;
 
@@ -121,6 +123,9 @@ class MidiPart : public Part {
 
 class WavePart : public Part {
 
+      // p3.3.31
+      AudioConvertMap _converters;
+      
    public:
       WavePart(WaveTrack* t);
       WavePart(WaveTrack* t, EventList* ev);
