@@ -21,6 +21,8 @@
 #include "alsamidi.h"
 #include "audio.h"
 #include "arranger.h"
+#include "plugin.h"
+#include "driver/jackmidi.h"
 
 //---------------------------------------------------------
 //   sendMsg
@@ -72,14 +74,58 @@ bool Audio::sendMessage(AudioMsg* m, bool doUndo)
 //---------------------------------------------------------
 
 void Audio::msgRemoveRoute(Route src, Route dst)
-      {
+{
       msgRemoveRoute1(src, dst);
-      if (!checkAudioDevice()) return;
-      if (src.type == JACK_ROUTE)
+      //if (!checkAudioDevice()) return;
+      if (src.type == Route::JACK_ROUTE)
+      {
+          if (!checkAudioDevice()) return;
+          
+          //if(dst.type == Route::JACK_MIDI_ROUTE)  
+          if(dst.type == Route::MIDI_DEVICE_ROUTE)  
+          {
+            //MidiJackDevice* jmd = dynamic_cast<MidiJackDevice*>(dst.device);
+            //if(jmd)
+            if(dst.device)
+            {
+              if(dst.device->deviceType() == MidiDevice::JACK_MIDI)
+                audioDevice->disconnect(src.jackPort, dst.device->clientPort());
+              //else
+              //{
+                // TODO...
+                //MidiAlsaDevice* amd = dynamic_cast<MidiAlsaDevice*>(dst.device);
+                //if(amd)
+              //}  
+            }
+          }
+          else  
             audioDevice->disconnect(src.jackPort, ((AudioInput*)dst.track)->jackPort(dst.channel));
-      else if (dst.type == JACK_ROUTE)
+      }
+      else if (dst.type == Route::JACK_ROUTE)
+      {
+          if (!checkAudioDevice()) return;
+          
+          //if(src.type == Route::JACK_MIDI_ROUTE)  
+          if(src.type == Route::MIDI_DEVICE_ROUTE)  
+          {
+            //MidiJackDevice* jmd = dynamic_cast<MidiJackDevice*>(src.device);
+            //if(jmd)
+            if(src.device)
+            {
+              if(src.device->deviceType() == MidiDevice::JACK_MIDI)
+                audioDevice->disconnect(src.device->clientPort(), dst.jackPort);
+              //else
+              //{
+                // TODO...
+                //MidiAlsaDevice* amd = dynamic_cast<MidiAlsaDevice*>(src.device);
+                //if(amd)
+              //}
+            }  
+          }
+          else  
             audioDevice->disconnect(((AudioOutput*)src.track)->jackPort(src.channel), dst.jackPort);
       }
+}
 
 //---------------------------------------------------------
 //   msgRemoveRoute1
@@ -100,16 +146,58 @@ void Audio::msgRemoveRoute1(Route src, Route dst)
 
 void Audio::msgAddRoute(Route src, Route dst)
       {
-      if (src.type == JACK_ROUTE) {
+      if (src.type == Route::JACK_ROUTE) 
+      {
             if (!checkAudioDevice()) return;
             if (isRunning())
+            {
+                //if(dst.type == Route::JACK_MIDI_ROUTE)  
+                if(dst.type == Route::MIDI_DEVICE_ROUTE)  
+                {
+                  //MidiJackDevice* jmd = dynamic_cast<MidiJackDevice*>(dst.device);
+                  //if(jmd)
+                  if(dst.device)
+                  {
+                    if(dst.device->deviceType() == MidiDevice::JACK_MIDI)  
+                      audioDevice->connect(src.jackPort, dst.device->clientPort());
+                    //else
+                    //{
+                      // TODO...
+                      //MidiAlsaDevice* amd = dynamic_cast<MidiAlsaDevice*>(dst.device);
+                      //if(amd)
+                    //}
+                  }  
+                }
+                else  
                   audioDevice->connect(src.jackPort, ((AudioInput*)dst.track)->jackPort(dst.channel));
-            }
-      else if (dst.type == JACK_ROUTE) {
+            }      
+      }
+      else if (dst.type == Route::JACK_ROUTE) 
+      {
             if (!checkAudioDevice()) return;
             if (audio->isRunning())
+            {
+                //if(src.type == Route::JACK_MIDI_ROUTE)  
+                if(src.type == Route::MIDI_DEVICE_ROUTE)  
+                {
+                  //MidiJackDevice* jmd = dynamic_cast<MidiJackDevice*>(src.device);
+                  //if(jmd)
+                  if(src.device)
+                  {
+                    if(src.device->deviceType() == MidiDevice::JACK_MIDI)  
+                      audioDevice->connect(src.device->clientPort(), dst.jackPort);
+                    //else
+                    //{
+                      // TODO...
+                      //MidiAlsaDevice* amd = dynamic_cast<MidiAlsaDevice*>(src.device);
+                      //if(amd)
+                    //}
+                  }  
+                }
+                else  
                   audioDevice->connect(((AudioOutput*)src.track)->jackPort(dst.channel), dst.jackPort);
-            }
+            }      
+      }
       msgAddRoute1(src, dst);
       }
 
@@ -164,6 +252,7 @@ void Audio::msgSetVolume(AudioTrack* src, double val)
       msg.snode    = src;
       msg.dval  = val;
       sendMsg(&msg);
+      //muse->arranger->controllerChanged(src);
       }
 
 //---------------------------------------------------------
@@ -177,6 +266,7 @@ void Audio::msgSetPan(AudioTrack* node, double val)
       msg.snode = node;
       msg.dval  = val;
       sendMsg(&msg);
+      //muse->arranger->controllerChanged(node);
       }
 
 //---------------------------------------------------------
@@ -203,55 +293,117 @@ void Audio::msgSetChannels(AudioTrack* node, int n)
       QString name = node->name();
       int mc       = std::max(n, node->channels());
 
-      if (!name.isEmpty()) {
-            if (node->type() == Track::AUDIO_INPUT) {
-                  if (!checkAudioDevice()) return;
-                  AudioInput* ai = (AudioInput*)node;
-                  for (int i = 0; i < mc; ++i) {
-                        if (i < n && ai->jackPort(i) == 0) {
-                              char buffer[128];
-                              snprintf(buffer, 128, "%s-%d", name.latin1(), i);
-                              ai->setJackPort(i, audioDevice->registerInPort(buffer));
-                              }
-                        else if ((i >= n) && ai->jackPort(i)) {
-                              RouteList* ir = node->inRoutes();
-                              for (iRoute ii = ir->begin(); ii != ir->end(); ++ii) {
-                                    Route r = *ii;
-                                    if ((r.type == JACK_ROUTE) && (r.channel == i)) {
-                                          msgRemoveRoute(r, Route(node,i));
-                                          break;
-                                          }
-                                    }
-                              audioDevice->unregisterPort(ai->jackPort(i));
-                              ai->setJackPort(i, 0);
-                              }
-                        }
+      if (!name.isEmpty()) 
+      {
+            if (node->type() == Track::AUDIO_INPUT) 
+            {
+              if (!checkAudioDevice()) return;
+              AudioInput* ai = (AudioInput*)node;
+              for (int i = 0; i < mc; ++i) 
+              {
+                if (i < n && ai->jackPort(i) == 0) 
+                {
+                  char buffer[128];
+                  snprintf(buffer, 128, "%s-%d", name.latin1(), i);
+                  //ai->setJackPort(i, audioDevice->registerInPort(buffer));
+                  ai->setJackPort(i, audioDevice->registerInPort(buffer, false));
+                }
+                else if ((i >= n) && ai->jackPort(i)) 
+                {
+                  RouteList* ir = node->inRoutes();
+                  for (iRoute ii = ir->begin(); ii != ir->end(); ++ii) 
+                  {
+                    Route r = *ii;
+                    if ((r.type == Route::JACK_ROUTE) && (r.channel == i)) 
+                    {
+                      msgRemoveRoute(r, Route(node,i));
+                      break;
+                    }
                   }
-            else if (node->type() == Track::AUDIO_OUTPUT) {
+                  audioDevice->unregisterPort(ai->jackPort(i));
+                  ai->setJackPort(i, 0);
+                }
+              }      
+            }
+            else if (node->type() == Track::AUDIO_OUTPUT) 
+            {
                   if (!checkAudioDevice()) return;
                   AudioOutput* ao = (AudioOutput*)node;
-                  for (int i = 0; i < mc; ++i) {
+                  for (int i = 0; i < mc; ++i) 
+                  {
                         void* jp = ao->jackPort(i);
-                        if (i < n && jp == 0) {
+                        if (i < n && jp == 0) 
+                        {
                               char buffer[128];
                               snprintf(buffer, 128, "%s-%d", name.latin1(), i);
-                              ao->setJackPort(i, audioDevice->registerOutPort(buffer));
-                              }
-                        else if (i >= n && jp) {
+                              //ao->setJackPort(i, audioDevice->registerOutPort(buffer));
+                              ao->setJackPort(i, audioDevice->registerOutPort(buffer, false));
+                        }
+                        else if (i >= n && jp) 
+                        {
                               RouteList* ir = node->outRoutes();
-                              for (iRoute ii = ir->begin(); ii != ir->end(); ++ii) {
+                              for (iRoute ii = ir->begin(); ii != ir->end(); ++ii) 
+                              {
                                     Route r = *ii;
-                                    if ((r.type == JACK_ROUTE) && (r.channel == i)) {
+                                    if ((r.type == Route::JACK_ROUTE) && (r.channel == i)) 
+                                    {
                                           msgRemoveRoute(Route(node,i), r);
                                           break;
-                                          }
                                     }
+                              }
                               audioDevice->unregisterPort(jp);
                               ao->setJackPort(i, 0);
-                              }
                         }
                   }
             }
+      }      
+      
+      /* TODO TODO: Change all stereo routes to mono. 
+      // If we are going from stereo to mono we need to disconnect any stray synti 'mono last channel'...
+      if(n == 1 && node->channels() > 1)
+      {
+        // This should always happen - syntis are fixed channels, user cannot change them. But to be safe...
+        if(node->type() != Track::AUDIO_SOFTSYNTH) 
+        {
+          if(node->type() != Track::AUDIO_INPUT) 
+          {
+            RouteList* rl = node->inRoutes();
+            for(iRoute r = rl->begin(); r != rl->end(); ++r)
+            {
+              // Only interested in synth tracks.
+              if(r->type != Route::TRACK_ROUTE || r->track->type() != Track::AUDIO_SOFTSYNTH)
+                continue;  
+              // If it's the last channel...
+              if(r->channel + 1 == ((AudioTrack*)r->track)->totalOutChannels())
+              {
+                msgRemoveRoute(*r, Route(node, r->channel));
+                //msgRemoveRoute(r, Route(node, r->remoteChannel));
+                break;
+              }
+            }
+          }  
+        
+          if(node->type() != Track::AUDIO_OUTPUT) 
+          {
+            RouteList* rl = node->outRoutes();
+            for(iRoute r = rl->begin(); r != rl->end(); ++r)
+            {
+              // Only interested in synth tracks.
+              if(r->type != Route::TRACK_ROUTE || r->track->type() != Track::AUDIO_SOFTSYNTH)
+                continue;  
+              // If it's the last channel...
+              if(r->channel + 1 == ((AudioTrack*)r->track)->totalOutChannels())
+              {
+                msgRemoveRoute(Route(node, r->channel), *r);
+                //msgRemoveRoute(Route(node, r->remoteChannel), r);
+                break;
+              }
+            }
+          }  
+        }   
+      }
+      */        
+              
       AudioMsg msg;
       msg.id    = AUDIO_SET_CHANNELS;
       msg.snode = node;
@@ -263,15 +415,19 @@ void Audio::msgSetChannels(AudioTrack* node, int n)
 //   msgSetPluginCtrlVal
 //---------------------------------------------------------
 
-void Audio::msgSetPluginCtrlVal(PluginI* plugin, int param, double val)
+//void Audio::msgSetPluginCtrlVal(PluginI* plugin, int param, double val)
+// p3.3.43
+void Audio::msgSetPluginCtrlVal(AudioTrack* track, int param, double val)
 {
       AudioMsg msg;
       
       msg.id     = AUDIO_SET_PLUGIN_CTRL_VAL;
       msg.ival   = param;
-      msg.dval = val;
-      msg.plugin = plugin;
+      msg.dval   = val;
+      //msg.plugin = plugin;
+      msg.snode  = track;
       sendMsg(&msg);
+      //muse->arranger->controllerChanged(track);
 }
 
 //---------------------------------------------------------
@@ -287,6 +443,7 @@ void Audio::msgSwapControllerIDX(AudioTrack* node, int idx1, int idx2)
       msg.a      = idx1;
       msg.b      = idx2;
       sendMsg(&msg);
+      //muse->arranger->controllerChanged(node);
 }
 
 //---------------------------------------------------------
@@ -301,6 +458,7 @@ void Audio::msgClearControllerEvents(AudioTrack* node, int acid)
       msg.snode  = node;
       msg.ival   = acid;
       sendMsg(&msg);
+      //muse->arranger->controllerChanged(node);
 }
 
 //---------------------------------------------------------
@@ -344,6 +502,7 @@ void Audio::msgEraseACEvent(AudioTrack* node, int acid, int frame)
       msg.ival   = acid;
       msg.a      = frame; 
       sendMsg(&msg);
+      //muse->arranger->controllerChanged(node);
 }
 
 //---------------------------------------------------------
@@ -360,6 +519,7 @@ void Audio::msgEraseRangeACEvents(AudioTrack* node, int acid, int frame1, int fr
       msg.a      = frame1; 
       msg.b      = frame2; 
       sendMsg(&msg);
+      //muse->arranger->controllerChanged(node);
 }
 
 //---------------------------------------------------------
@@ -376,6 +536,7 @@ void Audio::msgAddACEvent(AudioTrack* node, int acid, int frame, double val)
       msg.a      = frame; 
       msg.dval   = val;
       sendMsg(&msg);
+      //muse->arranger->controllerChanged(node);
 }
 
 //---------------------------------------------------------
@@ -412,7 +573,11 @@ void Audio::msgSeek(const Pos& pos)
       {
       if (!checkAudioDevice()) return;
       //audioDevice->seekTransport(pos.frame());
+            // p3.3.23
+            //printf("Audio::msgSeek before audioDevice->seekTransport frame:%d\n", pos.frame());
       audioDevice->seekTransport(pos);
+            // p3.3.23
+            //printf("Audio::msgSeek after audioDevice->seekTransport frame:%d\n", pos.frame());
       }
 
 //---------------------------------------------------------
