@@ -35,6 +35,7 @@ void initShortCuts()
       defShrt(SHRT_GOTO_RIGHT,    Qt::Key_PageDown,  "Transport: Goto right marker" , GLOBAL_SHRT, "goto_right");
       defShrt(SHRT_TOGGLE_LOOP,   Qt::Key_Slash, "Transport: Toggle Loop section", GLOBAL_SHRT, "toggle_loop");
       defShrt(SHRT_START_REC,     Qt::Key_Asterisk, "Transport: Toggle Record",    GLOBAL_SHRT, "toggle_rec");
+      defShrt(SHRT_REC_CLEAR,     Qt::Key_Backspace, "Transport: Clear all rec enabled tracks",    GLOBAL_SHRT, "rec_clear");
       defShrt(SHRT_COPY,          Qt::CTRL + Qt::Key_C, "Edit: Copy", INVIS_SHRT, "copy");
       defShrt(SHRT_UNDO,          Qt::CTRL + Qt::Key_Z, "Edit: Undo", INVIS_SHRT, "undo");
       defShrt(SHRT_REDO,          Qt::CTRL + Qt::Key_Y, "Edit: Redo", INVIS_SHRT, "redo");
@@ -79,7 +80,9 @@ void initShortCuts()
       defShrt(SHRT_GLOBAL_SPLIT,          0, "Structure: Global split", ARRANG_SHRT,  "global_split");
       defShrt(SHRT_COPY_RANGE,            0, "Structure: Copy range", ARRANG_SHRT,    "copy_range");
       defShrt(SHRT_CUT_EVENTS,            0, "Structure: Cut events", ARRANG_SHRT,    "cut_events");
-      defShrt(SHRT_OPEN_MIXER,            Qt::Key_F10, "View: Open mixer window", ARRANG_SHRT, "toggle_mixer");
+      //defShrt(SHRT_OPEN_MIXER,            Qt::Key_F10, "View: Open mixer window", ARRANG_SHRT, "toggle_mixer");
+      defShrt(SHRT_OPEN_MIXER,            Qt::Key_F10, "View: Open mixer #1 window", ARRANG_SHRT, "toggle_mixer");
+      defShrt(SHRT_OPEN_MIXER2,           Qt::CTRL + Qt::Key_F10, "View: Open mixer #2 window", ARRANG_SHRT, "toggle_mixer2");
       defShrt(SHRT_OPEN_TRANSPORT,        Qt::Key_F11, "View: Toggle transport window", ARRANG_SHRT, "toggle_transport");
       defShrt(SHRT_OPEN_BIGTIME,          Qt::Key_F12, "View: Toggle bigtime window", ARRANG_SHRT,   "toggle_bigtime");
       defShrt(SHRT_OPEN_MARKER,           Qt::Key_F9, "View: Open marker window", ARRANG_SHRT,   "marker_window");
@@ -92,7 +95,7 @@ void initShortCuts()
       defShrt(SHRT_CONFIG_SHORTCUTS,      0, "Settings: Configure shortcuts", ARRANG_SHRT, "configure_shortcuts");
       defShrt(SHRT_CONFIG_METRONOME,      0, "Settings: Configure metronome", ARRANG_SHRT, "configure_metronome");
       defShrt(SHRT_CONFIG_MIDISYNC,       0, "Settings: Midi sync configuration", ARRANG_SHRT, "configure_midi_sync");
-      defShrt(SHRT_MIDI_FILE_CONFIG,      0, "Settings: Midi file export configuration", ARRANG_SHRT, "configure_midi_file");
+      defShrt(SHRT_MIDI_FILE_CONFIG,      0, "Settings: Midi file import/export configuration", ARRANG_SHRT, "configure_midi_file");
       defShrt(SHRT_APPEARANCE_SETTINGS,   0, "Settings: Appearance settings", ARRANG_SHRT, "configure_appearance_settings");
       defShrt(SHRT_CONFIG_MIDI_PORTS,     0, "Settings: Midi ports / Soft Synth", ARRANG_SHRT, "configure_midi_ports");
       defShrt(SHRT_CONFIG_AUDIO_PORTS,    0, "Settings: Audio subsystem configuration", ARRANG_SHRT, "configure_audio_ports");
@@ -126,6 +129,12 @@ void initShortCuts()
       defShrt(SHRT_SEL_BELOW,             Qt::Key_Down, "Edit: Select nearest part on track below", ARRANG_SHRT, "sel_part_below");
       defShrt(SHRT_SEL_BELOW_ADD,         Qt::SHIFT + Qt::Key_Down, "Edit: Add nearest part on track below", ARRANG_SHRT, "sel_part_below_add");
 
+//      defShrt(SHRT_INSERT,                Qt::CTRL+Qt::SHIFT+ Qt::Key_I, "Edit: Insert parts, moving time", ARRANG_SHRT, "insert_parts");
+//      defShrt(SHRT_INSERTMEAS,            Qt::CTRL+Qt::SHIFT+ Qt::Key_M, "Edit: Insert empty measure", ARRANG_SHRT, "insert_measure");
+
+      defShrt(SHRT_SEL_TRACK_ABOVE,       Qt::CTRL + Qt::Key_Up, "Select track above", ARRANG_SHRT, "sel_track_above");
+      defShrt(SHRT_SEL_TRACK_BELOW,       Qt::CTRL + Qt::Key_Down, "Select track below", ARRANG_SHRT, "sel_track_below");
+
       //-----------------------------------------------------------
 
       defShrt(SHRT_TRANSPOSE,       0, "Midi: Transpose", ARRANG_SHRT + PROLL_SHRT, "midi_transpose");
@@ -137,6 +146,8 @@ void initShortCuts()
       defShrt(SHRT_SELECT_INVERT,    Qt::CTRL + Qt::Key_I, "Edit: Invert selection", ARRANG_SHRT + PROLL_SHRT + DEDIT_SHRT,  "sel_inv");
       defShrt(SHRT_SELECT_ILOOP,     0, "Edit: Select events/parts inside locators", ARRANG_SHRT + PROLL_SHRT + DEDIT_SHRT,  "sel_ins_loc");
       defShrt(SHRT_SELECT_OLOOP,     0, "Edit: Select events/parts outside locators", ARRANG_SHRT + PROLL_SHRT + DEDIT_SHRT, "sel_out_loc");
+      defShrt(SHRT_SELECT_PREV_PART, Qt::ALT + Qt::Key_Left, "Edit: Select previous part", ARRANG_SHRT + PROLL_SHRT + DEDIT_SHRT, "sel_prv_prt");
+      defShrt(SHRT_SELECT_NEXT_PART, Qt::ALT + Qt::Key_Right, "Edit: Select next part", ARRANG_SHRT + PROLL_SHRT + DEDIT_SHRT, "sel_nxt_prt");
       defShrt(SHRT_SEL_LEFT,         Qt::Key_Left,  "Edit: Select nearest part/event to the left", ARRANG_SHRT + PROLL_SHRT + DEDIT_SHRT, "sel_left");
       defShrt(SHRT_SEL_LEFT_ADD,     Qt::Key_Left + Qt::SHIFT,  "Edit: Add nearest part/event to the left to selection", PROLL_SHRT + DEDIT_SHRT, "sel_left_add");
       defShrt(SHRT_SEL_RIGHT,        Qt::Key_Right,  "Edit: Select nearest part/event to the left", ARRANG_SHRT + PROLL_SHRT + DEDIT_SHRT,"sel_right");
@@ -210,6 +221,9 @@ void initShortCuts()
       defShrt(SHRT_TOGGLE_PUNCT2,      Qt::Key_Comma, "Quantize: Toggle punctuation quantization (2)", PROLL_SHRT, "midi_quant_punct2");
       defShrt(SHRT_INSERT_AT_LOCATION, Qt::SHIFT + Qt::Key_Right, "Edit: Insert at location", PROLL_SHRT, "midi_insert_at_loc");
 
+      defShrt(SHRT_INCREASE_LEN, Qt::CTRL + Qt::SHIFT + Qt::Key_Right, "Edit: Increase length", PROLL_SHRT, "increase_len");
+      defShrt(SHRT_DECREASE_LEN, Qt::CTRL + Qt::SHIFT + Qt::Key_Left, "Edit: Decrease length", PROLL_SHRT, "decrease_len");
+
       //-----------------------------------------------------------
       // List edit:
       //-----------------------------------------------------------
