@@ -62,6 +62,7 @@ enum {
       SHRT_TOGGLE_LOOP, // Slash
       SHRT_TOGGLE_METRO, // C
       SHRT_START_REC, // *(keypad)
+      SHRT_REC_CLEAR, // *(keypad)
 
       //Main + Drumeditor
       SHRT_NEW, //Ctrl+N
@@ -108,6 +109,7 @@ enum {
       SHRT_OPEN_TRANSPORT, //F11
       SHRT_OPEN_BIGTIME, //F12
       SHRT_OPEN_MIXER, //Ctrl+*
+      SHRT_OPEN_MIXER2, //Ctrl+*
       SHRT_OPEN_MARKER, // F9
       SHRT_OPEN_CLIPS, //Default: undefined
 
@@ -162,12 +164,22 @@ enum {
       SHRT_SEL_BELOW, //Down
       SHRT_SEL_BELOW_ADD, //move down and add to selection
 
+      /*
+      SHRT_INSERT, //Ctrl+Shift+I   - insert parts instead of pasting
+      SHRT_INSERTMEAS, //Ctrl+Shift+M - insert measures
+      */
+      //Arranger tracks
+      SHRT_SEL_TRACK_BELOW,
+      SHRT_SEL_TRACK_ABOVE,
+
       //To be in arranger, pianoroll & drumeditor
       SHRT_SELECT_ALL, //Ctrl+A
       SHRT_SELECT_NONE, //Ctrl+Shift+A
       SHRT_SELECT_INVERT, //Ctrl+I
       SHRT_SELECT_ILOOP, //Default: Undefined
       SHRT_SELECT_OLOOP, //Default: Undefined
+      SHRT_SELECT_PREV_PART, // Ctrl+-
+      SHRT_SELECT_NEXT_PART, // Ctrl++
       SHRT_SEL_LEFT, //left
       SHRT_SEL_LEFT_ADD, //move left and add to selection
       SHRT_SEL_RIGHT, //Right
@@ -178,6 +190,8 @@ enum {
       SHRT_DEC_POS,
       SHRT_LOCATORS_TO_SELECTION, //Alt+P, currently in arranger & pianoroll
       SHRT_INSERT_AT_LOCATION, //Shift+CrsrRight
+      SHRT_INCREASE_LEN,
+      SHRT_DECREASE_LEN,
 
       SHRT_TOOL_1,//Shift+1 Pointer
       SHRT_TOOL_2,//Shift+2 Pen
