@@ -36,6 +36,7 @@ class MarkerList;
 class Marker;
 class SNode;
 class QPopupMenu;
+class QButton;
 
 class MidiPort;
 class MidiDevice;
@@ -301,6 +302,7 @@ class Song : public QObject {
       int execMidiAutomationCtlPopup(MidiTrack*, MidiPart*, const QPoint&, int);
       void connectJackRoutes(AudioTrack* track, bool disconnect);
       void updateSoloStates();
+      //void chooseMidiRoutes(QButton* /*parent*/, MidiTrack* /*track*/, bool /*dst*/);
 
       //-----------------------------------------
       //   undo, redo
@@ -322,6 +324,7 @@ class Song : public QObject {
       //void undoOp(UndoOp::UndoType, Part* oPart, Part* nPart);
       void undoOp(UndoOp::UndoType, Part* oPart, Part* nPart, bool doCtrls, bool doClones);
       void undoOp(UndoOp::UndoType type, const char* changedFile, const char* changeData, int startframe, int endframe);
+      void undoOp(UndoOp::UndoType type, Marker* copyMarker, Marker* realMarker);
       bool doUndo1();
       void doUndo2();
       void doUndo3();
@@ -369,6 +372,7 @@ class Song : public QObject {
       bool getLoop() { return loopFlag; }
       void setLoop(bool f);
       void setRecord(bool f, bool autoRecEnable = true);
+      void clearTrackRec();
       void setPlay(bool f);
       void setStop(bool);
       void forward();
