@@ -29,6 +29,7 @@
 #include "midictrl.h"
 #include "amixer.h"
 #include "conf.h"
+#include "driver/jackmidi.h"
 
 //struct ClonePart {
       //const EventList* el;
@@ -890,7 +891,8 @@ void Song::writeFont(int level, Xml& xml, const char* name,
    const QFont& font) const
       {
       xml.nput(level, "<%s family=\"%s\" size=\"%d\"",
-         name, font.family().latin1(), font.pointSize());
+         //name, font.family().latin1(), font.pointSize());
+         name, Xml::xmlString(font.family()).latin1(), font.pointSize());
       if (font.weight() != QFont::Normal)
             xml.nput(" weight=\"%d\"", font.weight());
       if (font.italic())
@@ -990,19 +992,37 @@ void MusE::readToplevels(Xml& xml)
                                     pl->add(part);
                               }
                         else if (tag == "pianoroll") {
-                              startPianoroll(pl);
-                              toplevels.back().cobject()->readStatus(xml);
-                              pl = new PartList;
+                              // p3.3.34
+                              // Do not open if there are no parts.
+                              // Had bogus '-1' part index for list edit in med file,
+                              //  causing list edit to segfault on song load.
+                              // Somehow that -1 was put there on write, because the
+                              //  current part didn't exist anymore, so no index number
+                              //  could be found for it on write. Watching... may be fixed.
+                              // But for now be safe for all the top levels...
+                              if(!pl->empty())
+                              {
+                                
+                                startPianoroll(pl);
+                                toplevels.back().cobject()->readStatus(xml);
+                                pl = new PartList;
+                              }  
                               }
                         else if (tag == "drumedit") {
-                              startDrumEditor(pl);
-                              toplevels.back().cobject()->readStatus(xml);
-                              pl = new PartList;
+                              if(!pl->empty())
+                              {
+                                startDrumEditor(pl);
+                                toplevels.back().cobject()->readStatus(xml);
+                                pl = new PartList;
+                              }  
                               }
                         else if (tag == "listeditor") {
-                              startListEditor(pl);
-                              toplevels.back().cobject()->readStatus(xml);
-                              pl = new PartList;
+                              if(!pl->empty())
+                              {
+                                startListEditor(pl);
+                                toplevels.back().cobject()->readStatus(xml);
+                                pl = new PartList;
+                              }  
                               }
                         else if (tag == "master") {
                               startMasterEditor();
@@ -1017,9 +1037,12 @@ void MusE::readToplevels(Xml& xml)
                               toplevels.back().cobject()->readStatus(xml);
                               }
                         else if (tag == "waveedit") {
-                              startWaveEditor(pl);
-                              toplevels.back().cobject()->readStatus(xml);
-                              pl = new PartList;
+                              if(!pl->empty())
+                              {
+                                startWaveEditor(pl);
+                                toplevels.back().cobject()->readStatus(xml);
+                                pl = new PartList;
+                              }  
                               }
                         else if (tag == "cliplist") {
                               startClipList();
@@ -1235,31 +1258,40 @@ void Song::read(Xml& xml)
                               WaveTrack* track = new WaveTrack();
                               track->read(xml);
                               insertTrack0(track,-1);
+                              // Now that the track has been added to the lists in insertTrack2(), 
+                              //  OSC can find the track and its plugins, and start their native guis if required...
+                              track->showPendingPluginNativeGuis();
                               }
                         else if (tag == "AudioInput") {
                               AudioInput* track = new AudioInput();
                               track->read(xml);
                               insertTrack0(track,-1);
+                              track->showPendingPluginNativeGuis();
                               }
                         else if (tag == "AudioOutput") {
                               AudioOutput* track = new AudioOutput();
                               track->read(xml);
                               insertTrack0(track,-1);
+                              track->showPendingPluginNativeGuis();
                               }
                         else if (tag == "AudioGroup") {
                               AudioGroup* track = new AudioGroup();
                               track->read(xml);
                               insertTrack0(track,-1);
+                              track->showPendingPluginNativeGuis();
                               }
                         else if (tag == "AudioAux") {
                               AudioAux* track = new AudioAux();
                               track->read(xml);
                               insertTrack0(track,-1);
+                              track->showPendingPluginNativeGuis();
                               }
                         else if (tag == "SynthI") {
                               SynthI* track = new SynthI();
                               track->read(xml);
+                              // Done in SynthI::read()
                               // insertTrack(track,-1);
+                              //track->showPendingPluginNativeGuis();
                               }
                         else if (tag == "Route") {
                               readRoute(xml);
@@ -1407,12 +1439,30 @@ void Song::write(int level, Xml& xml) const
 
       // write routing
       for (ciTrack i = _tracks.begin(); i != _tracks.end(); ++i) {
-            if ((*i)->isMidiTrack())
-                  continue;
-            WaveTrack* track = (WaveTrack*)(*i);
-            track->writeRouting(level, xml);
+            
+            // p3.3.38 Changed
+            //if ((*i)->isMidiTrack())
+            //      continue;
+            //WaveTrack* track = (WaveTrack*)(*i);
+            //track->writeRouting(level, xml);
+            
+            (*i)->writeRouting(level, xml);
             }
 
+      // Write midi device routing.
+      for (iMidiDevice i = midiDevices.begin(); i != midiDevices.end(); ++i) {
+            //MidiJackDevice* mjd = dynamic_cast<MidiJackDevice*>(*i);
+            //if (!mjd)
+            //  continue;
+            //mjd->writeRouting(level, xml);
+            (*i)->writeRouting(level, xml);
+            }
+      
+      // p3.3.49 Write midi port routing.
+      for (int i = 0; i < MIDI_PORTS; ++i) {
+            midiPorts[i].writeRouting(level, xml);
+            }
+      
       tempomap.write(level, xml);
       sigmap.write(level, xml);
       _markerList->write(level, xml);
