@@ -33,6 +33,7 @@
 #include "audio.h"
 #include "midiseq.h"
 #include "midictrl.h"
+//#include "stringparam.h"
 
 std::vector<Synth*> synthis;  // array of available synthis
 
@@ -126,12 +127,12 @@ static Synth* findSynth(const QString& sclass, const QString& label)
       }
 
 //---------------------------------------------------------
-//   createSynthI
+//   createSynthInstance
 //    create a synthesizer instance of class "label"
 //---------------------------------------------------------
 
 //static SynthI* createSynthI(const QString& sclass)
-static SynthI* createSynthI(const QString& sclass, const QString& label)
+static SynthI* createSynthInstance(const QString& sclass, const QString& label)
       {
       //Synth* s = findSynth(sclass);
       Synth* s = findSynth(sclass, label);
@@ -149,7 +150,7 @@ static SynthI* createSynthI(const QString& sclass, const QString& label)
                   }
             }
       else
-            printf("synthi class:%s label:%s not found\n", sclass.latin1(), label.latin1());
+            printf("createSynthInstance: synthi class:%s label:%s not found\n", sclass.latin1(), label.latin1());
       return si;
       }
 
@@ -230,6 +231,11 @@ SynthI::SynthI()
       _openFlags  = 1;
       _readEnable = false;
       _writeEnable = false;
+      
+      _curBankH = 0;
+      _curBankL = 0;
+      _curProgram  = 0;
+
       setVolume(1.0);
       setPan(0.0);
       }
@@ -282,6 +288,20 @@ void SynthI::setName(const QString& s)
       MidiDevice::setName(s);
       }
 
+//---------------------------------------------------------
+//   currentProg
+//---------------------------------------------------------
+
+void SynthI::currentProg(unsigned long *prog, unsigned long *bankL, unsigned long *bankH)
+{
+  if(prog)
+    *prog  = _curProgram;
+  if(bankL)
+    *bankL = _curBankL;
+  if(bankH)
+    *bankH = _curBankH;
+}
+
 //---------------------------------------------------------
 //   init
 //---------------------------------------------------------
@@ -300,6 +320,16 @@ int MessSynthIF::channels() const
       return _mess->channels();
       }
 
+int MessSynthIF::totalOutChannels() const
+      {
+      return _mess->channels();
+      }
+
+int MessSynthIF::totalInChannels() const
+      {
+      return 0;
+      }
+
 //SynthIF* MessSynth::createSIF() const
 SynthIF* MessSynth::createSIF(SynthI* si)
       {
@@ -327,8 +357,11 @@ bool SynthI::initInstance(Synth* s, const QString& instanceName)
       setIName(instanceName);   // set instrument name
       _sif        = s->createSIF(this);
       
-      AudioTrack::setChannels(_sif->channels());
-
+      // p3.3.38
+      //AudioTrack::setChannels(_sif->channels());
+      AudioTrack::setTotalOutChannels(_sif->totalOutChannels());
+      AudioTrack::setTotalInChannels(_sif->totalInChannels());
+      
       //---------------------------------------------------
       //  read available controller from synti
       //---------------------------------------------------
@@ -379,9 +412,14 @@ bool SynthI::initInstance(Synth* s, const QString& instanceName)
             iel->clear();
             }
 
-      int idx = 0;
+      unsigned long idx = 0;
       for (std::vector<float>::iterator i = initParams.begin(); i != initParams.end(); ++i, ++idx)
             _sif->setParameter(idx, *i);
+      
+      // p3.3.40 Since we are done with the (sometimes huge) initial parameters list, clear it. 
+      // TODO: Decide: Maybe keep them around for a 'reset to previously loaded values' (revert) command? ...
+      initParams.clear();      
+      
       return false;
       }
 
@@ -531,36 +569,44 @@ void initMidiSynth()
 //SynthI* Song::createSynthI(const QString& sclass)
 SynthI* Song::createSynthI(const QString& sclass, const QString& label)
       {
-      // Added by Tim. p3.3.13
       //printf("Song::createSynthI calling ::createSynthI class:%s\n", sclass.latin1());
       
       //SynthI* si = ::createSynthI(sclass);
-      SynthI* si = ::createSynthI(sclass, label);
+      //SynthI* si = ::createSynthI(sclass, label);
+      SynthI* si = createSynthInstance(sclass, label);
       if(!si)
         return 0;
-      // Added by Tim. p3.3.13
       //printf("Song::createSynthI created SynthI. Before insertTrack1...\n");
       
       insertTrack1(si, -1);
-      // Added by Tim. p3.3.13
       //printf("Song::createSynthI after insertTrack1. Before msgInsertTrack...\n");
       
       msgInsertTrack(si, -1, true);       // add to instance list
-      // Added by Tim. p3.3.13
       //printf("Song::createSynthI after msgInsertTrack. Before insertTrack3...\n");
       
       insertTrack3(si, -1);
 
-      // Added by Tim. p3.3.13
       //printf("Song::createSynthI after insertTrack3. Adding default routes...\n");
       
       OutputList* ol = song->outputs();
       // add default route to master (first audio output)
       if (!ol->empty()) {
             AudioOutput* ao = ol->front();
-            audio->msgAddRoute(Route(si, -1), Route(ao, -1));
+            // p3.3.38
+            //audio->msgAddRoute(Route(si, -1), Route(ao, -1));
+            //audio->msgAddRoute(Route((AudioTrack*)si, -1), Route(ao, -1));
+            // Make sure the route channel and channels are valid.
+            audio->msgAddRoute(Route((AudioTrack*)si, 0, ((AudioTrack*)si)->channels()), Route(ao, 0, ((AudioTrack*)si)->channels()));
+            
             audio->msgUpdateSoloStates();
             }
+      
+      // Now that the track has been added to the lists in insertTrack2(),
+      //  if it's a dssi synth, OSC can find the synth, and initialize (and show) its native gui.
+      // No, initializing OSC without actually showing the gui doesn't work, at least for 
+      //  dssi-vst plugins - without showing the gui they exit after ten seconds.
+      //si->initGui();
+      
       return si;
       }
 
@@ -596,6 +642,10 @@ void SynthI::write(int level, Xml& xml) const
                   xml.qrectTag(level, "geometry", QRect(x, y, w, h));
             }
 
+      _stringParamMap.write(level, xml, "stringParam");
+      
+      xml.tag(level, "curProgram bankH=\"%ld\" bankL=\"%ld\" prog=\"%ld\"/", _curBankH, _curBankL, _curProgram);
+      
       _sif->write(level, xml);
       xml.etag(level, "SynthI");
       }
@@ -627,6 +677,45 @@ void MessSynthIF::write(int level, Xml& xml) const
             }
       }
 
+//---------------------------------------------------------
+//   SynthI::readProgram
+//---------------------------------------------------------
+
+void SynthI::readProgram(Xml& xml, const QString& name)
+{
+  for (;;) 
+  {
+    Xml::Token token = xml.parse();
+    const QString tag = xml.s1();
+    switch (token) 
+    {
+          case Xml::Error:
+          case Xml::End:
+                return;
+          case Xml::TagStart:
+                xml.unknown(name);
+                break;
+          case Xml::Attribut:
+                if(tag == "bankH") 
+                  _curBankH = xml.s2().toUInt();
+                else
+                if(tag == "bankL") 
+                  _curBankL = xml.s2().toUInt();
+                else
+                if(tag == "prog") 
+                  _curProgram = xml.s2().toUInt();
+                else
+                  xml.unknown(name);
+                break;
+          case Xml::TagEnd:
+                if(tag == name) 
+                  return;
+          default:
+                break;
+    }
+  }
+}
+
 //---------------------------------------------------------
 //   SynthI::read
 //---------------------------------------------------------
@@ -637,7 +726,7 @@ void SynthI::read(Xml& xml)
       QString label;
       
       int port = -1;
-      bool startGui = false;
+      bool startgui = false;
       QRect r;
 
       for (;;) {
@@ -655,13 +744,17 @@ void SynthI::read(Xml& xml)
                         else if (tag == "port")
                               port  = xml.parseInt();
                         else if (tag == "guiVisible")
-                              startGui = xml.parseInt();
+                              startgui = xml.parseInt();
                         else if (tag == "midistate")
                               readMidiState(xml);
                         else if (tag == "param") {
                               float val = xml.parseFloat();
                               initParams.push_back(val);
                               }
+                        else if (tag == "stringParam") 
+                              _stringParamMap.read(xml, tag);
+                        else if (tag == "curProgram") 
+                              readProgram(xml, tag);
                         else if (tag == "geometry")
                               r = readGeometry(xml, tag);
                         else if (AudioTrack::readProperties(xml, tag))
@@ -678,11 +771,21 @@ void SynthI::read(Xml& xml)
                               song->insertTrack0(this, -1);
                               if (port != -1 && port < MIDI_PORTS)
                                     midiPorts[port].setMidiDevice(this);
-                              showGui(startGui);
+                              
+                              // Now that the track has been added to the lists in insertTrack2(),
+                              //  if it's a dssi synth, OSC can find the synth, and initialize (and show) its native gui.
+                              // No, initializing OSC without actually showing the gui doesn't work, at least for 
+                              //  dssi-vst plugins - without showing the gui they exit after ten seconds.
+                              //initGui();
+                              showGui(startgui);
                               setGeometry(r.x(), r.y(), r.width(), r.height());
                               
                               mapRackPluginsToControllers();
                               
+                              // Now that the track has been added to the lists in insertTrack2(), if it's a dssi synth 
+                              //  OSC can find the track and its plugins, and start their native guis if required...
+                              showPendingPluginNativeGuis();
+                              
                               return;
                               }
                   default:
