@@ -1,13 +1,14 @@
 //=========================================================
 //  MusE
 //  Linux Music Editor
-//  $Id: thread.cpp,v 1.4.2.3 2005/11/07 18:45:19 wschweer Exp $
+//  $Id: thread.cpp,v 1.4.2.5 2009/12/20 05:00:35 terminator356 Exp $
 //
 //  (C) Copyright 2001-2004 Werner Schweer (ws@seh.de)
 //=========================================================
 
 #include "thread.h"
 #include <stdio.h>
+#include <stdlib.h>
 #include <unistd.h>
 #include <sys/mman.h>
 #include <sys/poll.h>
@@ -39,13 +40,19 @@ static void* loop(void* mops)
 //   start
 //---------------------------------------------------------
 
-void Thread::start(void* ptr)
+//void Thread::start(void* ptr)
+void Thread::start(int prio, void* ptr)
       {
+      // Changed by Tim. p3.3.17
+      
       userPtr = ptr;
       pthread_attr_t* attributes = 0;
+      _realTimePriority = prio;
+    
+      /*
       attributes = (pthread_attr_t*) malloc(sizeof(pthread_attr_t));
-
       pthread_attr_init(attributes);
+      */
 
 //      pthread_mutexattr_t mutexattr;
 //      pthread_mutexattr_init(&mutexattr);
@@ -55,6 +62,32 @@ void Thread::start(void* ptr)
 
 //      pthread_mutex_lock(&lock);
 
+
+      if (_realTimePriority) {
+            attributes = (pthread_attr_t*) malloc(sizeof(pthread_attr_t));
+            pthread_attr_init(attributes);
+
+            if (pthread_attr_setschedpolicy(attributes, SCHED_FIFO)) {
+                  printf("cannot set FIFO scheduling class for RT thread\n");
+                  }
+            if (pthread_attr_setscope (attributes, PTHREAD_SCOPE_SYSTEM)) {
+                  printf("Cannot set scheduling scope for RT thread\n");
+                  }
+            if (pthread_attr_setinheritsched(attributes, PTHREAD_EXPLICIT_SCHED)) {
+                  printf("Cannot set setinheritsched for RT thread\n");
+                  }
+
+            struct sched_param rt_param;
+            memset(&rt_param, 0, sizeof(rt_param));
+            rt_param.sched_priority = _realTimePriority;
+            if (pthread_attr_setschedparam (attributes, &rt_param)) {
+                  printf("Cannot set scheduling priority %d for RT thread (%s)\n",
+                     _realTimePriority, strerror(errno));
+                  }
+            }
+
+
+      /*
       if (pthread_create(&thread, attributes, ::loop, this))
             perror("creating thread failed:");
 //      else
@@ -62,8 +95,18 @@ void Thread::start(void* ptr)
 //           pthread_cond_wait(&ready, &lock);
 //      }
 //      pthread_mutex_unlock(&lock);
+      */
 
-      undoSetuid();
+
+      int rv;
+      if ((rv = pthread_create(&thread, attributes, ::loop, this))) 
+      {
+            fprintf(stderr, "creating thread <%s> failed: %s\n",
+               _name, strerror(rv));
+            thread = 0;
+      }
+
+      //undoSetuid();
       }
 
 //---------------------------------------------------------
@@ -72,7 +115,12 @@ void Thread::start(void* ptr)
 
 void Thread::stop(bool force)
       {
-      if (force && thread > 0) {
+      // Changed by Tim. p3.3.17
+      
+      if (thread == 0)
+            return;
+      //if (force && thread > 0) {
+      if (force) {
             pthread_cancel(thread);
             threadStop();
             }
@@ -88,11 +136,16 @@ void Thread::stop(bool force)
 //    prio = 0    no realtime scheduling
 //---------------------------------------------------------
 
-Thread::Thread(int prio, const char* s)
+//Thread::Thread(int prio, const char* s)
+Thread::Thread(const char* s)
       {
+      // Changed by Tim. p3.3.17
+      
       userPtr          = 0;
       _name            = s;
-      realTimePriority = prio;
+      //realTimePriority = prio;
+      _realTimePriority = 0;
+      
       pfd              = 0;
       npfd             = 0;
       maxpfd           = 0;
@@ -100,8 +153,8 @@ Thread::Thread(int prio, const char* s)
       _pollWait        = -1;
       thread           = 0;
 
-      if (debugMsg)
-            printf("Start thread %s with priority %d\n", s, prio);
+      //if (debugMsg)
+      //      printf("Start thread %s with priority %d\n", s, prio);
 
       // create message channels
       int filedes[2];         // 0 - reading   1 - writing
@@ -181,7 +234,14 @@ void Thread::removePollFd(int fd, int action)
 
 void Thread::loop()
       {
+      // Changed by Tim. p3.3.17
 
+      if (!debugMode) {
+            if (mlockall(MCL_CURRENT | MCL_FUTURE))
+                  perror("WARNING: Cannot lock memory:");
+            }
+      
+/*      
       pthread_attr_t* attributes = 0;
       attributes = (pthread_attr_t*) malloc(sizeof(pthread_attr_t));
       pthread_attr_init(attributes);
@@ -210,35 +270,53 @@ void Thread::loop()
             if (error != 0)
               perror( "error set_schedparam 2:");
 
-    /*      if (!debugMode) {
-                if (mlockall(MCL_CURRENT|MCL_FUTURE))
-                      perror("WARNING: Cannot lock memory:");
-                }
-    */
+//          if (!debugMode) {
+//                if (mlockall(MCL_CURRENT|MCL_FUTURE))
+//                      perror("WARNING: Cannot lock memory:");
+//                }
+
             undoSetuid();
             }
 
+*/
 
 
-
+/*
 #define BIG_ENOUGH_STACK (1024*1024*1)
       char buf[BIG_ENOUGH_STACK];
       for (int i = 0; i < BIG_ENOUGH_STACK; i++)
             buf[i] = i;
 #undef BIG_ENOUGH_STACK
+*/
+
+#ifdef __APPLE__
+#define BIG_ENOUGH_STACK (1024*256*1)
+#else
+#define BIG_ENOUGH_STACK (1024*1024*1)
+#endif
+      char buf[BIG_ENOUGH_STACK];
+      for (int i = 0; i < BIG_ENOUGH_STACK; i++)
+            buf[i] = i;
+#undef BIG_ENOUGH_STACK
 
       pthread_setcancelstate(PTHREAD_CANCEL_ENABLE, 0);
       pthread_setcanceltype(PTHREAD_CANCEL_ASYNCHRONOUS, 0);
 
-      int policy;
+      int policy = 0;
       if ((policy = sched_getscheduler (0)) < 0) {
             printf("Thread: Cannot get current client scheduler: %s\n", strerror(errno));
             }
 
+      /*
       if (debugMsg)
             printf("Thread <%s> set to %s priority %d\n",
                _name, policy == SCHED_FIFO ? "SCHED_FIFO" : "SCHED_OTHER",
                 realTimePriority);
+      */          
+      if (debugMsg)
+            printf("Thread <%s, id %p> has %s priority %d\n",
+               _name, (void *)pthread_self(), policy == SCHED_FIFO ? "SCHED_FIFO" : "SCHED_OTHER",
+                _realTimePriority);
 
 
 //      pthread_mutex_lock(&lock);
@@ -251,6 +329,8 @@ void Thread::loop()
       while (_running) {
             if (debugMode)          // DEBUG
                   _pollWait = 10;   // ms
+            else
+                  _pollWait = -1;
 
             int n = poll(pfd, npfd, _pollWait);
             if (n < 0) {
@@ -283,9 +363,11 @@ void Thread::loop()
 //---------------------------------------------------------
 
 bool Thread::sendMsg(const ThreadMsg* m)
-      {
+{
+      // Changed by Tim. p3.3.17
 
-      if (_running) {
+      if (_running) 
+      {
             int rv = write(toThreadFdw, &m, sizeof(ThreadMsg*));
             if (rv != sizeof(ThreadMsg*)) {
                   perror("Thread::sendMessage(): write pipe failed");
@@ -295,18 +377,26 @@ bool Thread::sendMsg(const ThreadMsg* m)
            // wait for sequencer to finish operation
             char c;
             rv = read(fromThreadFdr, &c, 1);
-            if (rv != 1) {
+            if (rv != 1) 
+            {
                   perror("Thread::sendMessage(): read pipe failed");
                   return true;
-                  }
             }
-      else {
+            //int c;
+            //rv = read(fromThreadFdr, &c, sizeof(c));
+            //if (rv != sizeof(c)) {
+            //      perror("Thread::sendMessage(): read pipe failed");
+            //      return true;
+            //      }
+      }
+      else 
+      {
             // if thread is not running (during initialization)
             // process commands directly:
             processMsg(m);
-            }
-      return false;
       }
+      return false;
+}
 
 //---------------------------------------------------------
 //   send
@@ -330,7 +420,6 @@ bool Thread::sendMsg1(const void* m, int n)
 
 void Thread::readMsg()
       {
-
       ThreadMsg* p;
       if (read(toThreadFdr, &p, sizeof(p)) != sizeof(p)) {
             perror("Thread::readMessage(): read pipe failed");
@@ -341,6 +430,10 @@ void Thread::readMsg()
       int rv = write(fromThreadFdw, &c, 1);
       if (rv != 1)
             perror("Thread::readMessage(): write pipe failed");
+      //int c = p->serialNo;
+      //int rv = write(fromThreadFdw, &c, sizeof(c));
+      //if (rv != sizeof(c))
+      //      perror("Thread::readMsg(): write pipe failed");
       }
 
 //---------------------------------------------------------
