@@ -6,6 +6,9 @@
 //  (C) Copyright 2000-2004 Werner Schweer (ws@seh.de)
 //=========================================================
 
+#include <qt.h>
+#include <qstring.h>
+
 #include "track.h"
 #include "event.h"
 #include "mididev.h"
@@ -14,6 +17,9 @@
 #include "xml.h"
 #include "plugin.h"
 #include "drummap.h"
+#include "audio.h"
+#include "globaldefs.h"
+#include "route.h"
 
 unsigned int Track::_soloRefCnt = 0;
 Track* Track::_tmpSoloChainTrack = 0;
@@ -367,8 +373,8 @@ MidiTrack::MidiTrack(const MidiTrack& mt, bool cloneParts)
       {
       _outPort       = mt.outPort();
       _outChannel    = mt.outChannel();
-      _inPortMask    = mt.inPortMask();
-      _inChannelMask = mt.inChannelMask();
+      ///_inPortMask    = mt.inPortMask();
+      ///_inChannelMask = mt.inChannelMask();
       _events        = new EventList;
       _mpevents      = new MPEventList;
       transposition  = mt.transposition;
@@ -395,9 +401,9 @@ void MidiTrack::init()
       _outChannel    = 0;
       // Changed by Tim. p3.3.8
       //_inPortMask    = 0xffff;
-      _inPortMask    = 0xffffffff;
+      ///_inPortMask    = 0xffffffff;
       
-      _inChannelMask = 0xffff;      // "ALL"
+      ///_inChannelMask = 0xffff;      // "ALL"
       transposition  = 0;
       velocity       = 0;
       delay          = 0;
@@ -438,6 +444,83 @@ void MidiTrack::setOutPortAndUpdate(int i)
   addPortCtrlEvents(this);
 }
 
+//---------------------------------------------------------
+//   setInPortAndChannelMask
+//   For old song files with port mask (max 32 ports) and channel mask (16 channels), 
+//    before midi routing was added (the iR button). p3.3.48
+//---------------------------------------------------------
+
+void MidiTrack::setInPortAndChannelMask(unsigned int portmask, int chanmask) 
+{ 
+  //if(!portmask || !chanmask)
+  //  return;
+     
+  //RouteList* rl = inRoutes();
+  bool changed = false;
+  
+  for(int port = 0; port < 32; ++port)  // 32 is the old maximum number of ports.
+  {
+    // p3.3.50 If the port was not used in the song file to begin with, just ignore it.
+    // This saves from having all of the first 32 ports' channels connected.
+    if(!midiPorts[port].foundInSongFile())
+      continue;
+      
+    //if(!(portmask & (1 << port)))
+    //  continue;
+    
+    // p3.3.50 Removed. Allow to connect to port with no device so user can change device later.
+    //MidiPort* mp = &midiPorts[port];
+    //MidiDevice* md = mp->device();
+    //if(!md)
+    //  continue;
+        
+    //for(int ch = 0; ch < MIDI_CHANNELS; ++ch)  // p3.3.50 Removed.
+    //{
+      //if(!(chanmask & (1 << ch)))
+      //  continue;
+    
+      //Route aRoute(md, ch);
+      //Route bRoute(this, ch);
+      Route aRoute(port, chanmask);     // p3.3.50
+      Route bRoute(this, chanmask);
+    
+      // p3.3.50 Removed.
+      //iRoute iir = rl->begin();
+      //for(; iir != rl->end(); ++iir) 
+      //{
+        //if(*iir == aRoute)
+      //  if(iir->type == Route::MIDI_PORT_ROUTE && iir->midiPort == port)      // p3.3.50
+      //    break;
+      //}
+      
+      // Route wanted?
+      //if((portmask & (1 << port)) && (chanmask & (1 << ch)))
+      if(portmask & (1 << port))                                          // p3.3.50
+      {
+        // Route already exists?
+        //if(iir != rl->end()) 
+        //  continue;
+        audio->msgAddRoute(aRoute, bRoute);
+        changed = true;
+      }
+      else
+      {
+        // Route does not exist?
+        //if(iir == rl->end()) 
+        //  continue;
+        audio->msgRemoveRoute(aRoute, bRoute);
+        changed = true;
+      }
+    //}
+  }
+   
+  if(changed)
+  {
+    audio->msgUpdateSoloStates();
+    song->update(SC_ROUTE);
+  }  
+}
+
 /*
 //---------------------------------------------------------
 //   addPortCtrlEvents
@@ -647,6 +730,143 @@ bool Track::readProperties(Xml& xml, const QString& tag)
       return false;
       }
 
+//---------------------------------------------------------
+//   writeRouting
+//---------------------------------------------------------
+
+void Track::writeRouting(int level, Xml& xml) const
+{
+      QString s;
+      
+      if (type() == Track::AUDIO_INPUT) 
+      {
+        const RouteList* rl = &_inRoutes;
+        for (ciRoute r = rl->begin(); r != rl->end(); ++r) 
+        {
+          if(!r->name().isEmpty())
+          {
+            s = QT_TR_NOOP("Route");
+            if(r->channel != -1)
+              s += QString(QT_TR_NOOP(" channel=\"%1\"")).arg(r->channel);
+            
+            ///Route dst(name(), true, r->channel);
+            //xml.tag(level++, "Route");
+            xml.tag(level++, s);
+            
+            // p3.3.38 New routing scheme.
+            ///xml.strTag(level, "srcNode", r->name());
+            //xml.tag(level, "source type=\"%d\" name=\"%s\"/", r->type, r->name().latin1());
+            s = QT_TR_NOOP("source");
+            if(r->type != Route::TRACK_ROUTE)
+              s += QString(QT_TR_NOOP(" type=\"%1\"")).arg(r->type);
+            //s += QString(QT_TR_NOOP(" name=\"%1\"/")).arg(r->name());
+            s += QString(QT_TR_NOOP(" name=\"%1\"/")).arg(Xml::xmlString(r->name()));
+            xml.tag(level, s);
+            
+            ///xml.strTag(level, "dstNode", dst.name());
+            
+            //if(r->channel != -1)
+            //  xml.tag(level, "dest type=\"%d\" channel=\"%d\" name=\"%s\"/", Route::TRACK_ROUTE, r->channel, name().latin1());
+            //else  
+            //  xml.tag(level, "dest type=\"%d\" name=\"%s\"/", Route::TRACK_ROUTE, name().latin1());
+
+            //xml.tag(level, "dest name=\"%s\"/", name().latin1());
+            xml.tag(level, "dest name=\"%s\"/", Xml::xmlString(name()).latin1());
+            
+            xml.etag(level--, "Route");
+          }
+        }
+      }
+      
+      const RouteList* rl = &_outRoutes;
+      for (ciRoute r = rl->begin(); r != rl->end(); ++r) 
+      {
+        //if(!r->name().isEmpty())
+        if(r->midiPort != -1 || !r->name().isEmpty()) // p3.3.49
+        {
+          ///QString src(name());
+          ///if (type() == Track::AUDIO_OUTPUT) 
+          ///{ 
+                ///Route s(src, false, r->channel);
+                ///src = s.name();
+          ///}
+          
+          s = QT_TR_NOOP("Route");
+          if(r->type == Route::MIDI_PORT_ROUTE)  // p3.3.50
+          {
+            if(r->channel != -1 && r->channel != 0)
+              s += QString(QT_TR_NOOP(" channelMask=\"%1\"")).arg(r->channel);  // Use new channel mask.
+          }
+          else
+          {
+            if(r->channel != -1)
+              s += QString(QT_TR_NOOP(" channel=\"%1\"")).arg(r->channel);
+          }    
+          if(r->channels != -1)
+            s += QString(QT_TR_NOOP(" channels=\"%1\"")).arg(r->channels);
+          if(r->remoteChannel != -1)
+            s += QString(QT_TR_NOOP(" remch=\"%1\"")).arg(r->remoteChannel);
+          
+          //xml.tag(level++, "Route");
+          xml.tag(level++, s);
+          
+          ///xml.strTag(level, "srcNode", src);
+          //if(r->channel != -1)
+          
+          // Allow for a regular mono or stereo track to feed a multi-channel synti. 
+          // thisChannel is the 'starting' channel of this source if feeding a regular track.
+          //if(r->type == Route::TRACK_ROUTE && r->track->isSynti() && r->channel != -1)
+          //if(isSynti() && r->thisChannel != -1)
+            //xml.tag(level, "source type=\"%d\" channel=\"%d\" name=\"%s\"/", Route::TRACK_ROUTE, r->channel, name().latin1());
+          //  xml.tag(level, "source type=\"%d\" channel=\"%d\" name=\"%s\"/", Route::TRACK_ROUTE, r->thisChannel, name().latin1());
+          //else
+          
+          //if(r->channel != -1)
+          //  xml.tag(level, "source type=\"%d\" channel=\"%d\" name=\"%s\"/", Route::TRACK_ROUTE, r->channel, name().latin1());
+          //else  
+          //  xml.tag(level, "source type=\"%d\" name=\"%s\"/", Route::TRACK_ROUTE, name().latin1());
+          //xml.tag(level, "source name=\"%s\"/", name().latin1());
+          xml.tag(level, "source name=\"%s\"/", Xml::xmlString(name()).latin1());
+          
+          ///xml.strTag(level, "dstNode", r->name());
+          //if(r->channel != -1)
+          //  xml.tag(level, "dest type=\"%d\" channel=\"%d\" name=\"%s\"/", r->type, r->channel, r->name().latin1());
+          //else  
+          //  xml.tag(level, "dest type=\"%d\" name=\"%s\"/", r->type, r->name().latin1());
+          
+          // Allow for a regular mono or stereo track to feed a multi-channel synti. 
+          // Channel is the 'starting' channel of the destination.
+          //if(r->type == Route::TRACK_ROUTE && r->track->isSynti() && r->channel != -1)
+          
+          //if(r->type == Route::TRACK_ROUTE && r->track->type() == Track::AUDIO_SOFTSYNTH && r->remoteChannel != -1)
+          //  xml.tag(level, "dest type=\"%d\" channel=\"%d\" name=\"%s\"/", r->type, r->remoteChannel, r->name().latin1());
+          //else  
+          //if(r->type == Route::MIDI_DEVICE_ROUTE)
+          //  xml.tag(level, "dest devtype=\"%d\" name=\"%s\"/", r->device->deviceType(), r->name().latin1());
+          //else  
+          //  xml.tag(level, "dest type=\"%d\" name=\"%s\"/", r->type, r->name().latin1());
+          
+          s = QT_TR_NOOP("dest");
+          
+          //if(r->type == Route::MIDI_DEVICE_ROUTE)                                      // p3.3.49 Obsolete since 1.1-RC2    
+          //  s += QString(QT_TR_NOOP(" devtype=\"%1\"")).arg(r->device->deviceType());  //
+          //if(r->type != Route::TRACK_ROUTE)                                            //
+          if(r->type != Route::TRACK_ROUTE && r->type != Route::MIDI_PORT_ROUTE)
+            s += QString(QT_TR_NOOP(" type=\"%1\"")).arg(r->type);
+
+          //s += QString(QT_TR_NOOP(" name=\"%1\"/")).arg(r->name());
+          if(r->type == Route::MIDI_PORT_ROUTE)                                          // p3.3.49 
+            s += QString(QT_TR_NOOP(" mport=\"%1\"/")).arg(r->midiPort);
+          else  
+            s += QString(QT_TR_NOOP(" name=\"%1\"/")).arg(Xml::xmlString(r->name()));
+            
+          xml.tag(level, s);
+          
+          xml.etag(level--, "Route");
+        }
+      }
+}
+       
 //---------------------------------------------------------
 //   MidiTrack::write
 //---------------------------------------------------------
@@ -665,8 +885,8 @@ void MidiTrack::write(int level, Xml& xml) const
       xml.intTag(level, "device", outPort());
       xml.intTag(level, "channel", outChannel());
       //xml.intTag(level, "inportMap", inPortMask());
-      xml.uintTag(level, "inportMap", inPortMask());
-      xml.intTag(level, "inchannelMap", inChannelMask());
+      ///xml.uintTag(level, "inportMap", inPortMask());       // Obsolete
+      ///xml.intTag(level, "inchannelMap", inChannelMask());  // Obsolete
       xml.intTag(level, "locked", _locked);
       xml.intTag(level, "echo", _recEcho);
 
@@ -689,6 +909,9 @@ void MidiTrack::write(int level, Xml& xml) const
 
 void MidiTrack::read(Xml& xml)
       {
+      unsigned int portmask = 0;
+      int chanmask = 0;
+      
       for (;;) {
             Xml::Token token = xml.parse();
             const QString& tag = xml.s1();
@@ -721,9 +944,13 @@ void MidiTrack::read(Xml& xml)
                               setOutChannel(xml.parseInt());
                         else if (tag == "inportMap")
                               //setInPortMask(xml.parseInt());
-                              setInPortMask(xml.parseUInt());
+                              ///setInPortMask(xml.parseUInt());
+                              //xml.skip(tag);                      // Obsolete. 
+                              portmask = xml.parseUInt();           // p3.3.48: Support old files.
                         else if (tag == "inchannelMap")
-                              setInChannelMask(xml.parseInt());
+                              ///setInChannelMask(xml.parseInt());
+                              //xml.skip(tag);                      // Obsolete.
+                              chanmask = xml.parseInt();            // p3.3.48: Support old files.
                         else if (tag == "locked")
                               _locked = xml.parseInt();
                         else if (tag == "echo")
@@ -740,9 +967,11 @@ void MidiTrack::read(Xml& xml)
                   case Xml::Attribut:
                         break;
                   case Xml::TagEnd:
-                        if (tag == "miditrack" || tag == "drumtrack") {
-                              return;
-                              }
+                        if (tag == "miditrack" || tag == "drumtrack") 
+                        {
+                          setInPortAndChannelMask(portmask, chanmask); // p3.3.48: Support old files.
+                          return;
+                        }
                   default:
                         break;
                   }
