@@ -473,6 +473,18 @@ int VstSynthIF::channels() const
       return plugin->numOutputs;
       }
 
+int VstSynthIF::totalOutChannels() const
+      {
+      AEffect* plugin = _fst->plugin;
+      return plugin->numOutputs;
+      }
+
+int VstSynthIF::totalInChannels() const
+      {
+      AEffect* plugin = _fst->plugin;
+      return plugin->numInputs;
+      }
+
 //---------------------------------------------------------
 //   createSIF
 //---------------------------------------------------------
@@ -501,11 +513,20 @@ void VstSynthIF::deactivate3()
             }
       }
 
+//---------------------------------------------------------
+//   getParameter
+//---------------------------------------------------------
+
+float VstSynthIF::getParameter(unsigned long idx) const
+      {
+      return _fst->plugin->getParameter(_fst->plugin, idx);
+      }
+
 //---------------------------------------------------------
 //   setParameter
 //---------------------------------------------------------
 
-void VstSynthIF::setParameter(int idx, float value)
+void VstSynthIF::setParameter(unsigned long idx, float value)
       {
       _fst->plugin->setParameter(_fst->plugin, idx, value);
       }
