@@ -50,23 +50,29 @@ class VstSynthIF : public SynthIF
             _guiVisible = false;
             }
             
+      virtual bool initGui()       { return true; };
+      virtual void guiHeartBeat()  {  }
       virtual bool guiVisible() const;
       virtual void showGui(bool v);
       virtual bool hasGui() const;
       virtual void getGeometry(int*, int*, int*, int*) const {}
       virtual void setGeometry(int, int, int, int) {}
+      virtual void preProcessAlways() { };
       virtual iMPEvent getData(MidiPort*, MPEventList*, iMPEvent, unsigned pos, int ports, unsigned n, float** buffer) ;
       virtual bool putEvent(const MidiPlayEvent& ev);
       virtual MidiPlayEvent receiveEvent();
       virtual int eventsPending() const { return 0; }
       virtual bool init(Synth*);
       virtual int channels() const;
+      virtual int totalOutChannels() const;
+      virtual int totalInChannels() const;
       virtual void deactivate3();
       virtual const char* getPatchName(int, int, int, bool) const { return ""; }
       virtual const char* getPatchName(int, int, MType, bool) { return ""; }
       virtual void populatePatchPopup(QPopupMenu*, int, MType, bool) {};
       virtual void write(int level, Xml& xml) const;
-      virtual void setParameter(int idx, float value);
+      virtual float getParameter(unsigned long idx) const;
+      virtual void setParameter(unsigned long idx, float value);
       virtual int getControllerInfo(int, const char**, int*, int*, int*, int*) { return 0; }
       };
 
