@@ -29,6 +29,7 @@
 //#define WAVE_DEBUG
 //#define WAVE_DEBUG_PRC
 
+/*
 const char* audioFilePattern[] = {
       "Wave/Binary (*.wav *.ogg *.bin)",
       "Wave (*.wav *.ogg)",
@@ -36,6 +37,7 @@ const char* audioFilePattern[] = {
       "All Files (*)",
       0
       };
+*/
 const int cacheMag = 128;
 
 // ClipList* waveClips;
@@ -440,17 +442,32 @@ void SndFile::setFormat(int fmt, int ch, int rate)
       sfinfo.frames     = 0;
       }
 
+//---------------------------------------------------------
+//   readWithHeap
+//   not as realtime friendly but can retrieve bigger data
+//---------------------------------------------------------
+size_t SndFile::readWithHeap(int srcChannels, float** dst, size_t n, bool overwrite)
+      {
+      float *buffer = new float[n * sfinfo.channels];
+      int rn = readInternal(srcChannels,dst,n,overwrite, buffer);
+      delete buffer;
+      return rn;
+      }
+
 //---------------------------------------------------------
 //   read
 //---------------------------------------------------------
-
 size_t SndFile::read(int srcChannels, float** dst, size_t n, bool overwrite)
       {
-      // Changed by Tim. p3.3.17
-      //float *buffer = new float[n * sfinfo.channels];
       float buffer[n * sfinfo.channels];
+      int rn = readInternal(srcChannels,dst,n,overwrite, buffer);
+      return rn;
+      }
+
+size_t SndFile::readInternal(int srcChannels, float** dst, size_t n, bool overwrite, float *buffer)
+{
       size_t rn = sf_readf_float(sf, buffer, n);
-      
+
       float* src      = buffer;
       int dstChannels = sfinfo.channels;
       if (srcChannels == dstChannels) {
@@ -470,7 +487,7 @@ size_t SndFile::read(int srcChannels, float** dst, size_t n, bool overwrite)
             if(overwrite)
               for (size_t i = 0; i < rn; ++i)
                   *(dst[0] + i) = src[i + i] + src[i + i + 1];
-            else  
+            else
               for (size_t i = 0; i < rn; ++i)
                   *(dst[0] + i) += src[i + i] + src[i + i + 1];
             }
@@ -482,7 +499,7 @@ size_t SndFile::read(int srcChannels, float** dst, size_t n, bool overwrite)
                   *(dst[0]+i) = data;
                   *(dst[1]+i) = data;
                   }
-            else  
+            else
               for (size_t i = 0; i < rn; ++i) {
                   float data = *src++;
                   *(dst[0]+i) += data;
@@ -493,10 +510,11 @@ size_t SndFile::read(int srcChannels, float** dst, size_t n, bool overwrite)
             printf("SndFile:read channel mismatch %d -> %d\n",
                srcChannels, dstChannels);
             }
-      
-      //delete buffer;
+
       return rn;
-      }
+
+}
+
 
 //---------------------------------------------------------
 //   write
@@ -725,7 +743,7 @@ void SndFile::applyUndoFile(const QString& original, const QString& tmpfile, uns
             data2beoverwritten[i] = new float[tmpdatalen];
             }
       orig->seek(startframe, 0);
-      orig->read(file_channels, data2beoverwritten, tmpdatalen);
+      orig->readWithHeap(file_channels, data2beoverwritten, tmpdatalen);
 
       orig->close();
 
@@ -735,7 +753,7 @@ void SndFile::applyUndoFile(const QString& original, const QString& tmpfile, uns
             tmpfiledata[i] = new float[tmpdatalen];
             }
       tmp.seek(0, 0);
-      tmp.read(file_channels, tmpfiledata, tmpdatalen);
+      tmp.readWithHeap(file_channels, tmpfiledata, tmpdatalen);
       tmp.close();
 
       // Write temporary data to original file:
@@ -786,8 +804,9 @@ void MusE::importWave()
               "a wave track"));
             return;
             }
-      QString fn = getOpenFileName(lastWavePath, audioFilePattern, this,
-         tr("Import Wave File"), 0);
+      //QString fn = getOpenFileName(lastWavePath, audioFilePattern, this,
+      QString fn = getOpenFileName(lastWavePath, audio_file_pattern, this,
+         tr("Import Wave File"), 0);                                    
       if (!fn.isEmpty()) {
             lastWavePath = fn;
             importWaveToTrack(fn);
