@@ -551,6 +551,10 @@ void WaveView::cmd(int n)
       if (modifyoperation != -1) {
             if (selectionStart == selectionStop) {
                   printf("No selection. Ignoring\n"); //@!TODO: Disable menu options when no selection
+                  QMessageBox::information(this,
+                     QString("MusE"),
+                     QWidget::tr("No selection. Ignoring"));
+
                   return;
                   }
             
@@ -660,7 +664,7 @@ void WaveView::modifySelection(int operation, unsigned startpos, unsigned stoppo
                      tmpdata[i] = new float[tmpdatalen];
                      }
                file.seek(tmpdataoffset, 0);
-               file.read(file_channels, tmpdata, tmpdatalen);
+               file.readWithHeap(file_channels, tmpdata, tmpdatalen);
                file.close();
                tmpFile.write(file_channels, tmpdata, tmpdatalen);
                tmpFile.close();
@@ -865,7 +869,7 @@ void WaveView::editExternal(unsigned file_format, unsigned file_samplerate, unsi
             else {
                 // Re-read file again
                 exttmpFile.seek(0, 0);
-                size_t sz = exttmpFile.read(file_channels, tmpdata, tmpdatalen);
+                size_t sz = exttmpFile.readWithHeap(file_channels, tmpdata, tmpdatalen);
                 if (sz != tmpdatalen) {
                         // File must have been shrunken - not good. Alert user.
                         QMessageBox::critical(this, tr("MusE - file size changed"),
