@@ -35,69 +35,92 @@ void WaveTrack::fetchData(unsigned pos, unsigned samples, float** bp, bool doSee
       // reset buffer to zero
       for (int i = 0; i < channels(); ++i)
             memset(bp[i], 0, samples * sizeof(float));
-      PartList* pl = parts();
-
-      unsigned n = samples;
-      for (iPart ip = pl->begin(); ip != pl->end(); ++ip) {
-            WavePart* part = (WavePart*)(ip->second);
-            // Changed by Tim. p3.3.17
-            //if (part->mute() || isMute())
-            if (part->mute())
+      
+      // p3.3.29
+      // Process only if track is not off.
+      if(!off())
+      {  
+        
+        PartList* pl = parts();
+        unsigned n = samples;
+        for (iPart ip = pl->begin(); ip != pl->end(); ++ip) {
+              WavePart* part = (WavePart*)(ip->second);
+              // Changed by Tim. p3.3.17
+              //if (part->mute() || isMute())
+              if (part->mute())
+                  continue;
+              
+              unsigned p_spos = part->frame();
+              unsigned p_epos = p_spos + part->lenFrame();
+              if (pos + n < p_spos)
+                break;
+              if (pos >= p_epos)
                 continue;
-            
-            unsigned p_spos = part->frame();
-            unsigned p_epos = p_spos + part->lenFrame();
-            if (pos + n < p_spos)
-              break;
-            if (pos >= p_epos)
-              continue;
-
-            EventList* events = part->events();
-            for (iEvent ie = events->begin(); ie != events->end(); ++ie) {
-                  Event& event = ie->second;
-                  unsigned e_spos  = event.frame() + p_spos;
-                  unsigned nn      = event.lenFrame();
-                  unsigned e_epos  = e_spos + nn;
-                  
-                  if (pos + n < e_spos) 
-                    break;
-                  if (pos >= e_epos) 
-                    continue;
-
-                  int offset = e_spos - pos;
-
-                  unsigned srcOffset, dstOffset;
-                  if (offset > 0) {
-                        nn = n - offset;
-                        srcOffset = 0;
-                        dstOffset = offset;
-                        }
-                  else {
-                        srcOffset = -offset;
-                        dstOffset = 0;
-                        
-                        nn += offset;
-                        if (nn > n)
-                              nn = n;
-                        }
-                  float* bpp[channels()];
-                  for (int i = 0; i < channels(); ++i)
-                        bpp[i] = bp[i] + dstOffset;
-
-                  // By T356. Allow overlapping parts or events to mix together !
-                  // Since the buffers are cleared above, just read and add (don't overwrite) the samples.
-                  //event.read(srcOffset, bpp, channels(), nn);
-                  //event.read(srcOffset, bpp, channels(), nn, false);
-                  event.readAudio(srcOffset, bpp, channels(), nn, doSeek, false);
-                  
-                  }
-            }
+  
+              EventList* events = part->events();
+              for (iEvent ie = events->begin(); ie != events->end(); ++ie) {
+                    Event& event = ie->second;
+                    unsigned e_spos  = event.frame() + p_spos;
+                    unsigned nn      = event.lenFrame();
+                    unsigned e_epos  = e_spos + nn;
+                    
+                    if (pos + n < e_spos) 
+                      break;
+                    if (pos >= e_epos) 
+                      continue;
+  
+                    int offset = e_spos - pos;
+  
+                    unsigned srcOffset, dstOffset;
+                    if (offset > 0) {
+                          nn = n - offset;
+                          srcOffset = 0;
+                          dstOffset = offset;
+                          }
+                    else {
+                          srcOffset = -offset;
+                          dstOffset = 0;
+                          
+                          nn += offset;
+                          if (nn > n)
+                                nn = n;
+                          }
+                    float* bpp[channels()];
+                    for (int i = 0; i < channels(); ++i)
+                          bpp[i] = bp[i] + dstOffset;
+  
+                    // By T356. Allow overlapping parts or events to mix together !
+                    // Since the buffers are cleared above, just read and add (don't overwrite) the samples.
+                    //event.read(srcOffset, bpp, channels(), nn);
+                    //event.read(srcOffset, bpp, channels(), nn, false);
+                    //event.readAudio(srcOffset, bpp, channels(), nn, doSeek, false);
+                    // p3.3.33
+                    event.readAudio(part, srcOffset, bpp, channels(), nn, doSeek, false);
+                    
+                    }
+              }
+      }
+              
       if(config.useDenormalBias) {
             // add denormal bias to outdata
             for (int i = 0; i < channels(); ++i)
                   for (int j = 0; j < samples; ++j)
+                  {
+                      bp[i][j] +=denormalBias;
+                      
+                      /*
+                      // p3.3.41
+                      if(j & 1)
+                        bp[i][j] -=denormalBias;
+                      else  
                         bp[i][j] +=denormalBias;
+                      */  
+                  }      
             }
+      
+      // p3.3.41
+      //fprintf(stderr, "WaveTrack::fetchData data: samples:%ld %e %e %e %e\n", samples, bp[0][0], bp[0][1], bp[0][2], bp[0][3]);
+      
       _prefetchFifo.add();
       }
 
@@ -181,17 +204,45 @@ Part* WaveTrack::newPart(Part*p, bool clone)
 
 bool WaveTrack::getData(unsigned framePos, int channels, unsigned nframe, float** bp)
       {
-      // Added by Tim. p3.3.16
       //if(debugMsg)
       //  printf("WaveTrack::getData framePos:%u channels:%d nframe:%u processed?:%d\n", framePos, channels, nframe, processed());
       
       if ((song->bounceTrack != this) && !noInRoute()) {
             RouteList* irl = inRoutes();
             iRoute i = irl->begin();
-            i->track->copyData(framePos, channels, nframe, bp);
+            if(i->track->isMidiTrack())
+            {
+              if(debugMsg)
+                printf("WaveTrack::getData: Error: First route is a midi track route!\n");
+              return false;
+            }
+            // p3.3.38
+            //((AudioTrack*)i->track)->copyData(framePos, channels, nframe, bp);
+            ((AudioTrack*)i->track)->copyData(framePos, channels, 
+                                              //(i->track->type() == Track::AUDIO_SOFTSYNTH && i->channel != -1) ? i->channel : 0, 
+                                              i->channel, 
+                                              i->channels,
+                                              nframe, bp);
+            
             ++i;
             for (; i != irl->end(); ++i)
-                  i->track->addData(framePos, channels, nframe, bp);
+            {
+              if(i->track->isMidiTrack())
+              {
+                if(debugMsg)
+                  printf("WaveTrack::getData: Error: Route is a midi track route!\n");
+                //return false;
+                continue;
+              }
+              // p3.3.38
+              //((AudioTrack*)i->track)->addData(framePos, channels, nframe, bp);
+              ((AudioTrack*)i->track)->addData(framePos, channels, 
+                                               //(i->track->type() == Track::AUDIO_SOFTSYNTH && i->channel != -1) ? i->channel : 0, 
+                                               i->channel, 
+                                               i->channels,
+                                               nframe, bp);
+              
+            }
             if (recordFlag()) {
                   if (audio->isRecording() && recFile()) {
                         if (audio->freewheel()) {
@@ -250,7 +301,6 @@ bool WaveTrack::getData(unsigned framePos, int channels, unsigned nframe, float*
       else {
       */
       
-            // Added by Tim. p3.3.16
             //printf("WaveTrack::getData no out routes\n");
             
             if (audio->freewheel()) {
@@ -281,6 +331,10 @@ bool WaveTrack::getData(unsigned framePos, int channels, unsigned nframe, float*
                                     }
                               }
                         }
+                        
+                        // p3.3.41
+                        //fprintf(stderr, "WaveTrack::getData %s data: nframe:%ld %e %e %e %e\n", name().latin1(), nframe, bp[0][0], bp[0][1], bp[0][2], bp[0][3]);
+                        
                   }
             //}
       return true;
