@@ -19,14 +19,19 @@ class GlobalSettingsConfig : public GlobalSettingsDialogBase {
       Q_OBJECT
 
    private slots:
+      void updateSettings();
       void apply();
       void ok();
       void cancel();
       void mixerCurrent();
+      void mixer2Current();
       void bigtimeCurrent();
       void arrangerCurrent();
       void transportCurrent();
 
+    protected:
+      void showEvent(QShowEvent*);
+      
    public:
       GlobalSettingsConfig(QWidget* parent=0, const char* name=0);
       };
