@@ -1,7 +1,7 @@
 //=========================================================
 //  MusE
 //  Linux Music Editor
-//    $Id: lcombo.cpp,v 1.1.1.1 2003/10/27 18:54:45 wschweer Exp $
+//    $Id: lcombo.cpp,v 1.1.1.1.2.3 2009/07/01 22:14:56 spamatica Exp $
 //  (C) Copyright 2000 Werner Schweer (ws@seh.de)
 //=========================================================
 
@@ -37,3 +37,11 @@ void LabelCombo::insertItem(const QString& txt, int index)
       box->insertItem(txt, index);
       }
 
+void LabelCombo::clearFocus()
+{
+  box->clearFocus();
+}
+void LabelCombo::setFocusPolicy ( FocusPolicy fp )
+{
+  box->setFocusPolicy(fp);
+}
\ No newline at end of file
