@@ -1,7 +1,7 @@
 //=========================================================
 //  MusE
 //  Linux Music Editor
-//    $Id: lcombo.h,v 1.1.1.1 2003/10/27 18:54:29 wschweer Exp $
+//    $Id: lcombo.h,v 1.1.1.1.2.3 2009/07/01 22:14:56 spamatica Exp $
 //  (C) Copyright 2000 Werner Schweer (ws@seh.de)
 //=========================================================
 
@@ -25,12 +25,16 @@ class LabelCombo : public QWidget {
    signals:
       void activated(int);
 
+   public slots:
+      void clearFocus();
    public:
       LabelCombo(const QString& label, QWidget* parent,
          const char* name=0);
       void insertItem(const QString& txt, int index=-1);
       void setCurrentItem(int i) { box->setCurrentItem(i); }
       void setListBox(QListBox* lb) { box->setListBox(lb); }
+      void setFocusPolicy ( FocusPolicy fp );
+  
       };
 
 #endif
