@@ -31,10 +31,12 @@
 #include "sync.h"
 #include "globals.h"
 #include "midisyncimpl.h"
+#include "driver/audiodev.h"
+#include "audio.h"
 
-enum { DEVCOL_NO = 0, DEVCOL_NAME, DEVCOL_IN, DEVCOL_TICKIN, DEVCOL_RID, DEVCOL_RCLK, DEVCOL_RMMC, DEVCOL_RMTC, 
-//enum { DEVCOL_NAME = 0, DEVCOL_IN, DEVCOL_RID, DEVCOL_RCLK, DEVCOL_RMMC, DEVCOL_RMTC, 
-       DEVCOL_TID, DEVCOL_TCLK, DEVCOL_TMMC, DEVCOL_TMTC };
+enum { DEVCOL_NO = 0, DEVCOL_NAME, DEVCOL_IN, DEVCOL_TICKIN, DEVCOL_MRTIN, DEVCOL_MMCIN, DEVCOL_MTCIN, DEVCOL_MTCTYPE, 
+       DEVCOL_RID, DEVCOL_RCLK, DEVCOL_RMRT, DEVCOL_RMMC, DEVCOL_RMTC, DEVCOL_RREWSTART, 
+       DEVCOL_TID, DEVCOL_TCLK, DEVCOL_TMRT, DEVCOL_TMMC, DEVCOL_TMTC, /* DEVCOL_TREWSTART, */  };
 
 //MidiSyncInfo tmpMidiSyncPorts[MIDI_PORTS];
 
@@ -55,16 +57,24 @@ void MSyncHeaderTip::maybeTip(const QPoint &pos)
             case DEVCOL_NO:       p = QHeader::tr("Port Number"); break;
             case DEVCOL_NAME:     p = QHeader::tr("Name of the midi device associated with"
                        " this port number"); break;
-            case DEVCOL_IN:       p = QHeader::tr("Midi realtime input detected"); break;
+            case DEVCOL_IN:       p = QHeader::tr("Midi clock input detected"); break;
             case DEVCOL_TICKIN:   p = QHeader::tr("Midi tick input detected"); break;
-            case DEVCOL_RID:      p = QHeader::tr("Receive id number. Double click to edit."); break;
-            case DEVCOL_RCLK:     p = QHeader::tr("Accept midi realtime input"); break;
+            case DEVCOL_MRTIN:    p = QHeader::tr("Midi real time input detected"); break;
+            case DEVCOL_MMCIN:    p = QHeader::tr("MMC input detected"); break;
+            case DEVCOL_MTCIN:    p = QHeader::tr("MTC input detected"); break;
+            case DEVCOL_MTCTYPE:  p = QHeader::tr("Detected SMPTE format"); break;
+            case DEVCOL_RID:      p = QHeader::tr("Receive id number. 127 = Global. Double click to edit."); break;
+            case DEVCOL_RCLK:     p = QHeader::tr("Accept midi clock input"); break;
+            case DEVCOL_RMRT:     p = QHeader::tr("Accept midi real time input"); break;
             case DEVCOL_RMMC:     p = QHeader::tr("Accept MMC input"); break;
             case DEVCOL_RMTC:     p = QHeader::tr("Accept MTC input"); break;
-            case DEVCOL_TID:      p = QHeader::tr("Transmit id number. Double click to edit."); break;
-            case DEVCOL_TCLK:     p = QHeader::tr("Send midi realtime output"); break;
+            case DEVCOL_RREWSTART: p = QHeader::tr("Receive start rewinds before playing"); break;
+            case DEVCOL_TID:      p = QHeader::tr("Transmit id number. 127 = Global. Double click to edit."); break;
+            case DEVCOL_TCLK:     p = QHeader::tr("Send midi clock output"); break;
+            case DEVCOL_TMRT:     p = QHeader::tr("Send midi realtime output"); break;
             case DEVCOL_TMMC:     p = QHeader::tr("Send MMC output"); break;
             case DEVCOL_TMTC:     p = QHeader::tr("Send MTC output"); break;
+            //case DEVCOL_TREWSTART: p = QHeader::tr("Send continue instead of start"); break;
             default: return;
             }
       tip(r, p);
@@ -85,35 +95,62 @@ QString MSyncWhatsThis::text(const QPoint& pos)
             case DEVCOL_NAME:
                   return QHeader::tr("Name of the midi device associated with this port number");
             case DEVCOL_IN:
-                  return QHeader::tr("Midi realtime input detected, including clock/start/stop/continue, and song position. "
-                                     "Current port actually used is red. Click to force a port to be current.");
+                  return QHeader::tr("Midi clock input detected.\n"
+                                     "Current port actually used is red.\nClick to force a port to be used.");
             case DEVCOL_TICKIN:
                   return QHeader::tr("Midi tick input detected");
+            case DEVCOL_MRTIN:
+                  return QHeader::tr("Midi realtime input detected, including\n start/stop/continue, and song position.");
+            case DEVCOL_MMCIN:
+                  return QHeader::tr("MMC input detected, including stop/play/deferred play, and locate.");
+                                     //"Current port actually used is red. Click to force a port to be current.");
+            case DEVCOL_MTCIN:
+                  return QHeader::tr("MTC input detected, including forward quarter-frame sync and full-frame locate.\n"
+                                     "Current port actually used is red. Click to force a port to be current.");
+            case DEVCOL_MTCTYPE:
+                  return QHeader::tr("Detected SMPTE format: 24fps, 25fps, 30fps drop frame, or 30fps non-drop\n"
+                                     "Detects format of MTC quarter and full frame, and MMC locate.");
             case DEVCOL_RID:
-                  return QHeader::tr("Receive id number");
+                  return QHeader::tr("Receive id number. 127 = global receive all, even if not global.");
             case DEVCOL_RCLK:
-                  return QHeader::tr("Accept midi realtime input, including clock/start/stop/continue, and song position. "
-                                     "Only one input is used for clock. Auto-acquire: If two or more port realtime inputs "
-                                     "are enabled, the first clock detected is used, until clock is lost, then another "
-                                     "can take over. Non-clock events (start,stop etc) are accepted by ALL enabled ports. "
-                                     "This means you may have several master devices connected, and muse will accept input "
-                                     "from any, including one clock (best if each turns off its clock at stop, so muse can "
-                                     "re-acquire the clock from another port. Click on detect indicator to force another.) ");
+                  return QHeader::tr("Accept midi clock input. Only one input is used for clock.\n"
+                                     "Auto-acquire: If two or more port realtime inputs are enabled,\n"
+                                     " the first clock detected is used, until clock is lost,\n"
+                                     " then another can take over. Best if each turns off its clock\n" 
+                                     " at stop, so MusE can re-acquire the clock from another port.\n"
+                                     "Click on detect indicator to force another.");
+            case DEVCOL_RMRT:
+                  return QHeader::tr("Accept midi realtime input, including\n start/stop/continue, and song position.\n"
+                                     "Non-clock events (start,stop etc) are\n accepted by ALL enabled ports.\n"
+                                     "This means you may have several master\n devices connected, and muse will accept\n"
+                                     " input from them.");
             case DEVCOL_RMMC:
-                  return QHeader::tr("Accept MMC input");
+                  return QHeader::tr("Accept MMC input, including stop/play/deferred play, and locate.");
             case DEVCOL_RMTC:
-                  return QHeader::tr("Accept MTC input");
+                  return QHeader::tr("Accept MTC input, including forward quarter-frame sync and full-frame locate.\n"
+                                     "See 'rc' column for more help.");
+            case DEVCOL_RREWSTART:
+                  return QHeader::tr("When start is received, rewind before playing.\n"
+                                     "Note: It may be impossible to rewind fast\n"
+                                     " enough to synchronize with the external device.");
             case DEVCOL_TID:
-                  return QHeader::tr("Transmit id number");
+                  return QHeader::tr("Transmit id number. 127 = global transmit to all.");
             case DEVCOL_TCLK:
-                  return QHeader::tr("Send midi realtime output, including clock/start/stop/continue, and song position. "
-                                     "If 'Slave to External Sync' is chosen, muse can re-transmit midi realtime input "
-                                     " messages to any other chosen ports. This means you may have several slave devices "
-                                     "connected, and muse can re-send realtime messages to any or all of them. ");
+                  return QHeader::tr("Send midi clock output. If 'Slave to External Sync' is chosen,\n"
+                                     " muse can re-transmit clock to any other chosen ports.");
+            case DEVCOL_TMRT:
+                  return QHeader::tr("Send midi realtime output, including start/stop/continue,\n"
+                                     " and song position. If 'Slave to external sync' is chosen,\n"
+                                     " muse can re-transmit midi realtime input messages to any\n"
+                                     " other chosen ports. This means you may have several slave\n"
+                                     " devices connected, and muse can re-send realtime messages\n"
+                                     " to any or all of them.");
             case DEVCOL_TMMC:
                   return QHeader::tr("Send MMC output");
             case DEVCOL_TMTC:
                   return QHeader::tr("Send MTC output");
+            //case DEVCOL_TREWSTART:
+            //      return QHeader::tr("When transport is starting, send continue instead of start.\n");
             default:
                   break;
             }
@@ -159,11 +196,15 @@ void MidiSyncLViewItem::copyFromSyncInfo(const MidiSyncInfo &sp)
   _idOut         = sp.idOut();
   _idIn          = sp.idIn();
   _sendMC        = sp.MCOut();
+  _sendMRT       = sp.MRTOut();
   _sendMMC       = sp.MMCOut();
   _sendMTC       = sp.MTCOut();
   _recMC         = sp.MCIn();
+  _recMRT        = sp.MRTIn();
   _recMMC        = sp.MMCIn();
   _recMTC        = sp.MTCIn();
+  _recRewOnStart = sp.recRewOnStart();
+  //_sendContNotStart = sp.sendContNotStart();
 }
 
 //---------------------------------------------------------
@@ -176,11 +217,15 @@ void MidiSyncLViewItem::copyToSyncInfo(MidiSyncInfo &sp)
   sp.setIdOut(_idOut);
   sp.setIdIn(_idIn);
   sp.setMCOut(_sendMC);
+  sp.setMRTOut(_sendMRT);
   sp.setMMCOut(_sendMMC);
   sp.setMTCOut(_sendMTC);
   sp.setMCIn(_recMC);
+  sp.setMRTIn(_recMRT);
   sp.setMMCIn(_recMMC);
   sp.setMTCIn(_recMTC);
+  sp.setRecRewOnStart(_recRewOnStart);
+  //sp.setSendContNotStart(_sendContNotStart);
 }
 
 //---------------------------------------------------------
@@ -235,41 +280,65 @@ MidiSyncConfig::MidiSyncConfig(QWidget* parent, const char* name)
       devicesListView->setAllColumnsShowFocus(true);
       devicesListView->addColumn(tr("Port"));
       devicesListView->addColumn(tr("Device Name"), 120);
-      devicesListView->addColumn(tr("i"));
+      devicesListView->addColumn(tr("c"));
+      devicesListView->addColumn(tr("k"));
+      devicesListView->addColumn(tr("r"));
+      devicesListView->addColumn(tr("m"));
       devicesListView->addColumn(tr("t"));
+      devicesListView->addColumn(tr("type"));
       devicesListView->addColumn(tr("rid"));  // Receive
-      devicesListView->addColumn(tr("rmc")); // Receive
-      devicesListView->addColumn(tr("rmmc")); // Receive
-      devicesListView->addColumn(tr("rmtc")); // Receive
+      devicesListView->addColumn(tr("rc")); // Receive
+      devicesListView->addColumn(tr("rr")); // Receive
+      devicesListView->addColumn(tr("rm")); // Receive
+      devicesListView->addColumn(tr("rt")); // Receive
+      devicesListView->addColumn(tr("rw")); // Receive
       devicesListView->addColumn(tr("tid"));  // Transmit
-      devicesListView->addColumn(tr("tmc")); // Transmit
-      devicesListView->addColumn(tr("tmmc")); // Transmit
-      devicesListView->addColumn(tr("tmtc")); // Transmit
+      devicesListView->addColumn(tr("tc")); // Transmit
+      devicesListView->addColumn(tr("tr")); // Transmit
+      devicesListView->addColumn(tr("tm")); // Transmit
+      devicesListView->addColumn(tr("tt")); // Transmit
+      //devicesListView->addColumn(tr("trs")); // Transmit
       devicesListView->setFocusPolicy(NoFocus);
 
       devicesListView->setColumnAlignment(DEVCOL_NO, AlignHCenter);
       devicesListView->setColumnAlignment(DEVCOL_IN, AlignCenter);
       devicesListView->setColumnAlignment(DEVCOL_TICKIN, AlignCenter);
+      devicesListView->setColumnAlignment(DEVCOL_MRTIN, AlignCenter);
+      devicesListView->setColumnAlignment(DEVCOL_MMCIN, AlignCenter);
+      devicesListView->setColumnAlignment(DEVCOL_MTCIN, AlignCenter);
+      //devicesListView->setColumnAlignment(DEVCOL_MTCTYPE, AlignCenter);
       //devicesListView->setColumnAlignment(DEVCOL_RID, AlignCenter);
       devicesListView->setColumnAlignment(DEVCOL_RCLK, AlignCenter);
+      devicesListView->setColumnAlignment(DEVCOL_RMRT, AlignCenter);
       devicesListView->setColumnAlignment(DEVCOL_RMMC, AlignCenter);
       devicesListView->setColumnAlignment(DEVCOL_RMTC, AlignCenter);
+      devicesListView->setColumnAlignment(DEVCOL_RREWSTART, AlignCenter);
       //devicesListView->setColumnAlignment(DEVCOL_TID, AlignCenter);
       devicesListView->setColumnAlignment(DEVCOL_TCLK, AlignCenter);
+      devicesListView->setColumnAlignment(DEVCOL_TMRT, AlignCenter);
       devicesListView->setColumnAlignment(DEVCOL_TMMC, AlignCenter);
       devicesListView->setColumnAlignment(DEVCOL_TMTC, AlignCenter);
+      //devicesListView->setColumnAlignment(DEVCOL_TREWSTART, AlignCenter);
       devicesListView->header()->setResizeEnabled(false, DEVCOL_NO);
       devicesListView->header()->setResizeEnabled(false, DEVCOL_IN);
       devicesListView->header()->setResizeEnabled(false, DEVCOL_TICKIN);
+      devicesListView->header()->setResizeEnabled(false, DEVCOL_MRTIN);
+      devicesListView->header()->setResizeEnabled(false, DEVCOL_MMCIN);
+      devicesListView->header()->setResizeEnabled(false, DEVCOL_MTCIN);
       devicesListView->header()->setResizeEnabled(false, DEVCOL_RCLK);
+      devicesListView->header()->setResizeEnabled(false, DEVCOL_RMRT);
       devicesListView->header()->setResizeEnabled(false, DEVCOL_RMMC);
       devicesListView->header()->setResizeEnabled(false, DEVCOL_RMTC);
+      devicesListView->header()->setResizeEnabled(false, DEVCOL_RMTC);
+      devicesListView->header()->setResizeEnabled(false, DEVCOL_RREWSTART);
       devicesListView->header()->setResizeEnabled(false, DEVCOL_TCLK);
+      devicesListView->header()->setResizeEnabled(false, DEVCOL_TMRT);
       devicesListView->header()->setResizeEnabled(false, DEVCOL_TMMC);
-      devicesListView->header()->setResizeEnabled(false, DEVCOL_TMTC);
+      //devicesListView->header()->setResizeEnabled(false, DEVCOL_TREWSTART);
       //devicesListView->setResizeMode(QListView::LastColumn);
       devicesListView->setResizeMode(QListView::NoColumn);
 
+
       new MSyncWhatsThis(devicesListView, devicesListView->header());
       _synctooltip = new MSyncHeaderTip(devicesListView->header());
       //MSyncHeaderTip::add(devicesListView->header(), QString("Midi sync ports"));
@@ -295,8 +364,9 @@ MidiSyncConfig::MidiSyncConfig(QWidget* parent, const char* name)
       connect(extSyncCheckbox, SIGNAL(clicked()), SLOT(syncChanged()));
       connect(mtcSyncType, SIGNAL(activated(int)), SLOT(syncChanged()));
       connect(useJackTransportCheckbox, SIGNAL(clicked()), SLOT(syncChanged()));
+      connect(jackTransportMasterCheckbox, SIGNAL(clicked()), SLOT(syncChanged()));
       connect(&extSyncFlag, SIGNAL(valueChanged(bool)), SLOT(extSyncChanged(bool)));
-      
+      connect(syncDelaySpinBox, SIGNAL(valueChanged(int)), SLOT(syncChanged()));
   
       // Done in show().
       //connect(song, SIGNAL(songChanged(int)), SLOT(songChanged(int)));
@@ -333,18 +403,35 @@ void MidiSyncConfig::songChanged(int flags)
       
       extSyncCheckbox->blockSignals(true);
       useJackTransportCheckbox->blockSignals(true);
+      jackTransportMasterCheckbox->blockSignals(true);
+      syncDelaySpinBox->blockSignals(true);
       extSyncCheckbox->setChecked(extSyncFlag.value());
-      useJackTransportCheckbox->setChecked(useJackTransport);
+      useJackTransportCheckbox->setChecked(useJackTransport.value());
+      jackTransportMasterCheckbox->setChecked(jackTransportMaster);
+      //jackTransportMasterCheckbox->setEnabled(useJackTransport);
+      syncDelaySpinBox->setValue(syncSendFirstClockDelay);
+      syncDelaySpinBox->blockSignals(false);
+      jackTransportMasterCheckbox->blockSignals(false);
       useJackTransportCheckbox->blockSignals(false);
       extSyncCheckbox->blockSignals(false);
       
       mtcSyncType->setCurrentItem(mtcType);
 
+      mtcOffH->blockSignals(true);
+      mtcOffM->blockSignals(true);
+      mtcOffS->blockSignals(true);
+      mtcOffF->blockSignals(true);
+      mtcOffSf->blockSignals(true);
       mtcOffH->setValue(mtcOffset.h());
       mtcOffM->setValue(mtcOffset.m());
       mtcOffS->setValue(mtcOffset.s());
       mtcOffF->setValue(mtcOffset.f());
       mtcOffSf->setValue(mtcOffset.sf());
+      mtcOffH->blockSignals(false);
+      mtcOffM->blockSignals(false);
+      mtcOffS->blockSignals(false);
+      mtcOffF->blockSignals(false);
+      mtcOffSf->blockSignals(false);
 
       updateSyncInfoLV();
       
@@ -427,7 +514,142 @@ void MidiSyncConfig::heartBeat()
               lvi->setPixmap(DEVCOL_TICKIN, *dothIcon);
             }  
           }
+        
+          sdet = midiPorts[port].syncInfo().MRTDetect();
+          if(sdet)
+          {
+            if(!lvi->_MRTDet)
+            {
+              // Added by Tim. p3.3.6
+              //printf("MidiSyncConfig::heartBeat setting MRT on icon\n");
+          
+              lvi->_MRTDet = true;
+              lvi->setPixmap(DEVCOL_MRTIN, *dotIcon);
+            }  
+          } 
+          else
+          {
+            if(lvi->_MRTDet)
+            {
+              // Added by Tim. p3.3.6
+              //printf("MidiSyncConfig::heartBeat setting MRT off icon\n");
+          
+              lvi->_MRTDet = false;
+              lvi->setPixmap(DEVCOL_MRTIN, *dothIcon);
+            }  
+          }
+        
+          int type = midiPorts[port].syncInfo().recMTCtype();
+          sdet = midiPorts[port].syncInfo().MMCDetect();
+          bool mtcdet = midiPorts[port].syncInfo().MTCDetect();
+          if(sdet)
+          {
+            if(!lvi->_MMCDet)
+            {
+              // Added by Tim. p3.3.6
+              //printf("MidiSyncConfig::heartBeat setting MMC on icon\n");
+          
+              lvi->_MMCDet = true;
+              lvi->setPixmap(DEVCOL_MMCIN, *dotIcon);
+            }
+            // MMC locate command can contain SMPTE format type. Update now.
+            if(!mtcdet && lvi->_recMTCtype != type)
+            {
+              lvi->_recMTCtype = type;
+              switch(type)
+              {
+                case 0:
+                  lvi->setText(DEVCOL_MTCTYPE, "24");
+                break;  
+                case 1:
+                  lvi->setText(DEVCOL_MTCTYPE, "25");
+                break;  
+                case 2:
+                  lvi->setText(DEVCOL_MTCTYPE, "30D");
+                break;  
+                case 3:
+                  lvi->setText(DEVCOL_MTCTYPE, "30N");
+                break;  
+                default:
+                  lvi->setText(DEVCOL_MTCTYPE, "??");
+                break;  
+              }  
+            }
+          } 
+          else
+          {
+            if(lvi->_MMCDet)
+            {
+              // Added by Tim. p3.3.6
+              //printf("MidiSyncConfig::heartBeat setting MMC off icon\n");
+          
+              lvi->_MMCDet = false;
+              lvi->setPixmap(DEVCOL_MMCIN, *dothIcon);
+            }  
+          }
+          
+          if(mtcdet)
+          {
+            if(port == curMidiSyncInPort)
+            {
+              if(!lvi->_curMTCDet)
+              {
+                // Added by Tim. p3.3.6
+                //printf("MidiSyncConfig::heartBeat setting current red icon\n");
+            
+                lvi->_curMTCDet = true;
+                lvi->_MTCDet = false;
+                lvi->setPixmap(DEVCOL_MTCIN, *record1_Icon);
+              }  
+            }
+            else
+            if(!lvi->_MTCDet)
+            {
+              // Added by Tim. p3.3.6
+              //printf("MidiSyncConfig::heartBeat setting MTC on icon\n");
+          
+              lvi->_MTCDet = true;
+              lvi->_curMTCDet = false;
+              lvi->setPixmap(DEVCOL_MTCIN, *dotIcon);
+            }
+            
+            if(lvi->_recMTCtype != type)
+            {
+              lvi->_recMTCtype = type;
+              switch(type)
+              {
+                case 0:
+                  lvi->setText(DEVCOL_MTCTYPE, "24");
+                break;  
+                case 1:
+                  lvi->setText(DEVCOL_MTCTYPE, "25");
+                break;  
+                case 2:
+                  lvi->setText(DEVCOL_MTCTYPE, "30D");
+                break;  
+                case 3:
+                  lvi->setText(DEVCOL_MTCTYPE, "30N");
+                break;  
+                default:
+                  lvi->setText(DEVCOL_MTCTYPE, "??");
+                break;  
+              }  
+            }
+          } 
+          else
+          {
+            if(lvi->_curMTCDet || lvi->_MTCDet)
+            {
+              // Added by Tim. p3.3.6
+              //printf("MidiSyncConfig::heartBeat setting MTC off icon\n");
+          
+              lvi->_MTCDet = false;
+              lvi->_curMTCDet = false;
+              lvi->setPixmap(DEVCOL_MTCIN, *dothIcon);
+            }  
+          }
         }
+        
         //MidiDevice* dev = lvi->device();
         //bool sdet = dev->syncInfo().MCSyncDetect();
         //if(lvi->pixmap(DEVCOL_IN) != (sdet ? *dotIcon : *dothIcon))
@@ -449,6 +671,8 @@ void MidiSyncConfig::syncChanged()
       {
       setDirty();
       
+      //jackTransportMasterCheckbox->setEnabled(useJackTransport);
+      
       //acceptMTCCheckbox->setEnabled(val);
 //      acceptMTCCheckbox->setEnabled(false);
 //      acceptMCCheckbox->setEnabled(val);
@@ -549,6 +773,8 @@ void MidiSyncConfig::apply()
 //      genMCSync   = mcSync->isChecked();
 //      genMMC      = midiMachineControl->isChecked();
 
+      syncSendFirstClockDelay = syncDelaySpinBox->value();
+      
       mtcType     = mtcSyncType->currentItem();
       //extSyncFlag.setValue(syncMode->id(syncMode->selected()));
       //extSyncFlag.blockSignals(true);
@@ -556,7 +782,14 @@ void MidiSyncConfig::apply()
 //      if(extSyncFlag.value())
 //        song->setMasterFlag(false);
       //extSyncFlag.blockSignals(false);
-      useJackTransport = useJackTransportCheckbox->isChecked();
+      useJackTransport.setValue(useJackTransportCheckbox->isChecked());
+//      if(useJackTransport)
+        jackTransportMaster = jackTransportMasterCheckbox->isChecked();
+//      else  
+//        jackTransportMaster = false;
+//      jackTransportMasterCheckbox->setEnabled(useJackTransport);
+      if(audioDevice)
+        audioDevice->setMaster(jackTransportMaster);      
 
       mtcOffset.setH(mtcOffH->value());
       mtcOffset.setM(mtcOffM->value());
@@ -589,7 +822,8 @@ void MidiSyncConfig::apply()
   if(applyButton->isEnabled())
     applyButton->setEnabled(false);
   
-  midiSeq->msgUpdatePollFd();
+  // Do not call this. Causes freeze sometimes. Only will be needed if extra pollfds are used by midi seq thread.
+  //midiSeq->msgUpdatePollFd();
 }
 
 //---------------------------------------------------------
@@ -603,6 +837,16 @@ void MidiSyncConfig::updateSyncInfoLV()
       {
             MidiPort* port  = &midiPorts[i];
             MidiDevice* dev = port->device();
+            // p3.3.31
+            // Don't show if it is a synthesizer device.
+            // Hmm, some synths might support transport commands or even sync?
+            // If anything, the DSSI or VST synths just might... 
+            // TODO: Must test to see if it screws any of them up, especially clock out.
+            // Also, if we do this, we must prevent such messages from reaching
+            //  those ports at several other places in the code.
+            //if(dev && dev->isSynti())
+            //  continue;
+              
             QString s;
             s.setNum(i+1);
             MidiSyncLViewItem* lvi = new MidiSyncLViewItem(devicesListView);
@@ -652,6 +896,95 @@ void MidiSyncConfig::updateSyncInfoLV()
               lvi->setPixmap(DEVCOL_TICKIN, *dothIcon);
             }
             
+            if(portsi.MRTDetect())
+            {
+              lvi->_MRTDet = true;
+              lvi->setPixmap(DEVCOL_MRTIN, *dotIcon);
+            }
+            else
+            {
+              lvi->_MRTDet = false;
+              lvi->setPixmap(DEVCOL_MRTIN, *dothIcon);
+            }
+            
+            if(portsi.MMCDetect())
+            {
+              lvi->_MMCDet = true;
+              lvi->setPixmap(DEVCOL_MMCIN, *dotIcon);
+              // MMC locate command can have SMPTE format bits...
+              if(lvi->_recMTCtype != portsi.recMTCtype())
+              {
+                switch(portsi.recMTCtype())
+                {
+                  case 0:
+                    lvi->setText(DEVCOL_MTCTYPE, "24");
+                  break;  
+                  case 1:
+                    lvi->setText(DEVCOL_MTCTYPE, "25");
+                  break;  
+                  case 2:
+                    lvi->setText(DEVCOL_MTCTYPE, "30D");
+                  break;  
+                  case 3:
+                    lvi->setText(DEVCOL_MTCTYPE, "30N");
+                  break;  
+                  default:
+                    lvi->setText(DEVCOL_MTCTYPE, "??");
+                  break;  
+                }  
+              }  
+            }
+            else
+            {
+              lvi->_MMCDet = false;
+              lvi->setPixmap(DEVCOL_MMCIN, *dothIcon);
+            }
+            
+            if(portsi.MTCDetect())
+            {
+              if(i == curMidiSyncInPort)
+              {
+                lvi->_curMTCDet = true;
+                lvi->_MTCDet = false;
+                lvi->setPixmap(DEVCOL_MTCIN, *record1_Icon);
+              }
+              else
+              {
+                lvi->_curMTCDet = false;
+                lvi->_MTCDet = true;
+                lvi->setPixmap(DEVCOL_MTCIN, *dotIcon);
+              }
+                
+              if(lvi->_recMTCtype != portsi.recMTCtype())
+              {
+                switch(portsi.recMTCtype())
+                {
+                  case 0:
+                    lvi->setText(DEVCOL_MTCTYPE, "24");
+                  break;  
+                  case 1:
+                    lvi->setText(DEVCOL_MTCTYPE, "25");
+                  break;  
+                  case 2:
+                    lvi->setText(DEVCOL_MTCTYPE, "30D");
+                  break;  
+                  case 3:
+                    lvi->setText(DEVCOL_MTCTYPE, "30N");
+                  break;  
+                  default:
+                    lvi->setText(DEVCOL_MTCTYPE, "??");
+                  break;  
+                }  
+              }  
+            }
+            else
+            {
+              lvi->_curMTCDet = false;
+              lvi->_MTCDet = false;
+              lvi->setPixmap(DEVCOL_MTCIN, *dothIcon);
+              //lvi->setText(DEVCOL_MTCTYPE, "--");
+            }
+            
             //lvi->setText(DEVCOL_RID,    QString().setNum(si.idIn()) );
             //lvi->setRenameEnabled(DEVCOL_RID, true);
             //lvi->setPixmap(DEVCOL_RCLK, si.MCIn() ? *dotIcon : *dothIcon);
@@ -659,18 +992,22 @@ void MidiSyncConfig::updateSyncInfoLV()
             //lvi->setPixmap(DEVCOL_RMTC, si.MTCIn() ? *dotIcon : *dothIcon);
             lvi->setText(DEVCOL_RID,    QString().setNum(lvi->_idIn) );
             lvi->setPixmap(DEVCOL_RCLK, lvi->_recMC ? *dotIcon : *dothIcon);
+            lvi->setPixmap(DEVCOL_RMRT, lvi->_recMRT ? *dotIcon : *dothIcon);
             lvi->setPixmap(DEVCOL_RMMC, lvi->_recMMC ? *dotIcon : *dothIcon);
             lvi->setPixmap(DEVCOL_RMTC, lvi->_recMTC ? *dotIcon : *dothIcon);
+            lvi->setPixmap(DEVCOL_RREWSTART, lvi->_recRewOnStart ? *dotIcon : *dothIcon);
             
             //lvi->setText(DEVCOL_TID,    QString().setNum(si.idOut()) );
             //lvi->setRenameEnabled(DEVCOL_TID, true);
             //lvi->setPixmap(DEVCOL_TCLK, si.MCOut() ? *dotIcon : *dothIcon);
             //lvi->setPixmap(DEVCOL_TMMC, si.MMCOut() ? *dotIcon : *dothIcon);
             //lvi->setPixmap(DEVCOL_TMTC, si.MTCOut() ? *dotIcon : *dothIcon);
-            lvi->setText(DEVCOL_TID,    QString().setNum(lvi->_idOut) );
-            lvi->setPixmap(DEVCOL_TCLK, lvi->_sendMC ? *dotIcon : *dothIcon);
-            lvi->setPixmap(DEVCOL_TMMC, lvi->_sendMMC ? *dotIcon : *dothIcon);
-            lvi->setPixmap(DEVCOL_TMTC, lvi->_sendMTC ? *dotIcon : *dothIcon);
+            lvi->setText(DEVCOL_TID,          QString().setNum(lvi->_idOut) );
+            lvi->setPixmap(DEVCOL_TCLK,       lvi->_sendMC ? *dotIcon : *dothIcon);
+            lvi->setPixmap(DEVCOL_TMRT,       lvi->_sendMRT ? *dotIcon : *dothIcon);
+            lvi->setPixmap(DEVCOL_TMMC,       lvi->_sendMMC ? *dotIcon : *dothIcon);
+            lvi->setPixmap(DEVCOL_TMTC,       lvi->_sendMTC ? *dotIcon : *dothIcon);
+            //lvi->setPixmap(DEVCOL_TREWSTART,  lvi->_sendContNotStart ? *dotIcon : *dothIcon);
             
             devicesListView->insertItem(lvi);
       }
@@ -748,14 +1085,47 @@ void MidiSyncConfig::dlvClicked(int /*button*/, QListViewItem* item, const QPoin
                   //  and sync is in fact detected on this port, allow the user to force this port to now be the
                   //  current sync in port. 
                   //if(no != curMidiSyncInPort && si.MCIn() && midiPorts[no].syncInfo().MCSyncDetect())
-                  if(no != curMidiSyncInPort && lvi->_recMC && midiPorts[no].syncInfo().MCSyncDetect())
+                  //if(no != curMidiSyncInPort && lvi->_recMC && midiPorts[no].syncInfo().MCSyncDetect())
+                  if(no != curMidiSyncInPort)
                   {
-                    curMidiSyncInPort = no;
-                    lvi->setPixmap(DEVCOL_IN, *record1_Icon);
+                    if(lvi->_recMC && midiPorts[no].syncInfo().MCSyncDetect())
+                    {
+                      curMidiSyncInPort = no;
+                      lvi->setPixmap(DEVCOL_IN, *record1_Icon);
+                    }  
+                    if(lvi->_recMTC && midiPorts[no].syncInfo().MTCDetect())
+                    {
+                      curMidiSyncInPort = no;
+                      lvi->setPixmap(DEVCOL_MTCIN, *record1_Icon);
+                    }  
                   }  
                   break;
             case DEVCOL_TICKIN:
                   break;
+            case DEVCOL_MMCIN:
+                  break;
+            case DEVCOL_MTCIN:
+                  // If this is not the current midi sync in port, and sync in from this port is enabled,
+                  //  and sync is in fact detected on this port, allow the user to force this port to now be the
+                  //  current sync in port. 
+                  //if(no != curMidiSyncInPort && si.MTCIn() && midiPorts[no].syncInfo().MTCDetect())
+                  //if(no != curMidiSyncInPort && lvi->_recMTC && midiPorts[no].syncInfo().MTCDetect())
+                  if(no != curMidiSyncInPort)
+                  {
+                    if(lvi->_recMTC && midiPorts[no].syncInfo().MTCDetect())
+                    {
+                      curMidiSyncInPort = no;
+                      lvi->setPixmap(DEVCOL_MTCIN, *record1_Icon);
+                    }  
+                    if(lvi->_recMC && midiPorts[no].syncInfo().MCSyncDetect())
+                    {
+                      curMidiSyncInPort = no;
+                      lvi->setPixmap(DEVCOL_IN, *record1_Icon);
+                    }  
+                  }  
+                  break;
+            case DEVCOL_MTCTYPE:
+                  break;
             case DEVCOL_RID:
                   break;
             case DEVCOL_RCLK:
@@ -765,6 +1135,11 @@ void MidiSyncConfig::dlvClicked(int /*button*/, QListViewItem* item, const QPoin
                   lvi->setPixmap(DEVCOL_RCLK, lvi->_recMC ? *dotIcon : *dothIcon);
                   setDirty();
                   break;
+            case DEVCOL_RMRT:
+                  lvi->_recMRT = (lvi->_recMRT ? false : true);
+                  lvi->setPixmap(DEVCOL_RMRT, lvi->_recMRT ? *dotIcon : *dothIcon);
+                  setDirty();
+                  break;
             case DEVCOL_RMMC:
                   //si.setMMCIn(si.MMCIn() ? false : true);
                   //lvi->setPixmap(DEVCOL_RMMC, si.MMCIn() ? *dotIcon : *dothIcon);
@@ -779,6 +1154,11 @@ void MidiSyncConfig::dlvClicked(int /*button*/, QListViewItem* item, const QPoin
                   lvi->setPixmap(DEVCOL_RMTC, lvi->_recMTC ? *dotIcon : *dothIcon);
                   setDirty();
                   break;
+            case DEVCOL_RREWSTART:
+                  lvi->_recRewOnStart = (lvi->_recRewOnStart ? false : true);
+                  lvi->setPixmap(DEVCOL_RREWSTART, lvi->_recRewOnStart ? *dotIcon : *dothIcon);
+                  setDirty();
+                  break;
             case DEVCOL_TID:
                   break;
             case DEVCOL_TCLK:
@@ -788,6 +1168,11 @@ void MidiSyncConfig::dlvClicked(int /*button*/, QListViewItem* item, const QPoin
                   lvi->setPixmap(DEVCOL_TCLK, lvi->_sendMC ? *dotIcon : *dothIcon);
                   setDirty();
                   break;
+            case DEVCOL_TMRT:
+                  lvi->_sendMRT = (lvi->_sendMRT ? false : true);
+                  lvi->setPixmap(DEVCOL_TMRT, lvi->_sendMRT ? *dotIcon : *dothIcon);
+                  setDirty();
+                  break;
             case DEVCOL_TMMC:
                   //si.setMMCOut(si.MMCOut() ? false : true);
                   //lvi->setPixmap(DEVCOL_TMMC, si.MMCOut() ? *dotIcon : *dothIcon);
@@ -802,6 +1187,11 @@ void MidiSyncConfig::dlvClicked(int /*button*/, QListViewItem* item, const QPoin
                   lvi->setPixmap(DEVCOL_TMTC, lvi->_sendMTC ? *dotIcon : *dothIcon);
                   setDirty();
                   break;
+            //case DEVCOL_TREWSTART:
+            //      lvi->_sendContNotStart = (lvi->_sendContNotStart ? false : true);
+            //      lvi->setPixmap(DEVCOL_TREWSTART, lvi->_sendContNotStart ? *dotIcon : *dothIcon);
+            //      setDirty();
+            //      break;
       }
       //songChanged(-1);
 }
@@ -826,27 +1216,27 @@ void MidiSyncConfig::dlvDoubleClicked(QListViewItem* item, const QPoint&, int co
       bool ok = false;
       if(col == DEVCOL_RID)
       {
-        //int id = lvi->syncInfo().idIn();
-        int id = lvi->_idIn;
-        int newid = QInputDialog::getInteger("Muse: Sync info" , "Enter new id number:", id, 0, 127, 1, &ok, this);
+        //int val = lvi->syncInfo().idIn();
+        int val = lvi->_idIn;
+        int newval = QInputDialog::getInteger("Muse: Sync info" , "Enter new id number (127 = all):", val, 0, 127, 1, &ok, this);
         if(ok)
         {
-          //lvi->syncInfo().setIdIn(newid);
-          lvi->_idIn = newid;
-          lvi->setText(DEVCOL_RID, QString().setNum(newid)); 
+          //lvi->syncInfo().setIdIn(newval);
+          lvi->_idIn = newval;
+          lvi->setText(DEVCOL_RID, QString().setNum(newval)); 
         }  
       }  
       else
       if(col == DEVCOL_TID)
       {
-        //int id = lvi->syncInfo().idOut();
-        int id = lvi->_idOut;
-        int newid = QInputDialog::getInteger("Muse: Sync info" , "Enter new id number:", id, 0, 127, 1, &ok, this);
+        //int val = lvi->syncInfo().idOut();
+        int val = lvi->_idOut;
+        int newval = QInputDialog::getInteger("Muse: Sync info" , "Enter new id number (127 = global):", val, 0, 127, 1, &ok, this);
         if(ok)
         {
-          //lvi->syncInfo().setIdOut(newid);
-          lvi->_idOut = newid;
-          lvi->setText(DEVCOL_TID, QString().setNum(newid)); 
+          //lvi->syncInfo().setIdOut(newval);
+          lvi->_idOut = newval;
+          lvi->setText(DEVCOL_TID, QString().setNum(newval)); 
         }  
       }  
       
