@@ -1,7 +1,7 @@
 //=========================================================
 //  MusE
 //  Linux Music Editor
-//  $Id: fluid.cpp,v 1.18 2004/05/05 12:09:13 wschweer Exp $
+//  $Id: fluid.cpp,v 1.18.2.8 2009/12/06 10:05:00 terminator356 Exp $
 //
 //  This file is derived from fluid Synth and modified
 //    for MusE.
@@ -27,6 +27,8 @@
 #include <sched.h>
 
 #include "muse/midictrl.h"
+
+#include "muse/midi.h"
 #include "fluid.h"
 #include "fluidgui.h"
 
@@ -36,7 +38,7 @@
 
 class QWidget;
 
-static Mess* instantiate(int sr, QWidget*, const char* name)
+static Mess* instantiate(int sr, QWidget*, QString *projectPathPtr, const char* name)
       {
       ISynth* synth = new ISynth();
       synth->setSampleRate(sr);
@@ -54,7 +56,7 @@ static Mess* instantiate(int sr, QWidget*, const char* name)
 extern "C" {
       static MESS descriptor = {
             "fluid",
-            "Werner Schweer",
+            "fluid soundfont loader by Werner Schweer",   
             "0.1",      // fluid version string
             MESS_MAJOR_VERSION, MESS_MINOR_VERSION,
             instantiate,
@@ -291,31 +293,86 @@ ISynth::~ISynth()
             delete [] initBuffer;
       }
 
+//---------------------------------------------------------
+//   processMessages
+//   Called from host always, even if output path is unconnected.
+//---------------------------------------------------------
+
+void ISynth::processMessages()
+{
+  //Process messages from the gui
+  if (!_busy) 
+  {
+    //
+    //  get and process all pending events from the
+    //  synthesizer GUI
+    //
+    while (gui->fifoSize())
+          processEvent(gui->readEvent());
+  }
+}
+  
 //---------------------------------------------------------
 //   process
+//   Called from host, ONLY if output path is connected.
 //---------------------------------------------------------
 
 void ISynth::process(float** ports, int offset, int n)
       {
       if (!_busy) {
+            /*
             //
             //  get and process all pending events from the
             //  synthesizer GUI
             //
             while (gui->fifoSize())
                   processEvent(gui->readEvent());
+            */      
             fluid_synth_write_float(_fluidsynth, n, ports[0],
                offset, 1, ports[1], offset, 1);
             }
       // printf("%f %f\n", *ports[0], *(ports[0]+1));
       }
 
+//---------------------------------------------------------
+//   processEvent
+//    All events from the sequencer go here
+//---------------------------------------------------------
+bool ISynth::processEvent(const MidiPlayEvent& ev)
+      {
+      switch(ev.type()) {
+            case ME_CONTROLLER:
+                  setController(ev.channel(), ev.dataA(), ev.dataB());
+                  return true;
+            case ME_NOTEON:
+                  return playNote(ev.channel(), ev.dataA(), ev.dataB());
+            case ME_NOTEOFF:
+                  return playNote(ev.channel(), ev.dataA(), 0);
+            case ME_SYSEX:
+                  return sysex(ev.len(), ev.data());
+            case ME_PITCHBEND:
+                setController(ev.channel(), CTRL_PITCH, ev.dataA());
+                break;            
+            case ME_PROGRAM:
+                setController(ev.channel(), CTRL_PROGRAM, ev.dataA());
+                break;   
+            default:
+                break;
+            }
+      return false;
+      }
+
 //---------------------------------------------------------
 //   getPatchName
 //---------------------------------------------------------
 
-const char* ISynth::getPatchName(int /*ch*/, int val, int) const
+const char* ISynth::getPatchName(int /*ch*/, int val, int, bool /*drum*/) const
       {
+      int prog =   val & 0xff;
+      if(val == CTRL_VAL_UNKNOWN || prog == 0xff)
+            return "<unknown>";
+      prog &= 0x7f;
+      
       int hbank = (val & 0xff0000) >> 16;
       int lbank = (val & 0xff00) >> 8;
       if (hbank > 127)
@@ -324,8 +381,7 @@ const char* ISynth::getPatchName(int /*ch*/, int val, int) const
             lbank = 0;
       if (lbank == 127)       // drum HACK
             lbank = 128;
-      int prog =   val & 0x7f;
-      char* name = "---";
+      const char* name = "<unknown>";
 
       if (_busy) {
             printf("fluid: getPatchName(): busy!\n");
@@ -493,7 +549,7 @@ bool ISynth::init(const char* name)
       {
       fluid_settings_t* settings;
       settings = new_fluid_settings();
-      fluid_settings_setnum(settings, "synth.sample-rate", float(sampleRate()));
+      fluid_settings_setnum(settings, (char*) "synth.sample-rate", float(sampleRate()));
 
       _fluidsynth = new_fluid_synth(settings);
 
