@@ -1,7 +1,7 @@
 //=========================================================
 //  MusE
 //  Linux Music Editor
-//  $Id: organ.h,v 1.5.2.1 2005/12/29 23:33:50 spamatica Exp $
+//  $Id: organ.h,v 1.5.2.3 2009/11/19 04:20:33 terminator356 Exp $
 //
 //  Parts of this file taken from:
 //      Organ - Additive Organ Synthesizer Voice
@@ -174,11 +174,12 @@ class Organ : public Mess {
       OrganGui* gui;
 
    public:
+      virtual void processMessages();
       virtual void process(float**, int, int);
       virtual bool playNote(int channel, int pitch, int velo);
       virtual bool setController(int channel, int ctrl, int val);
 
-      virtual int getControllerInfo(int, const char**, int*, int*, int*) const;
+      virtual int getControllerInfo(int, const char**, int*, int*, int*, int*) const;
       virtual void getInitData(int*, const unsigned char**) const;
 
       virtual bool guiVisible() const;
