@@ -597,14 +597,17 @@ bool SimpleSynth::sysex(int /*len*/, const unsigned char* data)
     \fn SimpleSynth::getPatchName
     \brief Called from host to get names of patches
     \param index - which patchnr we're about to deliver
+    \param drum - is it a drum track?
     \return const char* with patchname
  */
 //---------------------------------------------------------
-const char* SimpleSynth::getPatchName(int /*index*/, int, int) const
+const char* SimpleSynth::getPatchName(int /*index*/, int, int, bool /*drum*/) const
       {
       SS_TRACE_IN
       SS_TRACE_OUT
-      return 0;
+      //return 0;
+      //return "<unknown>";
+      return "SimpleSynth";
       }
 
 //---------------------------------------------------------
@@ -638,7 +641,7 @@ const MidiPatch* SimpleSynth::getPatchInfo(int index, const MidiPatch* patch) co
     \return 0 when done, otherwise return next desired controller index
  */
 //---------------------------------------------------------
-int SimpleSynth::getControllerInfo(int index, const char** name, int* controller, int* min, int* max) const
+int SimpleSynth::getControllerInfo(int index, const char** name, int* controller, int* min, int* max, int* /*initval*/ ) const
       {
       SS_TRACE_IN
       if (index >= SS_NR_OF_CONTROLLERS) {
@@ -658,11 +661,42 @@ int SimpleSynth::getControllerInfo(int index, const char** name, int* controller
       return (index +1);
       }
 
+//---------------------------------------------------------
+//   processMessages
+/*!
+    \fn SimpleSynth::processMessages
+    \brief Called from host always, even if output path is unconnected
+ */
+//---------------------------------------------------------
+void SimpleSynth::processMessages()
+{
+  //Process messages from the gui
+  while (gui->fifoSize()) 
+  {
+    MidiPlayEvent ev = gui->readEvent();
+    if (ev.type() == ME_SYSEX) 
+    {
+      sysex(ev.len(), ev.data());
+      sendEvent(ev);
+    }
+    else if (ev.type() == ME_CONTROLLER) 
+    {
+      setController(ev.channel(), ev.dataA(), ev.dataB(), true);
+      sendEvent(ev);
+    }
+    else 
+    {
+      if(SS_DEBUG)
+        printf("SimpleSynth::process(): unknown event, type: %d\n", ev.type());
+    }
+  }
+}
+  
 //---------------------------------------------------------
 //   process
 /*!
     \fn SimpleSynth::process
-    \brief Realtime function where the processing actually occurs
+    \brief Realtime function where the processing actually occurs. Called from host, ONLY if output path is connected.
     \param channels - audio data
     \param offset - sample offset
     \param len - nr of samples to process
@@ -670,6 +704,7 @@ int SimpleSynth::getControllerInfo(int index, const char** name, int* controller
 //---------------------------------------------------------
 void SimpleSynth::process(float** out, int offset, int len)
       {
+      /*
       //Process messages from the gui
       while (gui->fifoSize()) {
             MidiPlayEvent ev = gui->readEvent();
@@ -686,7 +721,8 @@ void SimpleSynth::process(float** out, int offset, int len)
                         printf("SimpleSynth::process(): unknown event, type: %d\n", ev.type());
                   }
             }
-
+      */
+      
       if (synth_state == SS_RUNNING) {
 
       //Temporary mix-doubles
@@ -1040,10 +1076,10 @@ void SimpleSynth::parseInitData(const unsigned char* data)
 
                if (SS_DEBUG_INIT) {
                      printf("Channel %d:\n", ch);
-                     printf("buffer[%d] - channels[ch].volume_ctrlval = \t%d\n", ptr-data, *ptr);
-                     printf("buffer[%d] - channels[ch].pan = \t\t%d\n", ptr-data+1, *(ptr+1));
-                     printf("buffer[%d] - channels[ch].noteoff_ignore = \t%d\n", ptr-data+2, *(ptr+2));
-                     printf("buffer[%d] - channels[ch].channel_on = \t%d\n", ptr-data+3, *(ptr+3));
+                     printf("buffer[%zd] - channels[ch].volume_ctrlval = \t%d\n", ptr-data, *ptr);
+                     printf("buffer[%zd] - channels[ch].pan = \t\t%d\n", ptr-data+1, *(ptr+1));
+                     printf("buffer[%zd] - channels[ch].noteoff_ignore = \t%d\n", ptr-data+2, *(ptr+2));
+                     printf("buffer[%zd] - channels[ch].channel_on = \t%d\n", ptr-data+3, *(ptr+3));
                      }
                updateVolume(ch, *(ptr));
                guiUpdateVolume(ch, *(ptr));
@@ -1117,7 +1153,7 @@ void SimpleSynth::parseInitData(const unsigned char* data)
 
       for (int i=0; i<SS_NR_OF_SENDEFFECTS; i++) {
             if (SS_DEBUG_INIT)
-                  printf("buffer[%d] - sendeffect[%d], labelnamelen=%d\n", ptr-data, i, *ptr);
+                  printf("buffer[%zd] - sendeffect[%d], labelnamelen=%d\n", ptr-data, i, *ptr);
             int labelnamelen = *(ptr);
 
             if (labelnamelen != SS_NO_PLUGIN) {
@@ -1148,7 +1184,7 @@ void SimpleSynth::parseInitData(const unsigned char* data)
 
                   for (int j=0; j<params; j++) {
                         if (SS_DEBUG_INIT)
-                              printf("buffer[%d] - sendeffect[%d], parameter[%d]=%d\n", ptr-data, i, j, *ptr);
+                              printf("buffer[%zd] - sendeffect[%d], parameter[%d]=%d\n", ptr-data, i, j, *ptr);
                         setFxParameter(i, j, sendEffects[i].plugin->convertGuiControlValue(j, *(ptr)));
                         ptr++;
                         }
@@ -1270,7 +1306,7 @@ static void* loadSampleThread(void* p)
             smp->samples = smp->frames * smp->channels;
 
             if (SS_DEBUG) {
-                  printf("Resampling from %ld frames to %ld frames - srcration: %lf\n", sfi.frames, smp->frames, srcratio);
+                  printf("Resampling from %ld frames to %ld frames - srcration: %lf\n", (long) sfi.frames, smp->frames, srcratio);
                   printf("Nr of new samples: %ld\n", smp->samples);
                   }
 
@@ -1326,9 +1362,11 @@ static void* loadSampleThread(void* p)
       pthread_exit(0);
       }
 
+QString *projPathPtr;
 
-static Mess* instantiate(int sr, QWidget*, const char* name)
+static Mess* instantiate(int sr, QWidget*, QString* projectPathPtr, const char* name)
       {
+      projPathPtr = projectPathPtr;
       printf("SimpleSynth sampleRate %d\n", sr);
       SimpleSynth* synth = new SimpleSynth(sr);
       if (!synth->init(name)) {
@@ -1488,7 +1526,7 @@ extern "C"
       {
       static MESS descriptor = {
             "SimpleSynth",
-            "Mathias Lundgren (lunar_shuttle@users.sf.net)",
+            "SimpleSynth drums by Mathias Lundgren", // (lunar_shuttle@users.sf.net)
             "0.1",      //Version string
             MESS_MAJOR_VERSION, MESS_MINOR_VERSION,
             instantiate,
